c ************************************************************************
c     ratoms
c ************************************************************************
c  
c
c     subroutine to read in atom labels and possible the masses.  If the
c     masses are not present then they will be looked up in a table of
c     the most common isoptopes (ams).
C  
C     created : April 27, 2007 by E. E. Dahlke (taken from polyrate9.6)
C     revised :
C     purpose : Assign the masses to each atom in the input file from the 
C	        list of most common isotopes (ams)

      subroutine ratoms(natms,asym,amass)
c
      implicit none
      include 'common.inc'
      integer i, j
      integer natms, itry
      character * 2 asymb(103), tmp2
      character*1 xlett
      character*80 tmp
      double precision ams(103)
      double precision amass(maxat)



c
c     array of atomic masses
c
      data (ams(i),i=1,54)  /
     *   1.007825d+00,4.0026d+00,7.01600d+00,9.01218d+00,11.00931d+00,
     *   12.0d+00,14.00307d+00,15.99491d+00,18.99840d+00,19.99244d+00,
     *   22.9898d+00,23.98504d+00,26.98153d+00,27.97693d+00,
     *   30.97376d+00,31.97207d+00,34.96885d+00,39.948d+00,
     *   38.96371d+00,39.96259d+00,44.95592d+00,47.90d+00,50.9440d+00,
     *   51.9405d+00,54.9381d+00,55.9349d+00,58.9332d+00,57.9353d+00,
     *   62.9298d+00,63.9291d+00,68.9257d+00,73.9219d+00,74.9216d+00,
     *   79.9165d+00,78.9183d+00,83.9115d+00,
     *   84.9117d+00,87.9056d+00,89.9054d+00,89.9043d+00,92.9060d+00,
     *   97.9055d+00,97.0d+00,101.9037d+00,102.9048d+00,105.9032d+00,
     *   106.9041d+00,113.9036d+00,114.9041d+00,119.9022d+00,
     *   120.9038d+00,129.9067d+00,126.9044d+00,131.9042d+00/
      data (ams(i),i=55,103)  /
     *   132.9054d+00,137.9052d+00,138.9063d+00,139.9054d+00,
     *   140.9076d+00,141.9077d+00,144.9127d+00,151.9197d+00,
     *   152.9212d+00,157.9241d+00,158.9253d+00,163.9292d+00,
     *   164.9303d+00,165.9303d+00,168.9342d+00,173.9389d+00,
     *   174.9408d+00,179.9465d+00,180.9480d+00,183.9509d+00,
     *   186.9557d+00,191.9615d+00,192.9629d+00,194.9648d+00,
     *   196.9665d+00,201.9706d+00,
     *   204.9744d+00,207.9766d+00,208.9804d+00,208.9824d+00,
     *   209.9871d+00,222.0176d+00,
     *   223.0197d+00,226.0254d+00,
     *   227.0278d+00,232.0381d+00,231.0359d+00,238.0508d+00,
     *   237.0482d+00,244.0642d+00,243.0614d+00,247.0703d+00,
     *   247.0703d+00,251.0796d+00,252.0829d+00,257.0751d+00,
     *   258.0986d+00,259.1009d+00,260.1053d+00/
c
c     array of atomic symbols
c
      data (asymb(i),i=1,103)  /
     *   'h ','he','li','be','b ','c ','n ','o ','f ','ne',
     *   'na','mg','al','si','p ','s ','cl','ar',
     *   'k ','ca','sc','ti','v ','cr','mn','fe','co','ni','cu',
     *        'zn','ga','ge','as','se','br','kr',
     *   'rb','sr','y ','zr','nb','mo','tc','ru','rh','pd','ag',
     *        'cd','in','sn','sb','te','i ','xe',
     *   'cs','ba','la','ce','pr','nd','pm','sm','eu','gd','tb',
     *        'dy','ho','er','tm','yb','lu','hf','ta','w ','re',
     *        'os','ir','pt','au','hg','tl','pb','bi','po','at','rn',
     *   'fr','ra','ac','th','pa','u','np','pu','am','cm','bk',
     *        'cf','es','fm','md','no','lr'/

C  go through the array asym, which hold the atomic symbols (and possibly 
C  identification numbers if different atoms of the same type have different charges.
c  will need to change the first letter in asym back to lower case
 
      do i = 1,natms
         tmp(1:8) = asym(i)
         xlett = tmp(1:1)
         itry = ichar(xlett)
         if (xlett .ge. 'A' .and. xlett .le. 'Z') then
           itry = itry + 32
           tmp(1:1) = char(itry)
         end if    
         do j = 1, 103
	    tmp2 = asymb(j)
            if (tmp(2:2) .ge. '0' .and. tmp(2:2) .le. '9') then
		if (tmp(1:1).eq.tmp2(1:1) .and. tmp2(2:2).eq.' ')then
                   amass(i) = ams(j)
                 end if
            else 
                if (tmp(1:2) .eq. tmp2(1:2)) then
                   amass(i) = ams(j)
                end if
	    end if
         end do
      end do


      return


      end
C***********************************************************************
C     EXPND
C***********************************************************************
C
      SUBROUTINE expnd(A,B,N,ISYM)
C
C     CALLED BY:
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION A(*),B(N,*)
      PARAMETER (ZERO=0.0D+00)
C
C     EXPAND TRIANGULAR MATRIX A TO SQUARE MATRIX B
C     B IS SYMMETRIC IF ISYM=0, ANTISYMMETRIC OTHERWISE
C
      IJ = 0
      IF (ISYM .NE. 0) THEN
         DO 200 J = 1,N
            DO 100 I = 1,J
               IJ = IJ + 1
               B(I,J) = A(IJ)
               B(J,I) = -A(IJ)
  100       CONTINUE
            B(J,J) = ZERO
  200    CONTINUE
      ELSE
         DO 300 J = 1,N
         DO 300 I = 1,J
            IJ = IJ + 1
            B(I,J) = A(IJ)
            B(J,I) = A(IJ)
  300    CONTINUE
      ENDIF
      RETURN
      END
C
c************************************************************************
c     Subroutine freqcal 
c************************************************************************
c 
      subroutine freqcal(ageom,amass,natms,hess)
c  
c     Created:  May 10th by E. E. Dahlke
c               based on Yan Zhao's multilevel version
c     Purpose:  Normal mode analysis is performed in this subroutine and 
c     frequencies and normal mode coordinate are calculated from the hessian
c      matrix.
c 

      implicit none
      include 'common.inc'
c
      integer  i, j, k, n3tm, ij, m, ii, ll, mm, lstr, lend
      integer  kstr,kend, natms, na, l 
      double precision temp
      double precision hessms(3*maxat,3*maxat), u(3*maxat,3*maxat)
      double precision uu(3*maxat,3*maxat)
      double precision hessc(maxhess), massvec(3*maxat),nn(3*maxat)
      double precision eigval(3*maxat), uc(maxhess), freq(3*maxat) 
      double precision hessmsa(3*maxat,3*maxat), ua(3*maxat,3*maxat)
      double precision eigvala(3*maxat), uca(maxhess)
      double precision hessca(maxhess),freqa(3*maxat)
      double precision amass(maxat), ageom(3*maxat)
      double precision hess(3*maxat,3*maxat)
      
      character*1, xyz(3,3*maxat)
      character*1, ima(3*maxat), imaa(3*maxat)
      integer ind(3*maxat)
c
      n3tm = 3*natms
c     
c    
c
c     get the mass vector 
c
      ij = 1
      do i=1, natms
         do j = 1, 3
         massvec(ij)=amass(i)
         ij = ij + 1
         end do
      enddo
c     
c     calculate the mass scaled hessian  
c
      do i=1, n3tm
        do j=1, i
          hessms(i,j)=hess(i,j)/(sqrt(massvec(i)*massvec(j)))
          hessms(j,i)=hessms(i,j)
          hessmsa(i,j)=hessms(i,j)
          hessmsa(j,i)=hessms(j,i)
        enddo
      enddo
      call prjfc(hessms,ageom,n3tm,amass) 
c
c      diagonalize  projected and unprojected hessian 
c 
      ij = 0    
      do i = 1, n3tm
        do j = 1, i
          ij = ij+1
          hessc(ij) = hessms(i,j)  
          hessca(ij) = hessmsa(i,j)
        enddo
      enddo
      call rsp(hessc,n3tm,n3tm,eigval,uc) 
      call rsp(hessca, n3tm, n3tm, eigvala, uca)
      ij = 0
      do i = 1, n3tm
        do j = 1, n3tm
          ij = ij+1
          u(j,i) = uc(ij)
          ua(j,i)= uca(ij)
        enddo
       enddo
c
c    calculate the frequencies from the projected Hessian
c
      do i=1, n3tm 
       if (eigval(i) .ge. 0.0D0) then
        ima(i) = " "
        freq(i) = sqrt( eigval(i)*26422760.0)
       else
        ima(i) = "i"
        freq(i) = sqrt(-1*eigval(i)*26422760.0)
       endif
      enddo
c
c    calculate the frequencies from the unprojected Hessisan
c
      do i=1, n3tm
       if (eigvala(i) .ge. 0.0D0) then
        imaa(i) = " "
        freqa(i) = sqrt( eigvala(i)*26422760.0)
       else
        imaa(i) = "i"
        freqa(i) = sqrt(-1*eigvala(i)*26422760.0)
       endif
      enddo

c
c      calculate the normal mode coordinates in cartesian coordinates
c
      do i = 1, n3tm
        temp = 0
       do j = 1, n3tm
        if (freq(i) .le. 5.0) then 
          u(j,i)= ua(j,i)
        endif
        uu(j,i) = u(j,i)/sqrt(massvec(j))
        temp = temp + uu(j,i)* uu(j,i)
       enddo
        nn(i) = sqrt(temp) 
      enddo        
      do i = 1, n3tm
       do j = 1, n3tm
        uu(j,i) = uu(j,i)/nn(i)
       enddo
      enddo
c
c      write out the frequencies and normal mode
c

      do i=1, n3tm
       ind(i) = i
       xyz(1,i) = 'X'
       xyz(2,i) = 'Y'
       xyz(3,i) = 'Z'
      enddo
      write(2,3300)
      do m = n3tm, 1, -2
       lstr = m
       lend = m-1
       if (lend .le. 1) lend = 1 
       write(2,4000) (n3tm+1-ind(ii), ii=lstr,lend,-1)
       write(2,4100) (freq(ii),ima(ii),ii=lstr,lend,-1)
       write(2,4200) ((xyz(mm,ll),mm=1,3),ll=lstr,lend,-1)
       na = 1
       do i = 1, n3tm, 3
         kstr = i
         kend = i+2
         write(2,4300) na,((u(k,l), k=kstr,kend),l=lstr,lend,-1) 
         na = na + 1
       enddo
      enddo

      write(2,1500)

      write(2,3400)
      do m = n3tm, 1, -2
        lstr = m
        lend = m-1
        if (lend .le. 1) lend = 1
        write(2,4000) (n3tm+1-ind(ii), ii=lstr,lend,-1)
        write(2,4100) (freq(ii),ima(ii),ii=lstr,lend,-1)
        write(2,4200) ((xyz(mm,ll),mm=1,3),ll=lstr,lend,-1)
        na = 1
        do i = 1, n3tm, 3
         kstr = i
         kend = i+2
         write(2,4300) na,((uu(k,l), k=kstr,kend),l=lstr,lend,-1)
         na = na + 1
        enddo
       enddo

c      write(2,3500)
c      do m = n3tm, 1, -2
c        lstr = m
c        lend = m-1
c        if (lend .le. 1) lend = 1
c        write(2,4000) (n3tm+1-ind(ii), ii=lstr,lend,-1)
c        write(2,4100) (freqa(ii),imaa(ii),ii=lstr,lend,-1)
c        write(2,4200) ((xyz(mm,ll),mm=1,3),ll=lstr,lend,-1)
c        na = 1
c        do i = 1, n3tm, 3
c         kstr = i
c         ckend = i+2
c         write(2,4300) na,((ua(k,l), k=kstr,kend),l=lstr,lend,-1)
c         na = na + 1
c        enddo
c       enddo

 1500 FORMAT(/,80('_'))
 3300 FORMAT(/1X,'Frequencies and normalized eigenvector components ',
     *       /1X,'in mass-scaled space-fixed cartesian coordinates')
 3400 FORMAT(//1X,'Frequencies and normalized eigenvector components ',
     *       /1X,'in cartesian coordinates')
 3500 FORMAT(//1X,'Frequencies and normalized eigenvector components',
     *  /1X,'in mass-scaled space-fixed cartesian coordinates(unprj)')

 4000  FORMAT(/,1X,4HMode,16X,I3,27X,I3)
 4100  FORMAT(1X,'omega (cm**-1)',2X,F9.2,1X,A1,(19X,F9.2,1X,A1))
 4200  FORMAT(/,1X,4HAtom,2(3(8X,A1),3X))
 4300  FORMAT(1X,I3,5X,2(3(F7.3,2X),3X))
 2001  Format(t20,d15.8,t40,d15.8,t60,d15.8) 
       return
      end

c*******************************************************************
c      subroutine prjfc(f,xparam,nvar,atmass) 
c*******************************************************************
c
      subroutine prjfc(f,xparam,nvar,atmass) 
c                                                                       
c  calculates projected force constant matrix (f).                      
c  this routine came originally from polyrate. it is used by permission
c  of d. truhlar. the current version is lifted from gamess and 
c  adapted by f.jensen, odense, dk             
c  if we are at a stationary point (stpt=.t.), i.e. gnorm .lt. 10,      
c  then the rotational and translational modes are projected out        
c  and their frequencies become identical zero. if not at a stationary  
c  point then the mass-weighted gradient is also projected out and      
c  the corresponding frequency become zero.                             
c ************************************************                      
c   x : mass-weighted coordinate                                        
c   dx: normalized mass-weighted gradient vector                        
c   f : mass-weighted force constant matrix                             
c   rm: inversion of square root of mass                                
c   p, cof: buffer                                                      
c                                                                       
     
      implicit none
      include 'common.inc' 
      integer iscr(6), natm, nvar, nc1, info, indx, jndx, jend
      integer ii, jj, k
      double precision X(3*maxat),RM(3*maxat),F(3*maxat,3*maxat)   
      double precision P(3*maxat,3*maxat),COF(3*maxat,3*maxat)
      double precision atmass(3*maxat/3) 
      double precision TENS(3,3,3),ROT(3,3),SCR(3,3),CMASS(3)  
      double precision coord(3,maxat),dx(3*maxat),xparam(3*maxat)
      double precision zero, one, eps, cut5, cut8, tmp
      double precision totm, chk, det, trp, sum

      PARAMETER (ZERO=0.0d+00, ONE=1.0d+00, EPS=1.0d-14,
     *           CUT5=1.0d-05, CUT8=1.0d-08)                            
      integer l, ij, i, j, ia, ib, ic, ja, jb, jc, ip, jp
C                                                                       
C TOTALLY ASYMMETRIC CARTESIAN TENSOR.                                  
      DATA TENS/ 0.0d+00,  0.0d+00,  0.0d+00,                           
     X           0.0d+00,  0.0d+00, -1.0d+00,                           
     X           0.0d+00,  1.0d+00,  0.0d+00,                           
     Y           0.0d+00,  0.0d+00,  1.0d+00,                           
     Y           0.0d+00,  0.0d+00,  0.0d+00,                           
     Y          -1.0d+00,  0.0d+00,  0.0d+00,                           
     Z           0.0d+00, -1.0d+00,  0.0d+00,                           
     Z           1.0d+00,  0.0d+00,  0.0d+00,                           
     Z           0.0d+00,  0.0d+00,  0.0d+00  /                         
C                                                                       
      natm=nvar/3                                                        
      nc1=nvar
      ij=1
      do 2 i=1,natm                                                    
         coord(1,i)=xparam(ij)
         coord(2,i)=xparam(ij+1)
         coord(3,i)=xparam(ij+2)
         ij=ij+3
2     continue                                                          
C     CALCULATE 1/SQRT(MASS)                                            
      L=0                                                               
      DO 3  I=1, natm 
         TMP=ONE/SQRT(ATMASS(I))                                        
         DO 3 J=1,3                                                     
            L=L+1                                                       
           RM(L)=TMP
3     continue

C     PREPARE GRADIENT                                                  
         DO 4 I=1,nvar                                                   
4        DX(I)=ZERO                                                     
C     FIND CMS AND CALCULATED MASS WEIGHTED COORDINATES                 
      totm=zero                                                         
      cmass(1)=zero                                                     
      cmass(2)=zero                                                     
      cmass(3)=zero                                                     
      DO 6 I=1,NATM                                                     
         TOTM=TOTM+ATMASS(I)                                            
         DO 6 J=1,3                                                     
            CMASS(J)=CMASS(J)+ATMASS(I)*COORD(J,I)                      
6     CONTINUE                                                          
      DO 7 J=1,3                                                        
7     CMASS(J)=CMASS(J)/TOTM       
      L=0                                                               
      DO 8 I=1,NATM                                                     
         DO 8 J=1,3                                                     
         TMP=SQRT(ATMASS(I))                                            
             L=L+1                                                      
             X(L)=TMP*(COORD(J,I)-CMASS(J))
8     CONTINUE 
C
C 2. COMPUTE INERTIA TENSOR.          
C                                  
      DO 10 I=1,3                                                       
       DO 10 J=1,3                                                      
   10   ROT(I,J)=ZERO                                                   
      DO 20 I=1,NATM                                                    
       L=3*(I-1)+1                                                      
       ROT(1,1)=ROT(1,1)+X(L+1)**2+X(L+2)**2    
       ROT(1,2)=ROT(1,2)-X(L)*X(L+1)       
       ROT(1,3)=ROT(1,3)-X(L)*X(L+2)  
       ROT(2,2)=ROT(2,2)+X(L)**2+X(L+2)**2  
       ROT(2,3)=ROT(2,3)-X(L+1)*X(L+2)      
   20  ROT(3,3)=ROT(3,3)+X(L)**2+X(L+1)**2    
      ROT(2,1)=ROT(1,2)       
      ROT(3,1)=ROT(1,3)        
      ROT(3,2)=ROT(2,3)   
C                                                                       
CHECK THE INERTIA TENSOR.                                               
      CHK=ROT(1,1)*ROT(2,2)*ROT(3,3)                                    
      IF(ABS(CHK).GT.CUT8) GO TO 21                                     


      IF(ABS(ROT(1,1)).GT.CUT8) GO TO 11                                
C X=0                                                                   
      IF(ABS(ROT(2,2)).GT.CUT8) GO TO 12                                
C X,Y=0                                                                 
      IF(ABS(ROT(3,3)).GT.CUT8) GO TO 13                                
      WRITE(6,14) ROT(1,1),ROT(2,2),ROT(3,3)                            
   14 FORMAT(1X,'EVERY DIAGONAL ELEMENTS ARE ZERO ?',3F20.10)           
      RETURN                                                            
C                                                                       
C* 1. X,Y=0 BUT Z.NE.0                                                  
   13 ROT(3,3)=ONE/ROT(3,3)                                             
      GO TO 22                                                          
C Y.NE.0                                                                
   12 IF(ABS(ROT(3,3)).GT.CUT8) GO TO 15                                

C* 2. X,Z=0 BUT Y.NE.0                                                  
      ROT(2,2)=ONE/ROT(2,2)                                             
      GO TO 22                                                          
C X.NE.0                                                                
   11 IF(ABS(ROT(2,2)).GT.CUT8) GO TO 16                                
      IF(ABS(ROT(3,3)).GT.CUT8) GO TO 17                                

C* 3. Y,Z=0 BUT X.NE.0                                                  
      ROT(1,1)=ONE/ROT(1,1)                                             
      GO TO 22                                                          

C* 4. X,Y.NE.0 BUT Z=0                                                  
   16 DET=ROT(1,1)*ROT(2,2)-ROT(1,2)*ROT(2,1)                           
      TRP=ROT(1,1)                                                      
      ROT(1,1)=ROT(2,2)/DET                                             
      ROT(2,2)=TRP/DET                                                  
      ROT(1,2)=-ROT(1,2)/DET                                            
      ROT(2,1)=-ROT(2,1)/DET                                            
      GO TO 22                                                          

C* 5. X,Z.NE.0 BUT Y=0                                                  
   17 DET=ROT(1,1)*ROT(3,3)-ROT(1,3)*ROT(3,1)                           
      TRP=ROT(1,1)                                                      
      ROT(1,1)=ROT(3,3)/DET                                             
      ROT(3,3)=TRP/DET                                                  
      ROT(1,3)=-ROT(1,3)/DET                                            
      ROT(3,1)=-ROT(3,1)/DET                                            
      GO TO 22                                                          

C* 6. Y,Z.NE.0 BUT X=0                                                  
   15 DET=ROT(3,3)*ROT(2,2)-ROT(3,2)*ROT(2,3)                           
      TRP=ROT(3,3)                                                      
      ROT(3,3)=ROT(2,2)/DET                                             
      ROT(2,2)=TRP/DET                                                  
      ROT(3,2)=-ROT(3,2)/DET                                            
      ROT(2,3)=-ROT(2,3)/DET                                            
      GO TO 22                                                          
   21 CONTINUE                                                          

C                                                                       
C 4. COMPUTE INVERSION MATRIX OF ROT.                                   
C
      INFO=0                                                            
      CALL DGEFA(ROT,3,3,ISCR,INFO)                                     
      IF(INFO.NE.0) STOP                                                
      DET=ZERO                                                          
      CALL DGEDI(ROT,3,3,ISCR,DET,SCR,1)                                
C                                                                       
   22 CONTINUE                                                          
C                                                                       
C 5. TOTAL MASS ---> TOTM.                                              
C                                                                       
C 6. COMPUTE P MATRIX                                                   
C    ----------------                                                   
      DO 100 IP=1,NATM                                                  
       INDX=3*(IP-1)                                                    
       DO 100 JP=1,IP                                                   
        JNDX=3*(JP-1)                                                   
        DO 70 IC=1,3                                                    
         JEND=3                                                         
         IF(JP.EQ.IP) JEND=IC                                           
         DO 70 JC=1,JEND                                                
          SUM=ZERO                                                      
          DO 50 IA=1,3                                                  
           DO 50 IB=1,3                                                 
            IF(TENS(IA,IB,IC).EQ.0) GO TO 50                            
            DO 30 JA=1,3                                                
             DO 30 JB=1,3                                               
              IF(TENS(JA,JB,JC).EQ.0) GO TO 30                          
              SUM=SUM+TENS(IA,IB,IC)*TENS(JA,JB,JC)*ROT(IA,JA)*         
     &                X(INDX+IB)*X(JNDX+JB)                             
   30         CONTINUE                                                  
   50       CONTINUE                                                    
          II=INDX+IC                                                    
          JJ=JNDX+JC                                                    
          P(II,JJ)=SUM+DX(II)*DX(JJ)                                    
          IF(IC.EQ.JC) P(II,JJ)=P(II,JJ)+ONE/(RM(II)*RM(JJ)*TOTM)       
   70     CONTINUE                                                      
  100   CONTINUE 
C                                                                       
C 7. COMPUTE DELTA(I,J)-P(I,J)                                          
      DO 110 I=1,NC1                                                    
       DO 110 J=1,I                                                     
        P(I,J)=-P(I,J)                                                  
        IF(I.EQ.J) P(I,J) = ONE +P(I,J)                                 
  110   CONTINUE                                                        
C                                                                       
C 8. NEGLECT SMALLER VALUES THAN 10**-8.                
C                
      DO 120 I=1,NC1                                                    
       DO 120 J=1,I                                                     
        IF(ABS(P(I,J)).LT.CUT8) P(I,J)=ZERO                             
        P(J,I)=P(I,J)                                                   
  120   CONTINUE              
                                          
C                                                                       
C 10. POST AND PREMULTIPLY F BY P.                                      
C     USE COF FOR SCRATCH.  
      DO 150 I=1,NC1                                                    
       DO 150 J=1,NC1                                                   
        SUM=ZERO                                                        
        DO 140 K=1,NC1                                                  
  140    SUM=SUM+F(I,K)*P(K,J)                                          
  150   COF(I,J)=SUM
C                                                                       
C 11. COMPUTE P*F*P.
C                                                    
      DO 200 I=1,NC1                                                    
       DO 200 J=1,NC1                                                   
        SUM=ZERO                                                        
        DO 190 K=1,NC1                                                  
  190    SUM=SUM+P(I,K)*COF(K,J)                                        
  200   F(I,J)=SUM                                                      
2000  Format(t20,d15.8,t40,d15.8,t60,d15.8)
 
      RETURN                                                            
      END            

c******************************************************************
c      subroutine dgefa(a,lda,n,ipvt,info)                               
c*****************************************************************
c
      subroutine dgefa(a,lda,n,ipvt,info)                               
      implicit double precision(a-h,o-z)
      dimension a(lda,*),ipvt(*)                                        
c                                                                       
c     dgefa factors a double precision matrix by gaussian elimination.  
c                                                                       
c     dgefa is usually called by dgeco, but it can be called            
c     directly with a saving in time if  rcond  is not needed.          
c     (time for dgeco) = (1 + 9/n)*(time for dgefa) .                   
c                                                                       
c     on entry                                                          
c                                                                       
c        a       double precision(lda, n)                               
c                the matrix to be factored.                             
c                                                                       
c        lda     integer                                                
c                the leading dimension of the array  a .                
c                                                                       
c        n       integer                                                
c                the order of the matrix  a .                           
c                                                                       
c     on return                                                         
c                                                                       
c        a       an upper triangular matrix and the multipliers         
c                which were used to obtain it.                          
c                the factorization can be written  a = l*u  where       
c                l  is a product of permutation and unit lower          
c                triangular matrices and  u  is upper triangular.       
c                                                                       
c        ipvt    integer(n)                                             
c                an integer vector of pivot indices.                    
c                                                                       
c        info    integer                                                
c                = 0  normal value.                                     
c                = k  if  u(k,k) .eq. 0.0 .  this is not an error       
c                     condition for this subroutine, but it does        
c                     indicate that dgesl or dgedi will divide by zero  
c                     if called.  use  rcond  in dgeco for a reliable   
c                     indication of singularity.                        
c                                                                       
c     linpack. this version dated 08/14/78 .                            
c     cleve moler, university of new mexico, argonne national lab.      
c                                                                       
c     subroutines and functions                                         
c                                                                       
c     blas daxpy,dscal,idamax                                           
c                                                                       
c     gaussian elimination with partial pivoting                        
c                                                                       
      INFO = 0                                                          
      NM1 = N - 1                                                       
      IF (NM1 .LT. 1) GO TO 70                                          
      DO 60 K = 1, NM1                                                  
         KP1 = K + 1                                                    
C                                                                       
C        FIND L = PIVOT INDEX                                           
C                                                                       
         L = IDAMAX(N-K+1,A(K,K),1) + K - 1                             
         IPVT(K) = L                                                    
C                                                                       
C        ZERO PIVOT IMPLIES THIS COLUMN ALREADY TRIANGULARIZED          
C                                                                       
         IF (A(L,K) .EQ. 0.0D+00) GO TO 40                              
C                                                                       
C           INTERCHANGE IF NECESSARY                                    
C                                                                       
            IF (L .EQ. K) GO TO 10                                      
               T = A(L,K)                                               
               A(L,K) = A(K,K)                                          
               A(K,K) = T                                               
   10       CONTINUE                                                    
C                                                                       
C           COMPUTE MULTIPLIERS                                         
C                                                                       
            T = -1.0D+00/A(K,K)                                         
            CALL DSCAL(N-K,T,A(K+1,K),1)                                
C                                                                       
C           ROW ELIMINATION WITH COLUMN INDEXING                        
C                                                                       
            DO 30 J = KP1, N                                            
               T = A(L,J)                                               
               IF (L .EQ. K) GO TO 20                                   
                  A(L,J) = A(K,J)                                       
                  A(K,J) = T                                            
   20          CONTINUE                                                 
               CALL DAXPY(N-K,T,A(K+1,K),1,A(K+1,J),1)                  
   30       CONTINUE                                                    
         GO TO 50                                                       
   40    CONTINUE                                                       
            INFO = K                                                    
   50    CONTINUE                                                       
   60 CONTINUE                                                          
   70 CONTINUE                                                          
      IPVT(N) = N                                                       
      IF (A(N,N) .EQ. 0.0D+00) INFO = N                                 
      RETURN                                                            
      END           

c***************************************************************
c      subroutine dgedi(a,lda,n,ipvt,det,work,job)               
c***************************************************************
c      
      subroutine dgedi(a,lda,n,ipvt,det,work,job)                       
      implicit double precision(a-h,o-z)
      dimension a(lda,*),det(2),work(*),ipvt(*)                         
c                                                                       
c     dgedi computes the determinant and inverse of a matrix            
c     using the factors computed by dgeco or dgefa.                     
c                                                                       
c     on entry                                                          
c                                                                       
c        a       double precision(lda, n)                               
c                the output from dgeco or dgefa.                        
c                                                                       
c        lda     integer                                                
c                the leading dimension of the array  a .                
c                                                                       
c        n       integer                                                
c                the order of the matrix  a .                           
c                                                                       
c        ipvt    integer(n)                                             
c                the pivot vector from dgeco or dgefa.                  
c                                                                       
c        work    double precision(n)                                    
c                work vector.  contents destroyed.                      
c                                                                       
c        job     integer                                                
c                = 11   both determinant and inverse.                   
c                = 01   inverse only.                                   
c                = 10   determinant only.                               
c                                                                       
c     on return                                                         
c                                                                       
c        a       inverse of original matrix if requested.               
c                otherwise unchanged.                                   
c                                                                       
c        det     double precision(2)                                    
c                determinant of original matrix if requested.           
c                otherwise not referenced.                              
c                determinant = det(1) * 10.0**det(2)                    
c                with  1.0 .le. abs(det(1)) .lt. 10.0                   
c                or  det(1) .eq. 0.0 .                                  
c                                                                       
c     error condition                                                   
c                                                                       
c        a division by zero will occur if the input factor contains     
c        a zero on the diagonal and the inverse is requested.           
c        it will not occur if the subroutines are called correctly      
c        and if dgeco has set rcond .gt. 0.0 or dgefa has set           
c        info .eq. 0 .                                                  
c                                                                       
c     linpack. this version dated 08/14/78 .                            
c     cleve moler, university of new mexico, argonne national lab.      
c                                                                       
c     subroutines and functions                                         
c                                                                       
c     blas daxpy,dscal,dswap                                            
c     fortran abs,mod                                                   
c                                                                       
c     compute determinant                                               
c                                                                       
      IF (JOB/10 .EQ. 0) GO TO 70                                       
         DET(1) = 1.0D+00                                               
         DET(2) = 0.0D+00                                               
         TEN = 10.0D+00                                                 
         DO 50 I = 1, N                                                 
            IF (IPVT(I) .NE. I) DET(1) = -DET(1)                        
            DET(1) = A(I,I)*DET(1)                                      
C        ...EXIT                                                        
            IF (DET(1) .EQ. 0.0D+00) GO TO 60                           
   10       IF (ABS(DET(1)) .GE. 1.0D+00) GO TO 20                      
               DET(1) = TEN*DET(1)                                      
               DET(2) = DET(2) - 1.0D+00                                
            GO TO 10                                                    
   20       CONTINUE                                                    
   30       IF (ABS(DET(1)) .LT. TEN) GO TO 40                          
               DET(1) = DET(1)/TEN                                      
               DET(2) = DET(2) + 1.0D+00                                
            GO TO 30                                                    
   40       CONTINUE                                                    
   50    CONTINUE                                                       
   60    CONTINUE                                                       
   70 CONTINUE                                                          
C                                                                       
C     COMPUTE INVERSE(U)                                                
C                                                                       
      IF (MOD(JOB,10) .EQ. 0) GO TO 150                                 
         DO 100 K = 1, N                                                
            A(K,K) = 1.0D+00/A(K,K)                                     
            T = -A(K,K)                                                 
            CALL DSCAL(K-1,T,A(1,K),1)                                  
            KP1 = K + 1                                                 
            IF (N .LT. KP1) GO TO 90                                    
            DO 80 J = KP1, N                                            
               T = A(K,J)                                               
               A(K,J) = 0.0D+00                                         
               CALL DAXPY(K,T,A(1,K),1,A(1,J),1)                        
   80       CONTINUE                                                    
   90       CONTINUE                                                    
  100    CONTINUE                                                       
C                                                                       
C        FORM INVERSE(U)*INVERSE(L)                                     
C                                                                       
         NM1 = N - 1                                                    
         IF (NM1 .LT. 1) GO TO 140                                      
         DO 130 KB = 1, NM1                                             
            K = N - KB                                                  
            KP1 = K + 1                                                 
            DO 110 I = KP1, N                                           
               WORK(I) = A(I,K)                                         
               A(I,K) = 0.0D+00                                         
  110       CONTINUE                                                    
            DO 120 J = KP1, N                                           
               T = WORK(J)                                              
               CALL DAXPY(N,T,A(1,J),1,A(1,K),1)                        
  120       CONTINUE                                                    
            L = IPVT(K)                                                 
            IF (L .NE. K) CALL DSWAP(N,A(1,K),1,A(1,L),1)               
  130    CONTINUE                                                       
  140    CONTINUE                                                       
  150 CONTINUE                                                          
      RETURN                                                            
      END            

c****************************************************************
c      subroutine daxpy(n,da,dx,incx,dy,incy)
c****************************************************************
c
      subroutine daxpy(n,da,dx,incx,dy,incy)
c
c     constant times a vector plus a vector.
c     uses unrolled loops for increments equal to one.
c     jack dongarra, linpack, 3/11/78.
c
      double precision dx(1),dy(1),da
      integer i,incx,incy,ix,iy,m,mp1,n
c
      if(n.le.0)return
      if (da .eq. 0.0d0) return
      if(incx.eq.1.and.incy.eq.1)go to 20
c
c        code for unequal increments or equal increments
c          not equal to 1
c
      ix = 1
      iy = 1
      if(incx.lt.0)ix = (-n+1)*incx + 1
      if(incy.lt.0)iy = (-n+1)*incy + 1
      do 10 i = 1,n
        dy(iy) = dy(iy) + da*dx(ix)
        ix = ix + incx
        iy = iy + incy
   10 continue
      return
c
c        code for both increments equal to 1
c
c
c        clean-up loop
c
   20 m = mod(n,4)
      if( m .eq. 0 ) go to 40
      do 30 i = 1,m
        dy(i) = dy(i) + da*dx(i)
   30 continue
      if( n .lt. 4 ) return
   40 mp1 = m + 1
      do 50 i = mp1,n,4
        dy(i) = dy(i) + da*dx(i)
        dy(i + 1) = dy(i + 1) + da*dx(i + 1)
        dy(i + 2) = dy(i + 2) + da*dx(i + 2)
        dy(i + 3) = dy(i + 3) + da*dx(i + 3)
   50 continue
      return
      end
       
c**********************************************************************
c      SUBROUTINE RSP(A,N,MATZ,W,Z)
c**********************************************************************
c
      SUBROUTINE RSP(A,N,MATZ,W,Z)

      implicit none
      integer N, n3tm, nv, nm, matz, ierr, i, j
c

      double precision a(*),  w(n), z(n,n)
c*******************************************************************
c*
c*   eispack diagonalization routines: to find the eigenvalues and
c*           eigenvectors (if desired) of a real symmetric packed matrix.
c* on input-      n  is the order of the matrix  a,
c*                a  contains the lower triangle of the real symmetric
c*                   packed matrix stored row-wise,
c*             matz  is an integer variable set equal to zero if only
c*                   eigenvalues are desired,  otherwise it is set to
c*                   any non-zero integer for both eigenvalues and
c*                   eigenvectors.
c* on output-     w  contains the eigenvalues in ascending order,
c*                z  contains the eigenvectors if matz is not zero,
c*
c
c* this subroutine was chosen as being the most reliable. (jjps)
c     questions and comments should be directed to b. s. garbow,
c     applied mathematics division, argonne national laboratory
c
C     ------------------------------------------------------------------
C
C FV and FV2 are scratch arrays which shouldo be dimensioned to the
C to the order of the biggest diagonalizable matrix
c
      double precision FV1(N),FV2(N)

      n3tm = N
      NV=(N*(N+1))/2
      NM=N
C
20    call tred3(n,nv,a,w,fv1,fv2)
      IF (MATZ .NE. 0) GO TO 30
C     ********** FIND EIGENVALUES ONLY **********
       call tqlrat(n,w,fv2,ierr)
      GO TO 60
C     ********** FIND BOTH EIGENVALUES AND EIGENVECTORS **********
   30 DO 50    I = 1, N
C
         DO 40    J = 1, N
            Z(J,I)=0.0D0
   40    CONTINUE
C
         Z(I,I)=1.0D0
   50 CONTINUE
Ci
      call tql2(nm,n,w,fv1,z,ierr)
      IF (IERR .NE. 0) GO TO 70
      call trbak3(nm,n,nv,a,n,z,ierr)                                   IR0494
   60 return
C     ********** LAST CARD OF RSP **********
   70 write(2,'( "Error",I3," in RSP" )') ierr                          IR0494
      RETURN
      END
c

c===========================================================
c
      integer function idamax(n,dx,incx)
c
c     finds the index of element having max. absolute value.
c     jack dongarra, linpack, 3/11/78.
c     modified 3/93 to return if incx .le. 0.
c     modified 12/3/93, array(1) declarations changed to array(*)
c
      double precision dx(*),dmax
      integer i,incx,ix,n
c
      idamax = 0
      if( n.lt.1 .or. incx.le.0 ) return
      idamax = 1
      if(n.eq.1)return
      if(incx.eq.1)go to 20
c
c        code for increment not equal to 1
c
      ix = 1
      dmax = dabs(dx(1))
      ix = ix + incx
      do 10 i = 2,n
         if(dabs(dx(ix)).le.dmax) go to 5
         idamax = i
         dmax = dabs(dx(ix))
    5    ix = ix + incx
   10 continue
      return
c
c        code for increment equal to 1
c
   20 dmax = dabs(dx(1))
      do 30 i = 2,n
         if(dabs(dx(i)).le.dmax) go to 30
         idamax = i
         dmax = dabs(dx(i))
   30 continue
      return
      end
c
c====================================================
c
c************************************************************************
c  
      subroutine  dscal(n,da,dx,incx)
c  
c     scales a vector by a constant.
c     uses unrolled loops for increment equal to one.
c     jack dongarra, linpack, 3/11/78.
c     modified 3/93 to return if incx .le. 0.
c     modified 12/3/93, array(1) declarations changed to array(*)
c          
      double precision da,dx(*)
      integer i,incx,m,mp1,n,nincx
c          
      if( n.le.0 .or. incx.le.0 )return 
      if(incx.eq.1)go to 20
c
c        code for increment not equal to 1
c          
      nincx = n*incx
      do 10 i = 1,nincx,incx
        dx(i) = da*dx(i)
   10 continue 
      return
c  
c        code for increment equal to 1
c          
c
c        clean-up loop
c          
   20 m = mod(n,5)
      if( m .eq. 0 ) go to 40
      do 30 i = 1,m
        dx(i) = da*dx(i) 
   30 continue
      if( n .lt. 5 ) return
   40 mp1 = m + 1
      do 50 i = mp1,n,5
        dx(i) = da*dx(i)
        dx(i + 1) = da*dx(i + 1)
        dx(i + 2) = da*dx(i + 2)
        dx(i + 3) = da*dx(i + 3)
        dx(i + 4) = da*dx(i + 4)
   50 continue
      return
      end
c
c===============================================================
c
c===============================================================
c
      subroutine  dswap (n,dx,incx,dy,incy)
c
c     interchanges two vectors.
c     uses unrolled loops for increments equal one.
c     jack dongarra, linpack, 3/11/78.
c     modified 12/3/93, array(1) declarations changed to array(*)
c
      double precision dx(*),dy(*),dtemp
      integer i,incx,incy,ix,iy,m,mp1,n
c
      if(n.le.0)return
      if(incx.eq.1.and.incy.eq.1)go to 20
c
c       code for unequal increments or equal increments not equal
c         to 1
c
      ix = 1
      iy = 1
      if(incx.lt.0)ix = (-n+1)*incx + 1
      if(incy.lt.0)iy = (-n+1)*incy + 1
      do 10 i = 1,n
        dtemp = dx(ix)
        dx(ix) = dy(iy)
        dy(iy) = dtemp
        ix = ix + incx
        iy = iy + incy
   10 continue
      return
c
c       code for both increments equal to 1
c
c
c       clean-up loop
c
   20 m = mod(n,3)
      if( m .eq. 0 ) go to 40
      do 30 i = 1,m
        dtemp = dx(i)
        dx(i) = dy(i)
        dy(i) = dtemp
   30 continue
      if( n .lt. 3 ) return
   40 mp1 = m + 1
      do 50 i = mp1,n,3
        dtemp = dx(i)
        dx(i) = dy(i)
        dy(i) = dtemp
        dtemp = dx(i + 1)
        dx(i + 1) = dy(i + 1)
        dy(i + 1) = dtemp
        dtemp = dx(i + 2)
        dx(i + 2) = dy(i + 2)
        dy(i + 2) = dtemp
   50 continue
      return
      end
c
c===========================================================
c*******************************************************
c       SUBROUTINE tred3 (N,NV,A,DA,E,E2)
c********************************************************
c
        SUBROUTINE tred3 (N,NV,A,DA,E,E2)
c
c     called by:
c                 rsp
c
      implicit double precision (a-h,o-z)
      double precision a(nv),da(n),e(n),e2(n)
      double precision f,g,h,hh,scale
c     
c     real sqrt,abs,sign
c     
c     this subroutine is a translation of the algol procedure tred3,
c     num. math. 11, 181-195(1968) by martin, reinsch, and wilkinson.
c     handbook for auto. comp., vol.ii-linear algebra, 212-226(1971).
c     
c     this subroutine reduces a real symmetric matrix, stored as
c     a one-dimensional array, to a symmetric tridiagonal matrix
c     using orthogonal similarity transformations.
c
c     on input-
c     
c        n is the order of the matrix,
c     
c        nv must be set to the dimension of the array parameter a
c          as declared in the calling program dimension statement,
c    
c        a contains the lower triangle of the real symmetric
c          input matrix, stored row-wise as a one-dimensional
c          array, in its first n*(n+1)/2 positions.
c
c     on output-
c
c        a contains information about the orthogonal
c          transformations used in the reduction,
c
c        d contains the diagonal elements of the tridiagonal matrix,
c
c        e contains the subdiagonal elements of the tridiagonal
c          matrix in its last n-1 positions.  e(1) is set to zero,
c
c        e2 contains the squares of the corresponding elements of e.
c          e2 may coincide with e if the squares are not needed.
c
c     questions and comments should be directed to b. s. garbow,
c     applied mathematics division, argonne national laboratory
C
C     ------------------------------------------------------------------
C
C     ********** FOR I=N STEP -1 UNTIL 1 DO -- **********
C
      DATA ZERO / 0.0D0 /
      DO 90 II = 1, N
         I = N+1-II
         L = I-1
         IZ = (I*L)/2
         H = ZERO
         SCALE = ZERO
         SCALE = ZERO
         IF (L.LT.1) GO TO 20
C
C     ********** SCALE ROW (ALGOL TOL THEN NOT NEEDED) **********
C
         DO 10 K = 1, L
            IZ = IZ+1
            DA(K) = A(IZ)
            SCALE = SCALE+ABS(DA(K))
   10    CONTINUE
C
         IF (SCALE.NE.ZERO) GO TO 30
   20    E(I) = ZERO
         E2(I) = ZERO
         GO TO 80
C
   30    DO 40 K = 1, L
            DA(K) = DA(K)/SCALE
            H = H+DA(K)*DA(K)
   40    CONTINUE
C
         E2(I) = SCALE*SCALE*H
         F = DA(L)
         G = -SIGN(SQRT(H),F)
         E(I) = SCALE*G
         H = H-F*G
         DA(L) = F-G
         A(IZ) = SCALE*DA(L)
         IF (L.EQ.1) GO TO 80
         F = ZERO
C
         DO 60 J = 1, L
            G = ZERO
            JK = (J*(J-1))/2
C
C     ********** FORM ELEMENT OF A*U **********
C
            DO 50 K = 1, L
               JK = JK+1
               IF (K.GT.J) JK = JK+K-2
               G = G+A(JK)*DA(K)
   50       CONTINUE
C
C     ********** FORM ELEMENT OF P **********
C
            E(J) = G/H
            F = F+E(J)*DA(J)
   60    CONTINUE
C
         HH = F/(H+H)
         JK = 0
C
C     ********** FORM REDUCED A **********

         DO 70 J = 1, L
            F = DA(J)
            G = E(J)-HH*F
            E(J) = G
C
            DO 70 K = 1, J
               JK = JK+1
               A(JK) = A(JK)-F*E(K)-G*DA(K)
   70    CONTINUE
C
   80    DA(I) = A(IZ+1)
         A(IZ+1) = SCALE*SQRT(H)
   90 CONTINUE
C
      RETURN
C
C     ********** LAST CARD OF TRED3 **********
C
      END
C    
c*****************************************************************
c      SUBROUTINE tqlrat (N,DA,E2,IERR)
c*****************************************************************
c
      subroutine tqlrat (n,da,e2,ierr)
c
c     called by:
c                rsp
c
      implicit double precision (a-h,o-z)
      double precision da(n),e2(n)
      double precision b,c,f,g,h,p,r,s,machep
c
c     real sqrt,abs,sign
c
c     this subroutine is a translation of the algol procedure tqlrat,
c     algorithm 464, comm. acm 16, 689(1973) by reinsch.
c
c     this subroutine finds the eigenvalues of a symmetric
c     tridiagonal matrix by the rational ql method.
c
c     on input-
c
c        n is the order of the matrix,
c
c        d contains the diagonal elements of the input matrix,
c
c        e2 contains the squares of the subdiagonal elements of the
c          input matrix in its last n-1 positions.  e2(1) is arbitrary.
c
c      on output-
c
c        d contains the eigenvalues in ascending order.  if an
c          error exit is made, the eigenvalues are correct and
c          ordered for indices 1,2,...ierr-1, but may not be
c          the smallest eigenvalues,
c
c        e2 has been destroyed,
c
c        ierr is set to
c          zero       for normal return,
c          j          if the j-th eigenvalue has not been
c                     determined after 30 iterations.
c
c     questions and comments should be directed to b. s. garbow,
c     applied mathematics division, argonne national laboratory
c
c     ------------------------------------------------------------------
c
c                **********
c
      data zero,one,two / 0.0d0,1.0d0,2.0d0 /
c
c     ********** machep is a machine dependent parameter specifying
c                the relative precision of floating point arithmetic.
c
      MACHEP = TWO**(-46)
C
      IERR = 0
      IF (N.EQ.1) GO TO 140
C
      DO 10 I = 2, N
         E2(I-1) = E2(I)
   10 CONTINUE
C
      F = ZERO
      B = ZERO
      E2(N) = ZERO
C
      DO 120 L = 1, N
         J = 0
         H = MACHEP*(ABS(DA(L))+SQRT(E2(L)))
         IF (B.GT.H) GO TO 20
         B = H
         C = B*B
C
C     ********** LOOK FOR SMALL SQUARED SUB-DIAGONAL ELEMENT **********
C
   20    DO 30 M = L, N
            IF (E2(M).LE.C) GO TO 40
C
C     ********** E2(N) IS ALWAYS ZERO, SO THERE IS NO EXIT
C                THROUGH THE BOTTOM OF THE LOOP **********
C
   30    CONTINUE
C
   40    IF (M.EQ.L) GO TO 80
   50    IF (J.EQ.30) GO TO 130
         J = J+1
C
C     ********** FORM SHIFT **********
C
         L1 = L+1
         S = SQRT(E2(L))
         G = DA(L)
         P = (DA(L1)-G)/(TWO*S)
         R = SQRT(P*P+ONE)
         DA(L) = S/(P+SIGN(R,P))
         H = G-DA(L)
C
         DO 60 I = L1, N
            DA(I) = DA(I)-H
   60    CONTINUE
C
         F = F+H
C
C
C     ********** RATIONAL QL TRANSFORMATION **********
C
         G = DA(M)
         IF (G.EQ.ZERO) G = B
         H = G
         S = ZERO
         MML = M-L
C
C     ********** FOR I=M-1 STEP -1 UNTIL L DO -- **********
C
         DO 70 II = 1, MML
            I = M-II
            P = G*H
            R = P+E2(I)
            E2(I+1) = S*R
            S = E2(I)/R
            DA(I+1) = H+S*(H+DA(I))
            G = DA(I)-E2(I)/G
            IF (G.EQ.ZERO) G = B
            H = G*P/R
   70    CONTINUE
C
         E2(L) = S*G
         DA(L) = H
C
C     ********** GUARD AGAINST UNDERFLOW IN CONVERGENCE TEST **********
C
         IF (H.EQ.ZERO) GO TO 80
         IF (ABS(E2(L)).LE.ABS(C/H)) GO TO 80
         E2(L) = H*E2(L)
         IF (E2(L).NE.ZERO) GO TO 50
   80    P = DA(L)+F
C
C     ********** ORDER EIGENVALUES **********
C
         IF (L.EQ.1) GO TO 100
C
C     ********** FOR I=L STEP -1 UNTIL 2 DO -- **********
C
         DO 90 II = 2, L
            I = L+2-II
            IF (P.GE.DA(I-1)) GO TO 110
            DA(I) = DA(I-1)
   90    CONTINUE
C
  100    I = 1
  110    DA(I) = P
  120 CONTINUE
C
      GO TO 140
C
C     ********** SET ERROR -- NO CONVERGENCE TO AN
C                EIGENVALUE AFTER 30 ITERATIONS **********
C
  130 IERR = L
  140 RETURN
C     
C     ********** LAST CARD OF TQLRAT **********
C     
      END
c*********************************************************************
c      SUBROUTINE tql2 (NM,N,DA,E,Z,IERR)
c*********************************************************************
c     
      subroutine tql2 (nm,n,da,e,z,ierr)
c     
c     called by: 
c                rsp
c     
      implicit double precision (a-h,o-z)
      double precision da(n),e(n),z(nm,n)
      double precision b,c,f,g,h,p,r,s,machep
c     
c     real sqrt,abs,sign
c     
c     this subroutine is a translation of the algol procedure tql2,
c     num. math. 11, 293-306(1968) by bowdler, martin, reinsch, and
c     wilkinson.
c     handbook for auto. comp., vol.ii-linear algebra, 227-240(1971).
c     
c     this subroutine finds the eigenvalues and eigenvectors
c     of a symmetric tridiagonal matrix by the ql method.
c     the eigenvectors of a full symmetric matrix can also
c     be found if  tred2  has been used to reduce this
c     full matrix to tridiagonal form.
c     
c     on input-
c        
c        nm must be set to the row dimension of two-dimensional
c          array parameters as declared in the calling program
c          dimension statement,
c        
c        n is the order of the matrix,
c        
c        d contains the diagonal elements of the input matrix,
c        
c        e contains the subdiagonal elements of the input matrix
c          in its last n-1 positions.  e(1) is arbitrary,
c        
c        z contains the transformation matrix produced in the
c          reduction by  tred2, if performed.  if the eigenvectors
c          of the tridiagonal matrix are desired, z must contain
c          the identity matrix.
c      
c      on output-
c        
c        d contains the eigenvalues in ascending order.  if an
c          error exit is made, the eigenvalues are correct but
c          unordered for indices 1,2,...,ierr-1,
c        
c        e has been destroyed,
c        
c        z contains orthonormal eigenvectors of the symmetric 
c          tridiagonal (or full) matrix.  if an error exit is made,

c          z contains the eigenvectors associated with the stored
c          eigenvalues,
c
c        ierr is set to
c          zero       for normal return,
c          j          if the j-th eigenvalue has not been
c                     determined after 30 iterations.
c
c     questions and comments should be directed to b. s. garbow,
c     applied mathematics division, argonne national laboratory
c
c     ------------------------------------------------------------------
c
c                **********
c
      data zero,one,two / 0.0d0,1.0d0,2.0d0 /
c
c     ********** machep is a machine dependent parameter specifying
c                the relative precision of floating point arithmetic.
C
      MACHEP = TWO**(-46)
C
      IERR = 0
      IF (N.EQ.1) GO TO 160
C
      DO 10 I = 2, N
         E(I-1) = E(I)
   10 CONTINUE
C
      F = ZERO
      B = ZERO
      E(N) = ZERO
C
      DO 110 L = 1, N
         J = 0
         H = MACHEP*(ABS(DA(L))+ABS(E(L)))
         IF (B.LT.H) B = H
C
C     ********** LOOK FOR SMALL SUB-DIAGONAL ELEMENT **********
C
         DO 20 M = L, N
            IF (ABS(E(M)).LE.B) GO TO 30
C
C     ********** E(N) IS ALWAYS ZERO, SO THERE IS NO EXIT
C                THROUGH THE BOTTOM OF THE LOOP **********
C
   20    CONTINUE
C
   30    IF (M.EQ.L) GO TO 100
   40    IF (J.EQ.30) GO TO 150
         J = J+1
C
C     ********** FORM SHIFT **********
C
         L1 = L+1
         G = DA(L)
         P = (DA(L1)-G)/(TWO*E(L))
C
C        R = SQRT(P*P+ONE)
C
         IF (ABS(P).GT.1.0D+16) THEN

            R = ABS(P)
         ELSE
            R = SQRT(P*P+ONE)
         ENDIF
         DA(L) = E(L)/(P+SIGN(R,P))
         H = G-DA(L)  
C                     
         DO 50 I = L1, N
            DA(I) = DA(I)-H
   50    CONTINUE
C
         F = F+H
C
C     ********** QL TRANSFORMATION **********
C
         P = DA(M)
         C = ONE
         S = ZERO
         MML = M-L
C
C     ********** FOR I=M-1 STEP -1 UNTIL L DO -- **********
C
         DO 90 II = 1, MML
            I = M-II 
            G = C*E(I)
            H = C*P
            IF (ABS(P).LT.ABS(E(I))) GO TO 60
            C = E(I)/P
            R = SQRT(C*C+ONE)
            E(I+1) = S*P*R
            S = C/R
            C = ONE/R
            GO TO 70
   60       C = P/E(I)
            R = SQRT(C*C+ONE)
            E(I+1) = S*E(I)*R
            S = ONE/R
            C = C*S
   70       P = C*DA(I)-S*G
            DA(I+1) = H+S*(C*G+S*DA(I))
C        
C     ********** FORM VECTOR **********
C
            DO 80 K = 1, N
               H = Z(K,I+1)
               Z(K,I+1) = S*Z(K,I)+C*H
               Z(K,I) = C*Z(K,I)-S*H
   80       CONTINUE
C  
   90    CONTINUE
C        
         E(L) = S*P
         DA(L) = C*P
         IF (ABS(E(L)).GT.B) GO TO 40
  100    DA(L) = DA(L)+F
  110 CONTINUE
C        
C     ********** ORDER EIGENVALUES AND EIGENVECTORS **********
C        
      DO 140 II = 2, N
         I = II-1
         I = II-1
         K = I
         P = DA(I)
C
         DO 120 J = II, N
            IF (DA(J).GE.P) GO TO 120
            K = J
            P = DA(J)
  120    CONTINUE
C
         IF (K.EQ.I) GO TO 140
         DA(K) = DA(I)
         DA(I) = P
C
         DO 130 J = 1, N
            P = Z(J,I)
            Z(J,I) = Z(J,K)
            Z(J,K) = P
  130    CONTINUE
C
  140 CONTINUE
C
      GO TO 160
C
C     ********** SET ERROR -- NO CONVERGENCE TO AN
C                EIGENVALUE AFTER 30 ITERATIONS **********
C
  150 IERR = L
  160 RETURN
C
C     ********** LAST CARD OF TQL2 **********
C
      END

c****************************************************************
c       SUBROUTINE trbak3 (NM,N,NV,A,M,Z,IERR)
c****************************************************************
c
       subroutine trbak3 (nm,n,nv,a,m,z,ierr)

c     called by:
c                rsp
c
      implicit double precision (a-h,o-z)
      double precision a(nv),z(nm,m)
      double precision h,s
c
c     this subroutine is a translation of the algol procedure trbak3,
c     num. math. 11, 181-195(1968) by martin, reinsch, and wilkinson.
c     handbook for auto. comp., vol.ii-linear algebra, 212-226(1971).
c
c     this subroutine forms the eigenvectors of a real symmetric
c     matrix by back transforming those of the corresponding
c     symmetric tridiagonal matrix determined by  tred3.
c
c     on input-
c
c        nm must be set to the row dimension of two-dimensional
c          array parameters as declared in the calling program
c          dimension statement,
c
c        n is the order of the matrix,
c        
c        nv must be set to the dimension of the array parameter a
c          as declared in the calling program dimension statement,
c        
c        a contains information about the orthogonal transformations
c          used in the reduction by  tred3  in its first
c          n*(n+1)/2 positions,
c 
c        m is the number of eigenvectors to be back transformed,
c        
c        z contains the eigenvectors to be back transformed
c          in its first m columns.
c
c     on output-
c           
c        z contains the transformed eigenvectors
c          in its first m columns.
c 
c     note that trbak3 preserves vector euclidean norms.
c 
c     questions and comments should be directed to b. s. garbow,
c     applied mathematics division, argonne national laboratory
c
c     ------------------------------------------------------------------
C                
      DATA ZERO / 0.0D0 /
      IF (M.EQ.0) GO TO 60
      IF (N.EQ.1) GO TO 60
C
      DO 50 I = 2, N
         L = I-1
         IZ = (I*L)/2
         IK = IZ+I
         H = A(IK)
         IF (H.EQ.ZERO) GO TO 50
C
         DO 40 J = 1, M
            S = ZERO
            IK = IZ
C     
            DO 10 K = 1, L
               IK = IK+1
               S = S+A(IK)*Z(K,J)
   10       CONTINUE
C     
C     ********** DOUBLE DIVISION AVOIDS POSSIBLE UNDERFLOW **********
C     
            S = S/H
            IK = IZ
C
            DO 30 K = 1, L
               IK = IK+1
               IF (ABS(H).GT.1.0D0) GO TO 20
               SA = S*A(IK)
               IF (ABS(SA).LT.ABS(H)*1.6D+38) GO TO 20
               IERR = 1111
               RETURN
   20          CONTINUE
               Z(K,J) = Z(K,J)-S*A(IK)/H
   30       CONTINUE
C        
C
   40    CONTINUE
C
   50 CONTINUE
C
   60 RETURN
C
C     ********** LAST CARD OF TRBAK3 **********
C
       END



