c*****************************************************************************
C
C      GHOOKS
C
c******************************************************************************
C
C      this is the subroutine to call the appropriate gradient hook
C
C       created : April 12, by E. E. Dahlke
C       revised : May 20, 2009 by Hannah Leverentz
C                 June 12, 2009 by E. Dahlke Speetzen
C                 March 9, 2011 by Hannah Leverentz
C       purpose : to call the right gradient hook
C       revision: to incorporate into the parallel code the changes
C                 made by Hai Lin on May 11, 2009 to the serial version.
C                 Hai Lin's revision was to add a control variable, 
C                 icaltype, which indicates gradient calculation when 
C                 going to eg09; gg09 is not used any more.
C                 6/12/09 revised to make use of core and keywords options
C                 3/09/11 revised to allow EE-MB population/charge analyses

      subroutine ghooks(natms,rstrt,eemb,fragid,cm,afrag, nfrg,
     *                  basis,title,bgq,esm,method,geom,asym,nbasis,
C  revised 6/12/09 by EDS
     *                  mem,nbgq,bkey,ekey,gradkey,hesskey,keywrds,
C  Add by Hannah Leverentz, 3/9/2011
c    *                  core,pseudo,ncore)
     *                  core,pseudo,ncore,cpop,lscreen,
     *                  allmmpp,nmmpp,mmpseudo,leembce)                  !1110HL11; 1115HL11 add leembce
C  End by Hannah Leverentz, 3/9/2011
C  end 6/12/09 EDS


      implicit none
      include 'common.inc'

      include 'mpif.h'
      INTEGER NWORKERS
      PARAMETER (NWORKERS=4096)
      INTEGER SIZE,ierr,WorkersAvailable,RecordsToProcess,NULL
      INTEGER II, CurrentRecord
      LOGICAL DONE
      LOGICAL AVAILABLE(NWORKERS)
      INTEGER STATUS(MPI_STATUS_SIZE)

      integer   natms, rstrt, eemb,  nfrg, nf,nbasis, nbgq
      integer   fragid(maxfrag,maxapf), cm(maxfrag,2), afrag(maxfrag)
      integer   frglst(maxlist,3), nftc, ekey, gradkey, hesskey
      integer   joblist(maxjob,5),nolig(3),imrk                                 !0523HL12
      integer   nm, nd, nt, tstrt, tfin, tend,  iistrt, jj, jstrt
      integer i, j,k, tfid(3)                                                   !0523HL12 deleted",  donef(maxlist)"
      integer donef(maxjob),njtc                                                !0523HL12

C revised 6/12/09 EDS
      integer pseudo, ncore
C  end 6/12/09 EDS
c
c Added by Hai Lin, 5/11/2009
c
      integer   icaltype
c
c End by Hai Lin, 5/11/2009
c
      character*80 basis(1000), title(5), string, temp
C  added 6/12/09 by eds
      character*80 core(1000), keywrds
C  end 6/12/09 by eds
      character*20 bgq(maxat,5), esm, method,mem,  command,bkey
      character*30 tout
      character*6 tof
      character*1 cmode             !1115HL11
C  Add by Hannah Leverentz, 3/9/2011
      character*20 cpop
C  End by Hannah Leverentz, 3/9/2011
      double precision dip1b(3),totdip1b,qqm1b(maxat)                   ! 1110HL11
      integer lscreen
      integer leembce                                                   ! 1115HL11
      integer nmmpp,mmpseudo        !1110HL11
      character*80 allmmpp(1000)    !1110HL11

c Added by Hai Lin, 5/11/2009
c
c   This indicates gradient calculations when calling eg09
c
      icaltype = 1
c
c End by Hai Lin, 5/11/2009

c Initialize the joblist array to zero.  See the comments at the        !0523HL12
c  beginning of the updtjobs subroutine for a description of the        !0523HL12
c  contents of the joblist array.                                       !0523HL12
c                                                                       !0523HL12
      joblist = 0                                                       !0523HL12
      njtc    = 0                                                       !0523HL12
      imrk    = 0                                                       !0523HL12
c
      DO I=1,NWORKERS
        AVAILABLE(I)=.TRUE.
      ENDDO
      NULL=0

      do i = 1, eemb                                                    !0523HL12  rewrote
         call getfrag(nfrg,i,nf,frglst)                                 !0523HL12  this
         write(tout,*) i                                                !0523HL12  entire
         tfin = 1                                                       !0523HL12  section -
         tend = 1                                                       !0523HL12  deleted
         call fchar(tout,tfin,tend)                                     !0523HL12  old lines
         tstrt = tfin                                                   !0523HL12  and replaced
         call fspace(tout,tfin,tend)                                    !0523HL12  with these.
         tfin = tfin - 1                                                !0523HL12
         tof = tout(tstrt:tfin) // '.eemb'                              !0523HL12
         call updtjobs(rstrt,i,nf,frglst,njtc,tout,tstrt,tfin,imrk,     !0523HL12
     *                 joblist,donef)                                   !0523HL12
         nolig(i) = nf                                                  !0523HL12
      end do                                                            !0523HL12
                                                                        !0523HL12
                                                                        !0523HL12
C   if an eembce calculation is requested, the last job                 !0523HL12
C   described in joblist will be the full hartree-fock                  !0523HL12
C   calculation, denoted by a value of "-1" in each column              !0523HL12
C   of row 1:                                                           !0523HL12
                                                                        !0523HL12
      if (leembce.eq.1) then                                            !0523HL12
        cmode(1:1) = '1'                                                !0523HL12
        imrk = imrk + 1                                                 !0523HL12
        njtc = njtc + 1                                                 !0523HL12
        do i=1,5                                                        !0523HL12
          joblist(imrk,i) = -1                                          !0523HL12
        end do                                                          !0523HL12
        if (rstrt.ne.0) then                                            !0523HL12
          call fndstrt3(imrk,-1,donef,njtc,'null',1,1)                  !0523HL12
        endif                                                           !0523HL12
      else                                                              !0523HL12
        cmode(1:1) = '0'                                                !0523HL12
      end if                                                            !0523HL12

      CALL MPI_COMM_SIZE(MPI_COMM_WORLD, size, ierr)

           if (SIZE.EQ.1) THEN

         do i = 1, imrk                                                 !0523HL12  This portion of
           if (donef(i).eq.0) then                                      !0523HL12  the code was rewritten;
             do k = 1, 3                                                !0523HL12  it replaces a segment
               tfid(k) = joblist(i,k+2)                                 !0523HL12  of the old code.
             end do                                                     !0523HL12
             if (esm .eq. 'g09') then                                   !0523HL12
               if (tfid(1).eq.-1) then                                  !0523HL12
                 call setuphf(natms,method,basis,nbasis,bkey,cm,asym,   !0523HL12
     *                        geom,icaltype,keywrds,pseudo,core,ncore,  !0523HL12
     *                        mem)                                      !0523HL12
               else                                                     !0523HL12
                  call eg09(natms,fragid,cm,afrag,nfrg,basis,           !0523HL12
     *                   title,bgq,method,geom,asym,tfid,joblist(i,2),  !0523HL12
     *                   nbasis,mem,nbgq,joblist(i,1),bkey,icaltype,    !0523HL12
     *                   keywrds,core,pseudo,ncore,cpop,lscreen,        !0523HL12
     *                   allmmpp,nmmpp,mmpseudo,cmode)                  !0523HL12
               endif							!0523HL12
             elseif (esm .eq. 'nwchem') then                            !0523HL12
C                 call enwchem()                                        !0523HL12
             end if                                                     !0523HL12
           end if                                                       !0523HL12
         end do                                                         !0523HL12

           ELSE

      RecordsToProcess = njtc                                           !0523HL12 changed "nftc" to "njtc"
      CurrentRecord = imrk                                              !0523HL12 changed "= 1" to "= imrk"
!
      WorkersAvailable=size-1
      DONE=.FALSE.

! Start MAIN Program Loop
      DO WHILE (.NOT.DONE)
        IF (WorkersAvailable.GT.0.AND.RecordsToProcess.GT.0) THEN
          DO II=1,SIZE-1
          IF(AVAILABLE(II).AND.RecordsToProcess.GT.0) THEN

              j=CurrentRecord
	      if (donef(j) .eq. 0) then
              do k = 1, 3
                 tfid(k) = joblist(j,k+2)                               !0523HL12 changed "frglst(j,k)" to "joblist(j,k+2)"
              end do
              CALL MPI_Send(esm,20, MPI_CHARACTER,             !0627HQ12
     *                      II, 13, MPI_COMM_WORLD,ierr)       !0627HQ12
              CALL MPI_Send(ekey,1,MPI_INTEGER,
     *                      II, 14, MPI_COMM_WORLD,ierr)
              CALL MPI_Send(gradkey,1,MPI_INTEGER,
     *                      II, 15, MPI_COMM_WORLD,ierr)
              CALL MPI_Send(hesskey,1,MPI_INTEGER,
     *                      II, 16, MPI_COMM_WORLD,ierr)
              CALL MPI_Send(natms,1, MPI_INTEGER,
     *                      II, 17, MPI_COMM_WORLD,ierr)
              CALL MPI_Send(fragid,maxfrag*maxapf, MPI_INTEGER,
     *                      II, 18, MPI_COMM_WORLD,ierr)
              CALL MPI_Send(cm,maxfrag*2, MPI_INTEGER,
     *                      II, 19, MPI_COMM_WORLD,ierr)
              CALL MPI_Send(afrag,maxfrag, MPI_INTEGER,
     *                      II, 20, MPI_COMM_WORLD,ierr)
              CALL MPI_Send(nfrg,1, MPI_INTEGER,
     *                      II, 21, MPI_COMM_WORLD,ierr)
              CALL MPI_Send(basis,1000*80, MPI_CHARACTER,
     *                      II, 22, MPI_COMM_WORLD,ierr)
              CALL MPI_Send(title,5*80, MPI_CHARACTER,
     *                      II, 23, MPI_COMM_WORLD,ierr)
              CALL MPI_Send(bgq,20*maxat*5, MPI_CHARACTER,
     *                      II, 24, MPI_COMM_WORLD,ierr)
              CALL MPI_Send(method,20, MPI_CHARACTER,
     *                      II, 25, MPI_COMM_WORLD,ierr)
              CALL MPI_Send(geom,maxat*3, MPI_DOUBLE_PRECISION,
     *                      II, 26, MPI_COMM_WORLD,ierr)
              CALL MPI_Send(asym,maxat*8, MPI_CHARACTER,
     *                      II, 27, MPI_COMM_WORLD,ierr)
              CALL MPI_Send(tfid,3, MPI_INTEGER,
     *                      II, 28, MPI_COMM_WORLD,ierr)
              CALL MPI_Send(joblist(j,2),1,MPI_INTEGER,                 !0523HL12 changed "(j," to "(joblist(j,2),"
     *                      II, 29, MPI_COMM_WORLD,ierr)
              CALL MPI_Send(nbasis,1, MPI_INTEGER,
     *                      II, 30, MPI_COMM_WORLD,ierr)
              CALL MPI_Send(mem,20, MPI_CHARACTER,
     *                      II, 31, MPI_COMM_WORLD,ierr)
              CALL MPI_Send(nbgq,1, MPI_INTEGER,
     *                      II, 32, MPI_COMM_WORLD,ierr)
              CALL MPI_Send(joblist(j,1),1, MPI_INTEGER,                !0523HL12 changed "(i," to "(joblist(j,1),"
     *                      II, 33, MPI_COMM_WORLD,ierr)
              CALL MPI_Send(bkey,20, MPI_CHARACTER,
     *                      II, 34, MPI_COMM_WORLD,ierr)
c
c Added by Hannah Leverentz, 5/20/2009
c
              CALL MPI_Send(icaltype, 1, MPI_INTEGER,
     *                      II, 35, MPI_COMM_WORLD,ierr)
c
c End by Hannah Leverentz, 5/20/2009
c
c  added 6/17/09 by Erin Dahlke Speetzen
              CALL MPI_Send(ncore,1,MPI_INTEGER,
     *                      II, 36, MPI_COMM_WORLD,ierr)
              CALL MPI_Send(pseudo,1,MPI_INTEGER,
     *                      II, 37, MPI_COMM_WORLD,ierr)
              CALL MPI_Send(keywrds,80,MPI_CHARACTER,
     *                      II, 38, MPI_COMM_WORLD,ierr)
              CALL MPI_Send(core,1000*80, MPI_CHARACTER,
     *                      II, 39, MPI_COMM_WORLD,ierr)
c  end add 6/17/09
C  Add by Hannah Leverentz, 3/9/2011
              CALL MPI_Send(cpop, 20, MPI_CHARACTER,
     *                      II, 40, MPI_COMM_WORLD,ierr)
     		  CALL MPI_Send(lscreen, 1, MPI_INTEGER, 
     *		  			    II, 41, MPI_COMM_WORLD,ierr) 
              CALL MPI_Send(allmmpp,1000*80,MPI_CHARACTER,             !1110HL11
     *                      II, 42, MPI_COMM_WORLD,ierr)               !1110HL11
              CALL MPI_Send(nmmpp,1,MPI_INTEGER,                       !1110HL11
     *                      II, 43, MPI_COMM_WORLD,ierr)               !1110HL11
              CALL MPI_Send(mmpseudo,1,MPI_INTEGER,                    !1110HL11
     *                      II, 44, MPI_COMM_WORLD,ierr)               !1110HL11
              CALL MPI_Send(cmode,1,MPI_CHARACTER,                     !1115HL11
     *                      II, 45, MPI_COMM_WORLD,ierr)               !1115HL11
     		  			    
C  End by Hannah Leverentz, 3/9/2011

              AVAILABLE(II)=.FALSE.
              WorkersAvailable = WorkersAvailable -1
              CurrentRecord    = CurrentRecord    -1                  !0523HL12 changed "+1" to "-1"
              RecordsToProcess = RecordsToProcess -1
	      else
                CurrentRecord    = CurrentRecord    -1                !0523HL12 changed "+1" to "-1"
              end if
          ENDIF
          ENDDO
        ELSE
          IF (WorkersAvailable.LT.SIZE-1) THEN
          CALL MPI_Recv(NULL, 1,MPI_INTEGER, MPI_ANY_SOURCE, 12,
     *             MPI_COMM_WORLD, status, ierr)
          AVAILABLE(STATUS(MPI_SOURCE))=.TRUE.
          WorkersAvailable=WorkersAvailable+1
         ENDIF
        ENDIF
        IF (RecordsToProcess.EQ.0.AND.WorkersAvailable.EQ.size-1)
     *      DONE=.TRUE.
      ENDDO
!
! End ehook Loop
!
!
         ENDIF

C  write energy, dipole, and gradient information
         
C  Add by Hannah Leverentz, 3/9/2011
c     call wg09g(eemb,nfrg,natms,afrag,fragid)
      call wg09g(eemb,nfrg,natms,afrag,fragid,asym,cpop)
C  End by Hannah Leverentz, 3/9/2011
         
      return

      end

c******************************************************************************
C               
C      WG09G
C          
c******************************************************************************
C
C     created : May 30th, by E. E. Dahlke
C     revised : May 11, 2009 by Hai Lin
C               March 9, 2011 by Hannah Leverentz
c
C     purpose : to write energy, dipole and gradient information to output file
C     Revision: pass nfrg information to g09outg subroutine
C               03/09/2011 revised to allow EE-MB population/charge analyses
      
C  Add by Hannah Leverentz, 3/9/2011
c     subroutine wg09g(eemb,nfrg,natms,afrag,fragid)
      subroutine wg09g(eemb,nfrg,natms,afrag,fragid,asym,cpop)
C  End by Hannah Leverentz, 3/9/2011

      implicit none
      include 'common.inc'
         
      integer i, j, eemb,nfrg, nf, natms
      integer  eflg(maxe), frglst(maxlist,3), fragid(maxfrag,maxapf)
      integer afrag(maxfrag)
      character*30 etype(maxe), tout
      character*6 t1                                            !0213HL12 changed tof from a 6-char variable
      character*2 tof                                           !0213HL12  to a 2-character variable
      double precision menergy(maxe), denergy(maxe)
      double precision tenergy(maxe), v1(maxe), v2(maxe), v3(maxe)
      double precision eepa(maxe), ee3b(maxe)
      double precision gpa(3*maxat), g3b(3*maxat)
      double precision gv1(3*maxat), gv2(3*maxat), gv3(3*maxat)
      double precision mgrad(3*maxat), dgrad(3*maxat), tgrad(3*maxat)
      double precision dippa(3), dip3b(3), mdip(3), ddip(3), tdip(3)
      double precision dv1(3), dv2(3), dv3(3)
C  Add by Hannah Leverentz, 3/9/2011
      double precision totdippa, totdip3b
      double precision mqqm(maxat),dqqm(maxat),tqqm(maxat)
      double precision qqmpa(maxat),qqm3b(maxat)
      character*20 cpop
C  End by Hannah Leverentz, 3/9/2011
      double precision dip1b(3),totdip1b,qqm1b(maxat)                   ! 1110HL11

C  zero all energies
      do i = 1, maxe
         eepa(i) = 0
         ee3b(i) = 0
         v1(i) = 0
         v2(i) = 0
         v3(i) = 0
      end do

C  zero all dipoles
      do i = 1, 3
        dip1b(i) = 0                                                    ! 1110HL11
        dippa(i) = 0
        dip3b(i) = 0
      end do

c  Add by Hannah Leverentz, 3/2/2011
c
C  zero the partial charges obtained from oligomer wave functions
      mqqm = 0
      dqqm = 0
      tqqm = 0
c
C  zero the eepa and ee3b estimates of partial charges from the wave function
C    of the entire system
      qqm1b = 0                                                         ! 1110HL11
      qqmpa = 0
      qqm3b = 0
c
c  End by Hannah Leverentz, 3/2/2011

C  zero all gradients
      do i = 1, 3*natms
          gv1(i) = 0
          gv2(i) = 0
          gv3(i) = 0
          gpa(i) = 0
          g3b(i) = 0
      enddo

C  get info out of *.eemb files

      do i = 1, eemb
         call getfrag(nfrg,i,nf,frglst)
c
c Revised by Hai Lin, 5/11/2009
c
c   Pass nfrg to g09outg subroutine
c
         if (i .eq. 1) then
            tof = '_1'                                 !0213HL12 changed tof from '1.eemb' to '_1'
            call g09outg(tof,nf,menergy,eflg,etype,natms,mgrad,mdip,
c     *                    fragid,frglst,afrag)
c  Add by Hannah Leverentz, 3/2/2011
c    *                    fragid,frglst,afrag,nfrg)
     *                    fragid,frglst,afrag,nfrg,mqqm)
c  End by Hannah Leverentz, 3/2/2011
         elseif (i .eq. 2)  then
            tof = '_2'                                 !0213HL12 changed tof from '2.eemb' to '_2'
            call g09outg(tof,nf,denergy,eflg,etype,natms,dgrad,ddip,
c     *                     fragid,frglst,afrag)
c  Add by Hannah Leverentz, 3/2/2011
c    *                     fragid,frglst,afrag,nfrg)
     *                     fragid,frglst,afrag,nfrg,dqqm)
c  End by Hannah Leverentz, 3/2/2011
          elseif (i .eq. 3) then
            tof = '_3'                                 !0213HL12 changed tof from '3.eemb' to '_3'
            call g09outg(tof,nf,tenergy,eflg,etype,natms,tgrad,tdip,
c     *                     fragid,frglst,afrag)
c  Add by Hannah Leverentz, 3/2/2011
c    *                     fragid,frglst,afrag,nfrg)
     *                     fragid,frglst,afrag,nfrg,tqqm)
c  End by Hannah Leverentz, 3/2/2011
          end if
c
c End by Hai Lin, 5/11/2009
c
       end do

C  calculate the EEPA and EE3B energies
         do j = 1, maxe
            if (eflg(j) .eq. 1 ) then
                eepa(j) = denergy(j) - (nfrg-2)*menergy(j)
                v1(j) = menergy(j)
                v2(j) = denergy(j) - (nfrg-1)*menergy(j)
            end if
            if (eemb .eq. 3) then
              if (eflg(j) .eq. 1 ) then
                  ee3b(j) = tenergy(j) - (nfrg-3)*denergy(j)
                  ee3b(j) = ee3b(j) + (nfrg-2)*(nfrg-3)*menergy(j)/2
                  v3(j) = tenergy(j) - (nfrg-2)*denergy(j) +
     *                             (nfrg-2)*(nfrg-1)*menergy(j)/2
              end if
            end if
         end do

C  write header to file with information about V1, V2, and V3 and
C  what types of energy are reported

      if (eemb .eq. 2) then
         write(2,1210)
      elseif (eemb .eq. 3) then
         write(2,1220)
      end if

C  write V1, V2, and V3 (if appropriate to output file for each type of energy

      do i = 1, maxe
         if (eflg(i) .eq. 1) then
             if (eemb .eq. 2 ) then
                write(2,1230) etype(i), v1(i), v2(i)
             elseif (eemb .eq. 3) then
                write(2,1240) etype(i), v1(i), v2(i), v3(i)
             endif
         end if
      end do


C  write header to file with information about EEPA/EE3B and what types
C  of energy are reported
      if (eemb .eq. 2) then
          write(2,1300)
      elseif (eemb .eq. 3) then
          write(2,1400)
      end if

C  write eepa and ee3m (if appropriate) to output file for each type of energy

      do i = 1, maxe
         if (eflg(i) .eq. 1) then
            if (eemb .eq. 2) then
               write(2,1100) etype(i), eepa(i)
            elseif (eemb .eq. 3) then
               write(2,1200) etype(i), eepa(i), ee3b(i)
            end if
         end if
      end do


C  calculate the EEPA and EE3B dipole moments

      do j = 1, 3
         dip1b(j) = mdip(j)                                     ! 1110HL11
         dippa(j) = ddip(j) - (nfrg-2)*mdip(j)
         if (eemb .eq. 3) then
            dip3b(j) = tdip(j) - (nfrg-3)*ddip(j)
            dip3b(j) = dip3b(j) + (nfrg-2)*(nfrg-3)*mdip(j)/2
         endif
      end do

C  Add by Hannah Leverentz, 3/9/2011
c  calculate the EEPA and EE3B total dipole
c
      totdip1b = sqrt(dip1b(1)**2 + dip1b(2)**2 + dip1b(3)**2)  ! 1110HL11
      totdippa = sqrt(dippa(1)**2 + dippa(2)**2 + dippa(3)**2)
      totdip3b = sqrt(dip3b(1)**2 + dip3b(2)**2 + dip3b(3)**2)
c
C  End by Hannah Leverentz, 3/9/2011

C  write dipole moment information

      write(2,1501)

      write(2,1504) dip1b(1)*au2debye, dip1b(2)*au2debye,       ! 1110HL11
     *              dip1b(3)*au2debye, totdip1b*au2debye        ! 1110HL11

C  Add by Hannah Leverentz, 3/9/2011
c     write(2,1502) dippa(1), dippa(2), dippa(3)
      write(2,1502) dippa(1)*au2debye, dippa(2)*au2debye,               ! HL110413 bug fix
     *              dippa(3)*au2debye, totdippa*au2debye                ! HL110413 bug fix
C  End by Hannah Leverentz, 3/9/2011
      if (eemb .eq. 3) then
C  Add by Hannah Leverentz, 3/9/2011
c         write(2,1503) dip3b(1), dip3b(2), dip3b(3)
          write(2,1503) dip3b(1)*au2debye, dip3b(2)*au2debye,           ! HL110413 bug fix
     *                  dip3b(3)*au2debye, totdip3b*au2debye            ! HL110413 bug fix
C  End by Hannah Leverentz, 3/9/2011
      end if

C  Add by Hannah Leverentz, 3/9/2011
c
C Calculate the atomic partial charges
c
      do j = 1, natms
         qqm1b(j) = mqqm(j)                                     ! 1110HL11
         qqmpa(j) = dqqm(j) - (nfrg-2)*mqqm(j)
         if (eemb .eq. 3) then
            qqm3b(j) = tqqm(j) - (nfrg-3)*dqqm(j)
            qqm3b(j) = qqm3b(j) + (nfrg-2)*(nfrg-3)*mqqm(j)/2
         endif
      end do
c
C write the atomic partial charges to the output file
c
      if (cpop(1:4).ne."none") then
        write(2,1530)
        write(2,1540) cpop
        if (eemb.eq.2) then
          write(2,1550)
          do j = 1, natms
            write(2,1560) j, asym(j), qqm1b(j), qqmpa(j)        ! 1110HL11  added qqm1b(j)
          end do
        else if (eemb.eq.3) then
          write(2,1570)
          do j = 1, natms
            write(2,1580) j, asym(j), qqm1b(j), qqmpa(j), qqm3b(j)      ! 1110HL11  added qqm1b(j)
          end do
        end if
      end if
c
C  End by Hannah Leverentz, 3/9/2011

C  calculate the EEPA and EE3B gradients

      do j = 1, 3*natms
         gpa(j) = dgrad(j) - (nfrg-2)*mgrad(j)
         gv1(j) = mgrad(j)
         gv2(j) = dgrad(j) - (nfrg-1)*mgrad(j)
         if (eemb .eq. 3) then
            g3b(j) = tgrad(j) - (nfrg-3)*dgrad(j)
            g3b(j) = g3b(j) + (nfrg-2)*(nfrg-3)*mgrad(j)/2
            gv3(j) = tgrad(j) - (nfrg-2)*dgrad(j) +
     *                   (nfrg-2)*(nfrg-1)*mgrad(j)/2
         end if
       end do

C  write gradient information
      write(2,1405)

      do i = 1, natms
         write(2,1410) gpa(3*(i-1)+1), gpa(3*(i-1)+2), gpa(3*(i-1)+3)
      end do

      if (eemb .eq. 3) then
        write(2,1415)
        do i = 1, natms
           write(2,1410) g3b(3*(i-1)+1), g3b(3*(i-1)+2), g3b(3*(i-1)+3)
        end do
      end if

      return

 9999 write(2,1010) tof
 1010 format (/3x, 'ERROR:  could not open file ', a6)

 1100 format(3x,a30,f20.10)
 1200 format(3x,a30,f20.10,t60,f25.10)
 1210 format(/,/3x,'Type of Energy',t40, '  V1 (Hartrees)  ', t60,
     *        '  V2 (Hartrees)  ',/3x,'--------------',t40,
     *        '-----------------', t60,'-----------------')
 1220 format(/,/3x,'Type of Energy',t40, '  V1 (Hartrees)  ', t60,
     * '  V2 (Hartrees)  ',t80, '  V3 (Hartrees)  ',/3x,
     * '--------------',t40,'-----------------', t60,
     * '-----------------',t80, '-----------------')
 1230 format(3x,a30,f20.10,t55,f20.10)
 1240 format(3x,a30,f20.10,t55,f20.10,t75,f20.10)
 1300 format(/,/3x,'Type of Energy',t35, 'EE-PA Energy (Hartrees)',
     *          /3x,'--------------',t35, '-----------------------')
 1400 format(/,/3x,'Type of Energy',t35,'EE-PA Energy (Hartrees)',
     *       t65, 'EE-3B Energy (Hartrees)',/3x,'--------------',t35,
     *      '-----------------------',t65,'-----------------------')
 1405 format(/,/3x,'EE-PA Gradient (Hartrees/Bohr)')
 1410 format(10x,f12.8,f12.8,f12.8)
 1415 format(/,/3x, 'EE-3B Gradient (Hartrees/Bohr)')
C  Add by Hannah Leverentz, 3/9/2011
c  1501 format(/3x,/'Dipole Moment')
c  1502 format(/3x,'EEPA Dipole Moment : ', f16.8, f16.8, f16.8)
c  1503 format(/3x,'EE3B Dipole Moment : ', f16.8, f16.8, f16.8)
 1501 format(/3x,/'Dipole Moment (in debye)',//'Warning: for ',
     *       'correlated wave function methods such as MP2 and ',
     *       'CCSD the dipole ',/9x,'moment is not calculated by ',
     *       'Gaussian 09',//20x,'X',15x,'Y',15x,'Z',13x,'TOTAL')
 1502 format(/3x,'EEPA: ', f16.8, f16.8, f16.8, f16.8)
 1503 format(/3x,'EE3B: ', f16.8, f16.8, f16.8, f16.8)
 1504 format(/3x,'EE1B: ',f16.8, f16.8, f16.8, f16.8)                   ! 1110HL11
 1530 format(/,/3x,'-----------------------------------------------',
     *         /3x,'             EE-MB CHARGE ANALYSIS',
     *         /3x,'-----------------------------------------------')
 1540 format(/3x,'Method of Charge Analysis: ',a20)
 1550 format(/3x,'                       EE-1B',9x,'EE-PA',             ! 1110HL11
     *       /3x,'Atom #    Element    Charge (e)    Charge (e)',       ! 1110HL11
     *       /3x,'------    -------   ------------  ------------')      ! 1110HL11
 1560 format(3x,i6,4x,a7,3x,f12.6,2x,f12.6)                             ! 1110HL11
 1570 format(/3x,'                       EE-1B',9x,'EE-PA',9x,'EE-3B',  ! 1110HL11
     *       /3x,'Atom #    Element    Charge (e)    Charge (e)',       ! 1110HL11
     *           '    Charge (e)',                                      ! 1110HL11
     *       /3x,'------    -------   ------------  ------------',      ! 1110HL11
     *           '  ------------')                                      ! 1110HL11
 1580 format(3x,i6,4x,a7,3x,f12.6,2x,f12.6,2x,f12.6)                    ! 1110HL11
C  End by Hannah Leverentz, 3/9/2011

      end

c******************************************************************************
C
C      GG09
C
c******************************************************************************
C
C      created : April 12th, 2007 by E. E. Dahlke (modified EG09)
C      revised :
C      purpose :  write the G09 input files for gradient calc
C
C

      subroutine gg09(natms,fragid,cm,afrag,nfrg,basis,title,bgq,
     *                method,geom,asym,tfid,fn,nbasis,mem,nbgq,ee,bkey)


      implicit none
      include 'common.inc'

      integer   natms,  nfrg, fn, fn2, nbasis, len, bstr
      integer   mfin, mend, bfin, bend, tbeg, tend,tstrt,ierr
      integer   chrg, mlt, nbgq, flg, id,ee, estrt, eend, ebeg
      integer   i,j
      integer   fragid(maxfrag,maxapf), cm(maxfrag,2), afrag(maxfrag)
      integer   tfid(3), istrt, iend, strt(3), end(3)
      character*80 basis(1000), title(5), temp, command, temp2, eec
      character*80 temp3
      character*20 bgq(maxat,5), method, infile, name(3),mem, bq
      character*20 chkfl, bkey
      double precision fgeom(maxapf,3)

      write(temp2,*) fn

C  make name of input file equal to the fragment number

      tstrt = 1
      call fchar(temp2,tstrt,ierr)
      tbeg = tstrt
      call fspace(temp2,tstrt,ierr)
      tend = tstrt - 1
      infile = temp2(tbeg:tend) // '.g09'
      len = tend - tbeg + 5

C  make the name for the formated checkpoint file equal to 'fragment #'_'# of fragments'.FChk
      write(eec,*) ee
      estrt = 1
      call fchar(eec,estrt,ierr)
      ebeg = estrt
      call fspace(eec,estrt,ierr)
      eend = estrt
      chkfl = temp2(tbeg:tend) //'_'//eec(ebeg:eend)//'.FChk'
      
      
C  if method is CCSD(T) or QCISD(T) change method to get full MP4 energy
C  as well
       if (method(1:7) .eq. 'ccsd(t)') then
            method = 'ccsd(t,e4t)'
       elseif (method(1:8) .eq. 'qcisd(t)') then
            method = 'qcisd(t,e4t)'
       end if
      
      open(unit=4,file=infile,form='formatted',status='unknown',
     *      err=1000)
C     print memory information
      write(4,1001) mem

C  print route information
C  find the end of the basis set and method specifications
      mfin = 0
      mend = 0
      call fspace(method,mfin,mend)
      mfin = mfin - 1
      if (bkey(1:3) .eq. 'lib') then
        temp = basis(1)
      elseif (bkey(1:3) .eq. 'gen') then
        temp = bkey(1:3)
      end if
      bfin = 1
      bend = 1
      call fchar(temp,bfin,bend)
      bstr = bfin
      call fspace(temp,bfin,bend)
      bfin = bfin - 1
      temp3 = '# '//method(1:mfin)//'/'//temp(bstr:bfin)
     *           //' SCF=(tight,maxcycle=500,xqc) '
     *           //' FChk=All Force Nosymm Charge'
      write(4,'(a80)') temp3

      write(4,1002)

C  figure out charge and multiplicity for the smaller cluster
C  for version 1.0 all multiplicities must be the same.
      call chgmlt(tfid,cm,chrg,mlt)
      write(4,*)
      write(4,1012) chrg, mlt

C  write the geometries of the fragment cluster to the file
      do i = 1, 3
        if (tfid(i) .ne. 0) then
         call ggeom(tfid(i),geom,fragid,asym,afrag,fgeom)
         do j = 1, afrag(tfid(i))
           write(4,1014) asym(fragid(tfid(i),j)), fgeom(j,1),
     *             fgeom(j,2), fgeom(j,3)


         end do
        end if
      end do
      write (4,*)

C  if specifying a general basis set write the basis set info here
      if (bkey(1:3) .eq. 'gen') then
         do i = 1, nbasis
             write(4,'(a80)') basis(i)
         end do
         write(4,*)
       end if    
      
C  write the coorinates and charges for the background charges
      do i = 1, nfrg
       if ((i.ne.tfid(1)).and.(i.ne.tfid(2)).and.(i.ne.tfid(3))) then
         do j = 1, afrag(i) 
          id = fragid(i,j)
          call gbgq(id,bgq,bq,nbgq,asym)
          write(4,1016) geom(id,1), geom(id,2), geom(id,3), bq
         end do
       else
       end if
      end do 
        
      write(4,*)
         
           
      go to 9999   

 1000 write(2,1010)  infile
      close(4)
      stop 'ghooks 1'
 1001 format('%mem=',a20)
 1002 format(/,'title')
 1003 format(4a,2(3x,a))
 1010 format (3x, 'Error writting input file' , a20) 
      
 1012 format (2(i2))
 1014 format(a3,3(f15.8)) 
 1015 format(3(f15.5),a15)
 1016 format(3(f12.8),'  ',a20)

 9999 close(4)

      command = 'perl g09shuttle.pl ' // temp2(tbeg:tend)
     *            //' '// eec(ebeg:eend) // ' 1  >> energy'
      call system(command)

      return

      end


C**********************************************************************
C
C    G09OUTG
C
C**********************************************************************
C
C   created : April 12 2007, by E. E. Dahlke
C   revised : May 11, 2009   by Hai Lin
C             Feb 17, 2011   by Hannah Leverentz
C             March 2, 2011  by Hannah Leverentz
c
C   purpose : to pull the energies, gradients, and dipole moments
C             out of the *.eemb file (* = 1, 2, 3)
C   revision: pass nfrg to subroutine getbqap
C             2 Mar 2011 revised to allow EE-MB population/charge analyses
C            17 Feb 2011 revised to increase size of aplst array

      subroutine g09outg(tof,nf,energy,eflg,etype, natms,grad, dip,
c  Add by Hannah Leverentz, 3/2/2011
c    *                        fragid,frglst,afrag,nfrg)
     *                        fragid,frglst,afrag,nfrg,qqm)
c  End by Hannah Leverentz, 3/2/2011

      implicit none
      include 'common.inc'
      integer nf, i, istrt, ierr, j, ii, natms, nl, ng, l , k
c
c Added by Hai Lin, 5/11/2009
c
      integer nfrg
c
c Also, this seems to be a bug, now change it
c
c     integer  aplst(maxapf*9), ibeg, ifin, iquit
c
c  Added by Hannah Leverentz, 2/17/2011
c
c   We need to know how the energy of each fragment changes with respect
c   to changes in the coordinates of the external point charges as well
c   as how it changes with changes in the coordinates of the fragment's
c   nuclear coordinates; thus, the aplst array must be of dimension
c   maxat*3 instead of maxapf*3:
c
c     integer  aplst(maxapf*3), ibeg, ifin, iquit
      integer  aplst(maxat*3),ibeg,ifin,iquit
c
c  End by Hannah Leverentz, 2/17/2011
c
c End by Hai Lin, 5/11/2009
c
      integer eflg(maxe), fragid(maxfrag,maxapf), afrag(maxfrag)
      integer frglst(maxlist,3), na
      character*30 etype(maxe)
      character*2  tof                                  !0213HL12  changed tof from a 6-char variable to a 2-char variable
      character*10 flnm                                 !0213HL12
      character*80  ci                                  !0213HL12
      integer fstrt, fbeg, fend                         !0213HL12
      character*80 string, temp
      double precision energy(maxe), cenergy, senergy, tgrad, tmpdip
      double precision grad(3*maxat), dip(3)
c  Add by Hannah Leverentz, 3/2/2011
      double precision qqm(maxat)
c  End by Hannah Leverentz, 3/2/2011

C  open the *.eemb file

c     open(unit=3,file=tof,status='old',form='formatted',err=9999)     !0213HL12 commented out this line

C  and set all energies to zero, and clear array with the flags
      do i = 1, maxe
            energy(i) = 0.0d0
            eflg(i) = 0
      end do

C  zero the temporary dipole
      do i = 1, 3
        dip(i) = 0.0d0
      end do

c  Add by Hannah Leverentz, 3/2/2011
c
C  zero the QM partial charges array for the current oligomer type
      do i = 1, maxat
        qqm(i) = 0.0d0
      end do
c
c  End by Hannah Leverentz, 3/2/2011

C  zero the temporary gradient
      do i = 1, 3*natms
          grad(i) = 0.0d0
      end do

C  store the data for each data type
      i = 1
      istrt = 1
      do while (i .le. nf)
         write(ci,*) i                                                          !0213HL12
         fstrt = 1                                                              !0213HL12
         call fchar(ci,fstrt,ierr)                                              !0213HL12
         fbeg = fstrt                                                           !0213HL12
         call fspace(ci,fstrt,ierr)                                             !0213HL12
         fend = fstrt - 1                                                       !0213HL12
         flnm = ci(fbeg:fend) // tof // '.eemb'                                 !0213HL12
         open(unit=3,file=flnm,status='old',form='formatted',err=9999)          !0213HL12
         ng = 1
         read(3,'(a80)',err=9910,end=9998) string
         call fspace(string,istrt,ierr)
         read(string(1:istrt),*) ii
c
c ii-th calculation for a given order MB expansion
c
         na = 0
         call getap(ii,fragid, frglst, afrag, aplst,na)
c
c Added by Hai Lin, 5/11/2009
c
c   Find the location of background charge gradient in the oniom FChk file
c   For normal QM (including using the charge keyworda), the gradients are
c   available only for the QM atoms, as charges not counted as atoms.
c   For oniom and mm jobs, the gradients are given for all atoms.
c
         if (na .lt. natms*3) then
           call getbqap(ii,fragid, frglst, afrag, aplst,na,natms,nfrg)
c
c   Now set na to natms because we are reading the gradients for the whole
c   system.
c
           na = natms*3
         endif
c
c End by Hai Lin, 5/11/2009
c
c
c na is the total number of gradient components  = natms*3
c
         if (mod(na,5) .eq. 0) then
                nl = na/5
         else
                nl = int(na/5) + 1
         end if
c
c nl is how many lines for grandients
c
         read(3,'(a80)',err=9910,end=9998) string
         read(string(38:50),'(f21.10)') senergy
c  revised 6/18/09 by EDS
c        j = 0
         do while (string(1:12) .ne. 'Total Energy')
c           j = j + 1
            read(3,'(a80)',err=9910,end=9998) string
c           etype(j) = string(1:30)
            if (string(1:10) .eq. 'SCF Energy') then
               j = 1
               eflg(j) = 0
               etype(j) = string(1:30)
            elseif (string(1:10) .eq. 'MP2 Energy') then
               j = 2
               eflg(j) = 1
               etype(j) = string(1:30)
            elseif (string(1:10) .eq. 'MP3 Energy') then
               j = 3
               eflg(j) = 1
               etype(j)= string(1:30)
            elseif (string(1:11) .eq. 'MP4D Energy') then
               j = 4
               eflg(j) = 1
               etype(j) = string(1:30)
            elseif (string(1:12) .eq. 'MP4DQ Energy') then
               j = 5
               eflg(j) = 1
               etype(j) = string(1:30)
            elseif (string(1:13) .eq. 'MP4SDQ Energy') then
               j = 6
               eflg(j) = 1
               etype(j) = string(1:30)
            elseif (string(1:27) .eq.
     *                         'Cluster Energy with Triples') then
               j = 8
               eflg(j) = 1
               etype(j) = string(1:30)
            elseif (string(1:12) .eq. 'Total Energy') then
               j = 9
               eflg(j) = 1
               etype(j) = string(1:30)
            else
               j = 7
               eflg(j) = 1
               etype(j) = string(1:30)
            endif
            read(string(50:71),'(E22.15E2)') cenergy
            energy(j) = energy(j) + cenergy - senergy
c           eflg(j) = 1
c
c Revised by Hai Lin, 5/11/2009
c
c   Since we use oniom, the scf energy is meaningless and should not be shown
c
c            if (string(1:12) .eq. 'Total Energy') then
c            if (string(1:10) .eq. 'SCF Energy') then
c
c End by Hai Lin, 5/11/2009
c
c                eflg(j) = 0
c            end if
c  end revision 6/18/09 EDS
         end do
         read(3,'(a80)',err=9910,end=9998) string
         read(3,'(a80)',err=9910,end=9998) string
C  find the dipole moments
         istrt = 1
         ierr = 0
         do j = 1, 3
             call fchar(string, istrt, ierr)
             ibeg = istrt
             call fspace(string, istrt, ierr)
             ifin = istrt - 1
             temp = string(ibeg:ifin)
             read(unit=temp, fmt=*) tmpdip
             dip(j) = dip(j) + tmpdip
         end do
c  Add by Hannah Leverentz, 3/2/2011
c
C  read in the QM partial charges obtained from the oligomer wave function
c
         call readqqm(ii,frglst,fragid,qqm)
c
c  End by Hannah Leverentz, 3/2/2011
        read(3,'(a80)',err=9910,end=9998) string

C  put the gradient in the right part of the gradient array based on the
C  fragment number and the list of atoms for each fragment
C  read each line until all lines containing gradient information have been
C  read in, put them all in a temporary array for right now, will move to the
C  permanent arrays once they have all been read in

        j = 1
        do while (j .le. nl)
          read(3,'(a80)',err=9910,end=9998) string
          istrt = 1
          ierr = 0
          ifin = 1
          ibeg = 1
          l = 1
          do while (l .le. 5)
             if (ng .le. na) then
               call fchar(string, istrt, ierr)
               ibeg = istrt
               call fspace(string, istrt, ierr)
               if (ifin .eq. 80) then
                 ifin = istrt - 1
               else
                 ifin = istrt
               end if
               temp = string(ibeg:ifin)
               read(unit=temp, fmt=*) tgrad
               grad(aplst(ng)) = grad(aplst(ng)) + tgrad
               ng = ng + 1
               l = l + 1
             else
                  l = 6
             end if
          end do
          j = j + 1
         end do

      close(3)                                               !0213HL12
      i = i + 1
      end do

      return                                                    !0213HL12

 9998 close(3)
c     return                                                    !0213HL12 commented out this line
      stop 'g09cc 3'                                            !0213HL12

 9999 write(2,1900) flnm					!0213HL12 changed 'tof' to 'flnm'
      stop 'g09cc 1'
 1900 format(/3x,'Error : could not open file ', a12)		!0213HL12 changed 'a6' to 'a12'

 9910 write(2,1800) flnm, nf					!0213HL12 changed 'tof' to 'flnm'
      stop 'g09cc 2'
 1800 format(/3x, 'Error: end of file ', a12, ' reached but ', i5,   !0213HL12 changed 'a6' to 'a12'
     *       ' energies not yet read')

      end

C**********************************************************************
C              
C    GETAP     
C              
C**********************************************************************
C               
C   created : April 12th, 2007 by E. E. Dahlke
C   revised : Feb   17th, 2011 by H. R. Leverentz
C   purpose : given a fragment combination, find the corresponding
C             locations in the gradient array
             
      subroutine getap(fn,fragid, frglst, afrag, aplst,np)
          
      implicit none
      include 'common.inc'
      integer i, j, k, np, fn, fragid(maxfrag,maxapf), afrag(maxfrag)
c  Added by Hannah Leverentz, 2/17/2011
c     integer frglst(maxlist,3), aplst(maxapf*3), na
      integer frglst(maxlist,3), aplst(maxat*3), na
c  End by Hannah Leverentz, 2/17/2011

      np = 1
C  zero array
c  Added by Hannah Leverentz, 2/17/2011
c     do i = 1, 3*maxapf  
      do i = 1, 3*maxat
c  End by Hannah Leverentz, 2/17/2011
          aplst(i) = 0
      end do

      do i = 1, 3
         if (frglst(fn,i) .ne. 0) then
            do j = 1, afrag(frglst(fn,i))
               do k = 1, 3
                  aplst(np) = 3*(fragid(frglst(fn,i),j)-1) + k
                  np = np + 1
               end do
            end do
          end if
       end do

      np = np - 1

      return
      end
C**********************************************************************
C
C    FNDSTRTG
C
C**********************************************************************
C
C   created : April 14, 2007 by E. E. Dahlke (modified from FNDSTRT)
C   revised : 
C   purpose : to find the last completed gradient calculation in a
C 		restart

      subroutine fndstrtg(nf, donef,nftc)

      implicit none
      include 'common.inc'

      integer nf, i, j, istrt, ierr, ii, nftc
      integer donef(maxlist)
      character*80 string, efile

      efile = 'energy'

C  open the energy file

      open(unit=5,file=efile,status='old',form='formatted',err=9999)

C  read in the data and find out which fragments have been run.
C  donef(i) = 1 if the calculation for fragment i is done
C  donef(i) = 0 if the calculation for fragment i is not done
C  nftc is the number of fragments that have to be completed

      do i = 1, maxlist
        donef(i) = 0
      end do

      i = 1
      istrt = 1
      nftc = nf
      read(5,'(a80)',end=9998) string
      do while (i .le. nf)
         call fspace(string,istrt,ierr)
         read(string(1:istrt),*) ii
         donef(ii) = 1
         nftc = nftc - 1
         read(5,'(a80)',end=9998) string
         do while (string(1:18) .ne. 'Cartesian Gradient')
            read(5,'(a80)',end=9998) string
         end do
         read(5,'(a80)',end=9998) string
         do while (string(1:1) .eq. ' ')
            read(5,'(a80)',end=9998) string
         end do
         i = i + 1
      end do

      close(5)

 9998 close (5)
      return

 9999 write(2,1900)
      stop 'fndstrt 1'

 1900 format (/3x, 'Error could not open file energy')


      
      end
            
C**********************************************************************
C
C    GETBQAP
C
C**********************************************************************
C
C   created : May 11, 2009 by Hai Lin, Based on subroutine GETAP
C   revised : Feb 17, 2011 by Hannah Leverentz
C   purpose : given a fragment combination, find the corresponding
C             locations for the background charges in the gradient array
c

      subroutine getbqap(fn,fragid, frglst, afrag, aplst,np,natms,nfrg)
c
c The locations for the background charges are returned in the array
c aplst, where the first np elements are the locations for the QM atoms
c

      implicit none
      include 'common.inc'

      integer natms, id, nfrg
      integer i, j, k, np, fn, fragid(maxfrag,maxapf), afrag(maxfrag)
c  Added by Hannah Leverentz, 2/17/2011
c     integer frglst(maxlist,3), aplst(maxapf*3), na
      integer frglst(maxlist,3), aplst(maxat*3), na
c  End by Hannah Leverentz, 2/17/2011
      integer ilfrglst, ilfrag, ilfragid

      np = np + 1

c debug
c      write(*,*) 'bqap!'

      do i = 1, nfrg
        if ((i.ne.frglst(fn,1)) .and. (i.ne.frglst(fn,2))
     *    .and.(i.ne.frglst(fn,3))) then
          do j = 1, afrag(i)
            id = fragid(i,j)
            do k = 1, 3
              aplst(np) = 3*(id-1) + k
c debug
c            write(*,*) 'np= ', np, 'id= ', id
              np = np + 1
            end do
          end do
        else
        end if
      end do

      np = np - 1

      return
      end

