c*****************************************************************************
C
C	MAIN
C
C******************************************************************************
C
C	this is the driver program for the eemb program
C
C 	Created: September 20, 2006 by Erin E Dahlke
C	Revised: May 11, 2009       by Hai Lin
C                june 12, 2009 by Erin Dahlke Speetzen
C                March 9, 2011  by Hannah Leverentz
C
C	Purpose: Driver program for the EEMB script
C       Revision: Add comments --- otherwise too hard to read the code!
c      		  6/12/09 changes to use core and keywrds options
c                 3/09/11 revised to allow EE-MB population/charge analyses
C
C	Description:	Driver routine that opens the input and output files,
C			calls the subroutines which initiate calls to start
C			processes such as reading the input file, running the
C			calcs, etc.  Also writes output to output file.
C
      program main
  
      implicit none
      include 'common.inc'

      include 'mpif.h'
      integer rank,size,ierr
      integer src,dest
      integer status(MPI_STATUS_SIZE)


      character*20:: eembin, eembout
c
c eemb is the order of MB expansion
c nfrg is the number of fragments
c
      integer   natms, rstrt, eemb, nproc, ekey, gradkey, hesskey, nfrg
c
c
c fragid(i,j) is the atom index for the j-th atom in the i-th fragment
c afrag(i)    is the total number of atoms in the i-th fragment
c
      integer   fragid(maxfrag,maxapf), cm(maxfrag,2), afrag(maxfrag)
c
c nbgq is the number of background charge
c eflg(i) determines whether or not the i-th level of QM energy to be shown
c         Note: the SCF energy is meaningless in g09 output or FCHK file!
c               Always look for "Total Energy" if you want to be on the
c               safe side!
c
      integer nbasis,nbgq, eflg(maxe), nt
c added/6/12/09 eds
      integer pseudo, ncore
c  end 6/12/09
      integer i, optkey
      character*80 basis(1000), title(5)
c added/6/12/09 eds
      character*80 core(1000), keywrds
c end 6/12/09
C  Add by Hannah Leverentz, 3/9/2011
      character*20 cpop
C  End by Hannah Leverentz, 3/9/2011
      integer lscreen
      integer leembce                                                   !1115HL11 whether using EE-MB-CE method

c
c bgq(i,j) is the charge value given in characters for the i-th charge
c
      character*20 bgq(maxat,5), esm, method,mem, bkey
      integer nmmpp,mmpseudo        !1110HL11
      character*80 allmmpp(1000)    !1110HL11
c
c etype(i) is the name of the i-th level QM energy such as mp2, mp4, ...
c
      character*30 etype(maxe)
      double precision eepa(maxe), ee3b(maxe), v1(maxe), v2(maxe)
      double precision v3(maxe), gpa(3*maxat), g3b(3*maxat)
      double precision gv1(3*maxat), gv2(3*maxat), gv3(3*maxat)
      double precision dippa(3), dip3b(3), dv1(3), dv2(3), dv3(3)
      double precision hpa(maxhess), h3b(maxhess)
      double precision hv1(maxhess), hv2(maxhess), hv3(maxhess)

      call MPI_INIT(ierr)
      call MPI_COMM_RANK(MPI_COMM_WORLD, rank, ierr)
      call MPI_COMM_SIZE(MPI_COMM_WORLD, size, ierr)
      IF (rank.eq.0) THEN

C  open the input and output files

      eembin = 'eemb.inp'
      eembout = 'eemb.out'
      open(unit=1,file=eembin,status='old',form='formatted',err=1000)
      open(unit=2, file=eembout, status='unknown', form='formatted', 
     *      err=2000)
	 
C  print the EEMB header to the output file
      call eehed

C  set the default variables
      call defalt(rstrt,eemb,ekey,gradkey,hesskey,optkey,title,esm,
c revised 6/12/09 by eds
     *            mem,method,basis,bkey,nbasis,nt,cm,keywrds,core,
C  Add by Hannah Leverentz, 3/9/2011
c    *            pseudo,ncore)
     *            pseudo, ncore, cpop,lscreen,allmmpp,nmmpp,mmpseudo,           !1110HL11 add allmmpp,nmmpp,mmpseudo
     *            leembce)                                                      !1115HL11 add leembce
C  End by Hannah Leverentz, 3/9/2011

C  read the input from eemb.in  
      call readin(natms, rstrt,eemb, fragid, afrag, cm, ekey, 
     *            gradkey, hesskey, optkey, geom, asym, title, bgq, 
     *            basis, nfrg, esm, method, nbasis,mem,nbgq,nt,bkey,
c  added 6/12/09 by eds
C  Add by Hannah Leverentz, 3/9/2011
c    *            keywrds, core,pseudo, ncore)
     *            keywrds, core,pseudo, ncore, cpop,lscreen,
     *            allmmpp,nmmpp,mmpseudo,leembce) 		     !1110HL11; 1115HL11 add leembce

C  End by Hannah Leverentz, 3/9/2011
c  end 6/12/09

C  write a summary of the input to the output file
      call wsum(natms, rstrt,eemb, fragid, afrag, cm, ekey,
     *            gradkey, hesskey, optkey,geom, asym, title, bgq, 
     *            basis,esm, method,mem,bkey,nbasis,nt,nbgq,nfrg,
C  Add by Hannah Leverentz, 3/9/2011
c    *            keywrds, core, pseudo, ncore)
     *            keywrds, core, pseudo, ncore, cpop,lscreen,
     *            allmmpp,nmmpp,mmpseudo,leembce)                   !1110HL11; 1115HL11 add leembce
C  End by Hannah Leverentz, 3/9/2011

C  call either the hessian, gradient or energy hooks.  Call only one - the
C  most expensive one, all of the less expensive results will be calculated
      if (optkey .eq. 1) then
         ierr = 0
         call gau_ext_opt(natms,cm,asym,geom,ierr,nfrg,afrag,fragid,
C  Add by Hannah Leverentz, 3/9/2011
c    *                    eemb)
     *                    eemb, cpop,lscreen,allmmpp,nmmpp,mmpseudo)                !1110HL11 add allmmpp,nmpp,mmpseudo
C  End by Hannah Leverentz, 3/9/2011

      elseif (hesskey .eq. 1) then
         call hhooks(natms,rstrt,eemb,fragid,cm,afrag, nfrg,
     *                  basis,title,bgq,esm,method,geom,asym,nbasis,
c revised 6/12/09 by eds
     *                  mem,nbgq,bkey,ekey,gradkey,hesskey,keywrds,
C  Add by Hannah Leverentz, 3/9/2011
c    *                  core,pseudo,ncore)
     *                  core,pseudo,ncore,cpop, lscreen,
     *                  allmmpp,nmmpp,mmpseudo,leembce)                     !1110HL11; 1115HL11 add leembce
C  End by Hannah Leverentz, 3/9/2011
C end 6/12/09 eds
      elseif (gradkey .eq. 1) then
         call ghooks(natms,rstrt,eemb,fragid,cm,afrag, nfrg,
     *                  basis,title,bgq,esm,method,geom,asym,nbasis,
c  revised 6/12/09 by eds
     *                  mem,nbgq,bkey,ekey,gradkey,hesskey,keywrds,
C  Add by Hannah Leverentz, 3/9/2011
c    *                  core,pseudo,ncore)
     *                  core,pseudo,ncore,cpop, lscreen,
     *                  allmmpp,nmmpp,mmpseudo,leembce)                    !1110HL11; 1115HL11 add leembce
C  End by Hannah Leverentz, 3/9/2011
c end 6/12/09 eds
      elseif (ekey .eq. 1) then
         call ehooks(natms,rstrt,eemb,fragid,cm,afrag, nfrg,
     *                  basis,title,bgq,esm,method,geom,asym,nbasis,
c  revised 6/12/09 by eds
     *                  mem,nbgq,bkey,ekey,gradkey,hesskey,keywrds,
C  Add by Hannah Leverentz, 3/9/2011
c    *                  core,pseudo,ncore)
     *                  core,pseudo,ncore,cpop, lscreen,
     *                  allmmpp,nmmpp,mmpseudo,leembce)                    !1110HL11; 1115HL11 add leembce
C  End by Hannah Leverentz, 3/9/2011
c  end 6/12/09 by eds
      end if

      write(2,1500)
   
      goto 9999
 1000 print*, 'ERROR: cannot open input file.' 
      close(1)
	  stop 'main 1'

 2000 print*, 'ERROR: cannot open output file.'
      close(2)
	  stop 'main 2'

 1500 format(/,/,/,80('_'),/,'END OF OUTPUT FILE')


 9999 close(1)
      close(2)

      ELSE                     ! More PC than master and slave ;)
        CALL WORKER
      ENDIF


      IF (SIZE.GT.1) THEN
      DO I=1,SIZE-1
        CALL MPI_Send(0,1,MPI_INTEGER,I,0,MPI_COMM_WORLD,ierr)
      ENDDO
      ENDIF


      CALL MPI_FINALIZE(ierr)

      end
