C**********************************************************************
C
C      Gau_ext_opt
C
C**********************************************************************
C    Gau_ext_opt is an  which uses Gaussian's external keyword
C    and Gaussian's optmizer
C
C	MBPAC <--> G09 <--> Gau_External <--> MBPAC
C
C   Created: May 23, 2007 by E. E. Dahlke (modified from Multilevel 4.2)
C   Revised: March 9, 2011 by H. R. Leverentz
C            March 30, 2011 by H. R. Leverentz
C   Purpose:  To perform optimization using Gaussian's optimizer
C
C

      subroutine gau_ext_opt(natms,cm,asym,geom,ierr,nfrg,afrag,
C  Add by Hannah Leverentz, 3/9/2011
c    *                       fragid, eemb)
     *                       fragid, eemb, cpop)
C  End by Hannah Leverentz, 3/9/2011

      implicit none
      include 'common.inc'

      integer ierr, i, j, k, n3tm, natms, cm(maxfrag,2),eemb
      integer nfrg, afrag(maxfrag), fragid(maxfrag,maxapf)
      double precision energy
      double precision grad(maxat,3)
C  Add by Hannah Leverentz, 3/30/2011
      double precision outgeom(3*maxat)
C  End by Hannah Leverentz, 3/30/2011
      character*80 command
      character*16 ogeom(3*maxat)
C  Add by Hannah Leverentz, 3/9/2011
      character*20 cpop
C  End by Hannah Leverentz, 3/9/2011
    
      n3tm = 3*natms

      open (unit=7,status='unknown',file='extopt.inp', err=9991)
      call og09(natms,cm,geom,asym,nfrg)                        ! 1110HL11  added nfrg
      command = 'perl g09-ex-shuttle.pl extopt.inp extopt.out'
      call system(command)
      call g09outo(natms,ierr,ogeom)

C  Add by Hannah Leverentz, 3/30/2011
c 
c     Convert the output geometry from a character variable into
c     a double precision variable and then convert the coordinates
c     from bohr to angstroms:
c 
      k = 0
      do i=1,natms
        do j=1,3
          k = k + 1
          read(ogeom(k),*) outgeom(k)
          outgeom(k) = outgeom(k)/ang2br
        end do
      end do
c 
C  End by Hannah Leverentz, 3/30/2011

      write(2,1100)

      if (ierr .eq. 1) then
         go to 7000
      else 
         write(2,1200)
      end if

C  call the subroutine to write energy and gradient information

c Add by Hannah Leverentz, 3/9/2011
c     call wg09g(eemb,nfrg,natms,afrag,fragid)
      call wg09g(eemb,nfrg,natms,afrag,fragid,asym,cpop)   
c End by Hannah Leverentz, 3/9/2011
  
      write(2,1400)
      do  i = 1, natms
C  Add by Hannah Leverentz, 3/30/2011
c         write(2,1500) asym(i), (ogeom(3*(i-1)+j),j=1,3)
          write(2,1500) asym(i), (outgeom(3*(i-1)+j),j=1,3)
C  End by Hannah Leverentz, 3/30/2011
      end do
      write(2,*)
      write(2,1000)
      write(2,1000)
      
 

 1000  format(80('-'))
 1100  format(/2x,'External Optimization with Gaussian',
     *         a,//80('-')//)
 1200  format(/80('*'),/,t30,'OPTIMIZATION SUCCESSFUL',/80('*'))
C  Add by Hannah Leverentz, 3/30/2011
c 1400  format(/2x,'Optimized Geometry:',/)
c 1500  format(5x,a3,'     ',a16,'     ',a16,'     ',a16)
 1400  format(/2x,'Optimized Geometry (in Angstroms):',/)
 1500  format(5x,a3,3(5x,f16.8))
C  End by Hannah Leverentz, 3/30/2011

      return

 9991  write(2,*) 'Error creating Gaussian 09 input file'
 7000  write (2,'(/a)') 'ERROR: External Optimization failed.'
       end


c******************************************************************************
C
C      OG09
C
c******************************************************************************
C
C      created : May 23, 2007 by E. E. Dahlke (modified EG09)
C      revised : May 11, 2009 by Hai Lin
c
C      purpose :  write the G09 input files for external optimization
C      revision: add g09 keyword gdiis to opt (good is for floppy systems.)
C

      subroutine og09(natms,cm,geom,asym,nfrg)                  ! 1110HL11  added nfrg

      implicit none
      include 'common.inc'

      integer   natms
      integer   nfrg                                            ! 1110HL11
      integer   charge, mult
      integer   i,j
      integer   cm(maxfrag,2)
      character*80 temp
c
c Added by Hai Lin, 5/11/2009
c
c      temp = '# external="Gau_External_2" opt fchk=all'
C  Add by Hannah Leverentz, 4/04/2011
c     temp = '# external="Gau_External_2" opt=gdiis fchk=all'
      temp = '# external="Gau_External_2" opt=(rfo,cartesian,'
     &        // ' maxcycles=100) fchk=all'
C  End by Hannah Leverentz, 4/04/2011
c
c End by Hai Lin, 5/11/2009
c

      write(7,'(a80)') temp
      write(7, 1000)


      charge = 0
C  find the charge for the whole structure
      do i = 1, nfrg                                            ! 1110HL11  changed "1, natms" to "1, nfrg"
        charge = charge + cm(i,1)
      end do

      mult = cm(1,2)                                            ! 1110HL11  changed "cm(i,2)" to "cm(1,2)"

      write(7,1100) charge, mult

C  write the geometry

      do i = 1, natms
         write(7,1200) asym(i), (geom(i,j), j = 1, 3)
      end do

      write(7,*)

      close(7)

 1000 format(/,'title')
 1100 format(/,2(i2))
 1200 format(a3,3(f15.8))

      return

 9991 close(7)

      end

c*****************************************************************************
C
C      g09outo
C
c*****************************************************************************
C
C     Created : May 23, 2007 by E. E. Dahlke
C     Revised : 
C     Purpose : To find the optimized coordinates from 
C		an external optimization and put them in an array

       subroutine g09outo(natms,ierr,ogeom)

       include 'common.inc'
       integer natms, ierr, i, etr, ctr, gstrt, gend
       character*80 string, case
       character*16 ogeom(3*maxat)

C  open output file and see if optimization completed

      open (unit=9,file='extopt.out',status='old',err=9930)
      read(9,'(a80)',end=9920) string
      string = case(string)
C  Add by Hannah Leverentz, 3/30/2011
c     do while (string(1:24) .ne. ' optimization completed.')
      do while (string(1:23) .ne. ' optimization completed')
C  End by Hannah Leverentz, 3/30/2011
         read(9,'(a80)',end=9920) string
         string = case(string)
      end do

      close(9)

C open the formated checkpoint file and get the geometry

      open(unit=10,file='Test.FChk',status='old',err=9900)
      read(10,'(a80)',end=9910) string
      string = case(string)

      do while ( string(1:29) .ne. 'current cartesian coordinates')
         read(10,'(a80)', end=9910) string
         string = case(string)
      end do


      ctr = 1
      do while (ctr .le. 3*natms)
         read(10,'(a80)', end=9910) string
         i = 0
         do while (ctr .le. 3*natms .and. i .lt. 5)
           gstrt = i*(16)+2
           gend = (i+1)*(16)
           ogeom(ctr) = string(gstrt:gend)
           ctr = ctr + 1
           i = i + 1
         end do
       end do
       close (10)

       return

 9900  write(2,*) 'Error opening Gaussian09 checkpoint file.'
      stop 'g09outo 1'
 9910  write(2,*) 'Error reading Gaussian09 checkpoint file.'
      stop 'g09outo 2'
 9920  write(2,*) 'Error opening Gaussian09 output file.'
      stop 'g09outo 3'

 9930  ierr = 1


       end 
