#!/usr/bin/perl -w

# Written by Erin Dahkle, 2007
# Revised by Hai Lin, 5/11/2009
# Revised by Hannah Leverentz, 3/8/2011

#  this perl script is designed to set up the directory
#  structure for a g09 job, run the input file and extract 
#  energy from the formatted checkpoint file

# Revision adds the 4th arguement, which indicates oniom & mm runs
# are needed if its value is 0, or not needed if 1.      

#  usage:
# 
#	perl g09shuttle.pl filename eemb <0,1,2> <0,1,2> itotalqm cpop imode    #1115HL11
#
#  where 
#     filename is the gaussian input filename without the .g09 extension
#     eemb is the number of total fragments
#     3rd arguement: 0 = energy, 1 = gradient, 2 = frequency calculation 
#     4th arguement: 0 = oniom + mm calculations, 1 = normal calculation
#                     2 = screened charge calculations using PP     #0603BW11
#     itotalqm is the number of atoms in the current fragment
#     cpop tells which type of population analysis is to be performed
#     imode: 0 = regular EE-MB calculation, 1 = EE-MB-CE calculation	#1115HL11

use Cwd;

                       #############################
                       #                           #
######################## User Needs to Change Here ############################
                       #                           #
                       #############################

#  these variables must be changed to match the user's computer system

$g09 = '/soft/koronis/gaussian/g09.c01.MNGFM6.2';

$scratchdir="/scratch/uvdev1/hannahl/mbpac/testruns";


                             ##################
                             #                # 
############################## Main Procedure ##################################
                             #                #
                             ##################

#======================== Prepare files ==============================

# print "inormal $ARGV[3]\n";
$inormal = $ARGV[3];
$imode   = $ARGV[6];			#1115HL11

$home = cwd;
$name = "$ARGV[0]\_$ARGV[1]";  			# 0516HL12 changed $ARGV[0] to $ARGV[0]\_$ARGV[1]
$temp = "$ARGV[0]";		                # 0516HL12 changed $ARGV[0]\_$ARGV[1] to $ARGV[0]

$scratchdir=$scratchdir . "/$name";		# 0516HL12 changed $temp to $name
mkdir "$scratchdir" || die "Failed to create directory $scratchdir!!\n";;

$ENV{"GAUSS_EXEDIR"}="$g09/g09";
$ENV{"g09root"}=$g09;
$ENV{"GAUSS_SCRDIR"}="$scratchdir";

if ($inormal == 0) {                       #061611 
  system("mv mm$name\.g09 $scratchdir");
}
system("mv $name\.g09 $scratchdir");

chdir $scratchdir;

#========================= Step 1: MM calculations =========================

#--------------------------------------------------------------------------
# 1.1 Do an MM calculations and save the fchk files as mmTest.FChk.
#     Get the self-energy as Total energy from mmTest.FChk"
 
if ($inormal == 0) {
  system("$g09/g09/g09 $scratchdir\/mm$name\.g09 $scratchdir\/mm$name\.out");    
  system("mv $scratchdir\/Test.FChk $scratchdir\/mmTest.FChk");                  
  $mmcharge = `grep 'Total Energy' mmTest.FChk`;
  @mmcharge = split(' ', $mmcharge);
#  $charge = " Self energy of the charges = $mmcharge[3]\n";
  $charge =  $mmcharge[3];
} elsif ($inormal == 2) {                            #0601BW11 do MM calculations using perl and get results
  system("mv $home\/mmcorr$name\.out $scratchdir\/mmcorr$name\.out");        # 0516HL12 changed $ARGV[0] to $name
  $mmcharge = `grep 'mm correction' mmcorr$name\.out`;     #0603BW11 to be consistent, use -enercorr;  0516HL12 changed $ARGV[0] to $name
  @mmcharge = split(' ', $mmcharge);
  $charge =  $mmcharge[3];
}

#---------------------------------------------------------------------------
# 1.2 Read in MM gradient or Hessians

if ($inormal == 0) {
  open(TMP, 'mmTest.FChk') || die "Cannot open mmTest.FChk for $name!!\n";	# 0516HL12 changed $temp to $name
  @contents = <TMP>;
  close (TMP);

  for $i (0..$#contents) {
    if ($contents[$i] =~m/^Dipole Moment/) {
        $mmdipole = $contents[$i+1];
    } elsif ($contents[$i] =~m/^Cartesian Gradient/ && $ARGV[2] >= 1) { 
        @mmgrad = &get_grad(\$i, \@contents);
    } elsif ($contents[$i] =~m/^Cartesian Force Constants/ && $ARGV[2] == 2) {
        @mmhess = &get_hess(\$i, \@contents);
    }
  }

#---------------------------------------------------------------------------
# 1.3 Write MM dipole, gradient, and hessian (for debug only)

#  if ($ARGV[2] == 1) {
#     print "MM Dipole Moment\n";
#     print $mmdipole;
#     for $j (0..$#mmgrad) {
#       print "$mmgrad[$j]\n";
#     }
#  } elsif ( $ARGV[2] == 2)  {
#     print "MM Dipole Moment\n";
#     print $mmdipole;
#     for $j (0..$#mmgrad) {
#        print "$mmgrad[$j]\n";
#     }
#     for $j (0..$#mmhess) {
#        print "$mmhess[$j]\n";
#     }
#  }

} elsif ($inormal == 2) {
## read in MM gradient or Hessians
  open(TMP, "mmcorr$name\.out") || die "Cannot open mmTest.FChk for $name!!\n";	# 0516HL12 changed $ARGV[0] to $name
  @contents = <TMP>;
  close (TMP);

  for $i (0..$#contents) {
    if ($contents[$i] =~m/^Dipole Moment/) {
        $mmdipole = $contents[$i+1];
    } elsif ($contents[$i] =~m/^Cartesian Gradient/ && $ARGV[2] >= 1) {
        @mmgrad = &get_grad(\$i, \@contents);
    } elsif ($contents[$i] =~m/^Cartesian Force Constants/ && $ARGV[2] == 2) {
        @mmhess = &get_hess(\$i, \@contents);
    }
  }

}
 
#========================== Step 2: ONIOM or QM calculation ===============

#---------------------------------------------------------------------------
# 2.1 Only qm calculations need to grep the self-energy, which is acutally 0.

system("$g09/g09/g09 $scratchdir\/$name\.g09 $scratchdir\/$name\.out");    

if ($inormal == 1) {
  $charge = 0.0;
}

$energy = `grep Energy Test.FChk`;

if ($imode == 1) {				#1115HL11
  $hftvar = `grep "SCF Done" $name\.out`;	#1115HL11
  @hftarr = split(" ",$hftvar);			#1115HL11
  $hftvar = $hftarr[4];				#1115HL11
  $ehf = sprintf("Hartree Fock Energy                        R %26.15E\n",$hftvar);  #1115HL11
}						#1115HL11

#----------------------------------------------------------------------------
# 2.2 Get qm dipole information, gradient, hessian

open(TMP, 'Test.FChk') || die "Cannot open Test.FChk for $name!!\n";	# 0516HL12 changed $temp to $name
@contents = <TMP>;
close (TMP);

#  Add by Hannah Leverentz, 3/10/2011
#
#  For correlated wave function methods such as MP2, the Dipole Moment is not
#  printed to the Test.FChk file (because the only dipole moment computed is 
#  from the reference hartree fock wave function).  This will cause an error in 
#  MBPAC when it tries to read the dipole moments and the calculation will fail.
#  Thus, we initialize the qmdipole array to zero so that the MBPAC calculation
#  will continue even though the dipole moment will be zero.
#
  $qmdipole = "  0.00000000E+00  0.00000000E+00  0.00000000E+00 \n";
#
#  End by Hannah Leverentz, 3/10/2011

for $i (0..$#contents) {
	if ($contents[$i] =~m/^Dipole Moment/) {
		$qmdipole = $contents[$i+1];
	} elsif ($contents[$i] =~m/^Cartesian Gradient/ && $ARGV[2] >= 1) {
		@qmgrad = &get_grad(\$i, \@contents);
	} elsif ($contents[$i] =~m/^Cartesian Force Constants/ && $ARGV[2] == 2) {
		@qmhess = &get_hess(\$i, \@contents);
	}
}

#  Add by Hannah Leverentz, 3/2/2011
#
#  We need to get the QM charges (obtained from charge analysis of the oligomer
#   wave function) from the regular output file because they are not printed
#   in the Test.FChk file when an ONIOM calculation is run.
#
$itotalqm = $ARGV[4];
$cpop = $ARGV[5];
if ($cpop eq "mk") {
  $popmatch = "Charges from ESP fit";
}
elsif ($cpop eq "chelpg") {
  $popmatch = "Charges from ESP fit";
}
elsif ($cpop eq "mulliken") {
  $popmatch = "Mulliken atomic charges";
}
else {
  $itotalqm = 0;
}
$outfile  = "$name\.out";
#
undef(@contents);
open(TMP,"<$outfile") || die "Cannot open $outfile";
@contents = <TMP>;
close(TMP);
chomp(@contents);
#
$i = 0;
$flag = 0;
while (($i <= $#contents)&&($flag == 0)&&($itotalqm != 0)) {
  if ($contents[$i] =~ m/$popmatch/) {
    $flag = 1;
    @qmchrg = &get_chrg(\$i,\@contents,\$cpop,\$itotalqm)
  }
  else {
    ++$i;
  }
}
#
#  End by Hannah Leverentz, 3/2/2011


#=================== Step 3: Output the combined data ===============

#----------------------------------------------------------------------
# 3.1 Write self-energy and energy

chdir $home || die "cannot change to the home directory $home!!!\n";
# system("rm -rf $scratchdir");

print "$temp\n";				# 0516HL12 changed $name to $temp
print " Self energy of the charges =";
printf "%21.10f\n", $charge;
if ($imode == 1) {                              #1115HL11
  print $ehf;					#1115HL11
}                                               #1115HL11
print $energy;

#---------------------------------------------------------------------------
# 3.2 If normal calculation, write QM dipole, gradient, and hessian 

if ($inormal == 1) {

#  Add by Hannah Leverentz, 3/2/2011
#
# Always print the dipole moment:
  print "Dipole Moment\n";
  print $qmdipole;
#
# Always print the QM charges:
  printf "Atomic partial charges   N= %10i \n",$itotalqm;
  if ($itotalqm > 0) {
    for $i (0..$itotalqm-1) {
      printf " %15.8f \n",$qmchrg[$i];
    }
  }
#
#  End by Hannah Leverentz, 3/2/2011

  if ($ARGV[2] == 1) {
#  Modified by Hannah Leverentz, 3/2/2011
#    print "Dipole Moment\n";
#    print $qmdipole;
#  End by Hannah Leverentz, 3/2/2011
     for $j (0..$#qmgrad) {
       print "$qmgrad[$j]\n";
     }
  } elsif ( $ARGV[2] == 2)  {
#  Modified by Hannah Leverentz, 3/2/2011
#    print "Dipole Moment\n";
#    print $qmdipole;
#  End by Hannah Leverentz, 3/2/2011
     for $j (0..$#qmgrad) {
        print "$qmgrad[$j]\n";
     }
     for $j (0..$#qmhess) {
        print "$qmhess[$j]\n";
     }
  }
} elsif ($inormal == 2) {     #0601BW11
#-----------------------------
# screened charge calculations, combine gradient and hessian
# Always print the dipole moment:
    print "Dipole Moment\n";
    @tempqm = split(' ',$qmdipole);
    for $jj (0..$#tempqm) {
      printf "%16.8E", $tempqm[$jj] ;
    }
    print "\n";
#
# Always print the QM charges:
  printf "Atomic partial charges   N= %10i \n",$itotalqm;
  if ($itotalqm > 0) {
    for $i (0..$itotalqm-1) {
      printf " %15.8f \n",$qmchrg[$i];
    }
  }

} elsif ($inormal == 0) {

#---------------------------------------------------------------------
#     If oniom calculations, combine oniom and mm gradient and hessian

#  Add by Hannah Leverentz, 3/2/2011
# Always print the dipole moment:
    print "Dipole Moment\n";
    @tempqm = split(' ',$qmdipole);
    @tempmm = split(' ',$mmdipole);
    for $jj (0..$#tempqm) {
      printf "%16.8E", $tempqm[$jj] - $tempmm[$jj];
    }
    print "\n";
#
# Always print the QM charges:
  printf "Atomic partial charges   N= %10i \n",$itotalqm;
  if ($itotalqm > 0) {
    for $i (0..$itotalqm-1) {
      printf " %15.8f \n",$qmchrg[$i];
    }
  }
#
#  End by Hannah Leverentz, 3/2/2011

  if ($ARGV[2] == 1) {
#  Modified by Hannah Leverentz, 3/2/2011
# H. Leverentz commented out the following lines:
#   print "Dipole Moment\n";
#   @tempqm = split(' ',$qmdipole);
#   @tempmm = split(' ',$mmdipole);
#   for $jj (0..$#tempqm) {
#     printf "%16.8E", $tempqm[$jj] - $tempmm[$jj];
#   }
#   print "\n";
#  End by Hannah Leverentz, 3/2/2011

    print "$qmgrad[0]\n";
    for $j (1..$#qmgrad) {
      @tempqm = split(' ',$qmgrad[$j]);
      @tempmm = split(' ',$mmgrad[$j]);
      for $jj (0..$#tempqm) {
        printf "%16.8E", $tempqm[$jj] - $tempmm[$jj];   
      }
      print "\n";
    }
  } elsif ( $ARGV[2] == 2)  {
#  Modified by Hannah Leverentz, 3/2/2011
# H. Leverentz commented out the following lines:
#    print "Dipole Moment\n";
#    @tempqm = split(' ',$qmdipole);
#    @tempmm = split(' ',$mmdipole);
#    for $jj (0..$#tempqm) {
#      printf "%16.8E", $tempqm[$jj] - $tempmm[$jj];
#    }
#    print "\n";
#  End by Hannah Leverentz, 3/2/2011

     print "$qmgrad[0]\n";
     for $j (1..$#qmgrad) {
       @tempqm = split(' ',$qmgrad[$j]);
       @tempmm = split(' ',$mmgrad[$j]);
       for $jj (0..$#tempqm) {
         printf "%16.8E", $tempqm[$jj] - $tempmm[$jj];
       }
       print "\n";
     }
     print "$qmhess[0]\n";
     for $j (1..$#qmhess) {
       @tempqm = split(' ',$qmhess[$j]);
       @tempmm = split(' ',$mmhess[$j]);
       for $jj (0..$#tempqm) {
          printf "%16.8E", $tempqm[$jj] - $tempmm[$jj];
       }
       print "\n";
     }
  }
}
                             ##################
                             #                #
##############################   Subroutines  ##################################
                             #                #
                             ##################

#====================== Subroutine get_grad =========================

sub get_grad{
   my ($iref, $cref) = @_;
   my ($i, $j, $nc, @grad);
   @temp = split(' ', $$cref[$$iref]);
   chomp $$cref[$$iref];
   push @grad, $$cref[$$iref];
   $nc = $temp[4];
   if ($nc%5 == 0) { $nl = $nc/5;
   } else { $nl = int($nc/5 + 1);}
   $i = 1;
   while($i <=$nl) {
        chomp $$cref[$$iref+$i];
        push @grad, $$cref[$$iref+$i];
        $i++;
   }
   return @grad;
}

#====================== Subroutine get_hess ============================

sub get_hess{
   my ($iref, $cref) = @_;
   my ($i, $j, $nc, @hess);
   chomp $$cref[$$iref];
   push @hess, $$cref[$$iref];
   @temp = split(' ', $$cref[$$iref]);
   $nc = $temp[5];
   if ($nc%5 == 0) { $nl = $nc/5;
   } else { $nl = int($nc/5 + 1);}
   $i = 1;
   while($i <= $nl) {
        chomp $$cref[$$iref+$i];
        push @hess, $$cref[$$iref+$i];
        $i++;
   }
   return @hess;
}

# start mod_by_hl
#====================== Subroutine get_chrg ============================

sub get_chrg{
   my ($iref, $cref, $poptyp, $nqm) = @_;
   my ($i, $j, @chrg, @tline);
   if (($$poptyp eq "mk")||($$poptyp eq "chelpg")) {
     $i = $$iref + 3;
     for $j (0..$$nqm-1) {  
       undef(@tline);
       @tline = split(' ',$$cref[$i+$j]);
       $chrg[$j] = $tline[2];
     }
   }
   elsif ($$poptyp eq "mulliken") {
     $i = $$iref + 2;
     for $j (0..$$nqm-1) {
       undef(@tline);
       @tline = split(' ',$$cref[$i+$j]);
       $chrg[$j] = $tline[2];
     }
   }
   return @chrg;
}
# end mod_by_hl


                                #################
                                #               #
################################# End of Script ############################
                                #               #
                                #################
