#!/usr/bin/perl -w

#
#  this perl script is designed to set up the directory
#  structure for a molpro job, run the input file and extract 
#  energy from the output file

# This is based on the g09shuttle script    

#  usage:
# 
#	perl molpshuttle.pl filename eemb <0,1,2> <0,1,2> itotalqm cpop imode    #1115HL11
#
#  where 
#     filename is the gaussian input filename without the .g09 extension
#     eemb is the number of total fragments
#     3rd arguement: 0 = energy, 1 = gradient, 2 = frequency calculation 
#     4th arguement: 0 = oniom + mm calculations, 1 = normal calculation
#                     2 = screened charge calculations using PP     #0603BW11
#     itotalqm is the number of atoms in the current fragment
#     cpop tells which type of population analysis is to be performed
#     imode: 0 = regular EE-MB calculation, 1 = EE-MB-CE calculation	#1115HL11

use Cwd;
use strict;

                       #############################
                       #                           #
######################## User Needs to Change Here ############################
                       #                           #
                       #############################

#  these variables must be changed to match the user's computer system

my $molp = '/soft/koronis/molpro/2010.1.24/bin/molpro';

my $scratchdir="/scratch/uvdev1/hannahl/mbpac/testruns";


                             ##################
                             #                # 
############################## Main Procedure ##################################
                             #                #
                             ##################

my $inormal = $ARGV[3];
my $imode   = $ARGV[6];		

my $home = cwd;
my $name = "$ARGV[0]\_$ARGV[1]";  
my $nameonly = "$ARGV[0]";		

$scratchdir=$scratchdir . "/$name";	
mkdir "$scratchdir" || die "Failed to create directory $scratchdir!!\n";;

if ($inormal == 0) {                      
  system("mv $name\.lat $scratchdir");
}
system("mv $name\.inp $scratchdir");

chdir $scratchdir;

system("$molp $scratchdir\/$name\.inp");    

my $outfile  = "$name\.out";

open(TMP,"<$outfile") || die "Cannot open $outfile";
my @contents = <TMP>;
close(TMP);
chomp(@contents);
my $energy=0;
my $hf=0;
foreach my $line (@contents) {
    if (!($line=~/\*\*\*/) && !($line=~/Variable memory released/)) {
	my @spl = split(' ',$line);
	$energy = $spl[0];
	$hf = $spl[1];
    }
}


chdir $home || die "cannot change to the home directory $home!!!\n";

my $thisishf = 0;
if ($name=~/fullhf/){
    $thisishf = 1;
}

if ($thisishf==0){
    open(EEMB,">$name\.eemb") || die "Cannot open .eemb file";
    print EEMB "$nameonly\n";	
    print EEMB "Self energy of the charges =       0\n";
    if ($imode==1) {
	print EEMB "Hartree Fock Energy                        R     $hf\n";
    }
    print EEMB "SCF Energy                                 R     $energy
Cluster Energy with triples                R     $energy
Total Energy                               R     $energy\n";
    print EEMB "Dipole Moment
  0 0 0
Atomic partial charges   N=          0 ";
    close(EEMB);
} else {
    open(EEMB,">$name\.txt") || die "Cannot open .txt file";
    print EEMB "$nameonly\n";	
    print EEMB "Self energy of the charges =         0
Hartree Fock Energy                        R     $energy
SCF Energy                                 R     $energy
Total Energy                               R     $energy
Dipole Moment
 0 0 0
Atomic partial charges   N=          0 \n";
    close (EEMB);
}


                             ##################
                             #                #
##############################   Subroutines  ##################################
                             #                #
                             ##################





                                #################
                                #               #
################################# End of Script ############################
                                #               #
                                #################
