c******************************************************************************
C
C	EEHED
C
C******************************************************************************
C
C	subroutine eehed
C
C   created:	September 20, 2006 by E. E. Dahlke
C   revised:    June 10, 2009 by Erin Dahlke Speetzen 
C
C   purpose:    write EEMB program header to output file
C   revised: 	to make sure of the core and keywrds options
C
C   description:	this subroutine writes the correct reference for the program
C					to the output file
C
C	called by:	main.f
C	calls:		
C
      subroutine eehed
	  
      implicit none
	  
      write(2,1001)
      write(2,1011)
      write(2,1021)
      write(2,1031)
      write(2,1041)
      write(2,1051)
      write(2,1061)
	  
	  
 1001 format (/,80(1h*))
 1011 format (/3x,'MBPAC 2012-4A')
 1021 format (3x,'Hannah Leverentz, Erin Dahlke, Hai Lin, Bo Wang,',
     *     ' Helena Qi, Jeremy Tempkin,',/3x,'Donald G. Truhlar')
 1031 format (3x,'Department of Chemistry and Supercomputing ',
     *         'Institute')
 1041 format (3x,'University of Minnesota')
 1051 format (3x,'Minneapolis, MN 55455-0431') 
 1061 format (/,80(1h*))

	  
	  end

c******************************************************************************
C
C      WSUM
C
c******************************************************************************
C
C       subroutine wsum
C    created : October 4, 2006 by E. E. Dahlke
C    revised : March 9, 2011 by H. R. Leverentz
C    purpose : write a summary of the user defined input to the output file
C
C    description :  take the information the user provides in the input file
C 	 	    and write a summary to the output file
C
C    
      subroutine wsum(natms, rstrt,eemb, fragid, afrag, cm,  ekey,
     *            gradkey, hesskey, optkey, geom, asym, title, bgq, 
     *            basis,esm, method,mem,bkey,nbasis,nt,nbgq,nfrg,
C revised 6/10/09 by EDS
C  Add by Hannah Leverentz, 3/9/2011
C    *            keywrds, core, pseudo, ncore)
     *            keywrds, core, pseudo, ncore, cpop,lscreen,
     *            allmmpp,nmmpp,mmpseudo,leembce)             !0603BW11 add lscreen; 0825BW11 add pseudo; 1115HL11 add leembce
C  End by Hannah Leverentz, 3/9/2011
C  end 6/12/09

      implicit none
      include 'common.inc'

      integer natms, rstrt, eemb,  ekey, gradkey, hesskey, optkey
      integer nbgq, nbasis, nt, nfrg
C  added 6/10/09 by eds
      integer pseudo, ncore
C   end 6/10/09
      integer fragid(maxfrag,maxapf), afrag(maxfrag), cm(maxfrag,2)
      integer i, j
      character*80 title(5), basis(1000)
      character*20 bgq(maxat,5), esm, method, mem, bkey !bgq 2->6
C  Add by Hannah Leverentz, 3/9/2011
      character*20 cpop
C  End by Hannah Leverentz, 3/9/2011
      integer lscreen     !0603BW11
      integer leembce     !1115HL11
      integer nmmpp,mmpseudo        !0825BW11
      character*80 allmmpp(1000)    !0825BW11
c added 6/10/09 by eds
      character*80 core(1000), keywrds
C  end 6/10/09 by eds
     
      write(2,1050)
C  write summary to out put file
C -- title
      do i = 1, nt
         write(2,'(a80)') title(i)
      end do

      write(2,1050)
 1050 format (/,80('_'),/,80('_'),/)

      write(2,1075)
 1075 format(/,'INPUT DATA:',/)

C -- natoms
      write(2,1100) natms
 1100 format(/3x,'Number of atoms : ', i5)

C -- geometry
      write(2,1150)
      do i = 1, natms
         write(2,1200) asym(i), geom(i,1), geom(i,2), geom(i,3)
      end do

 1150 format(/3x,'Atomic coordinates:', //8x,'Atom',t28,'X',
     *       t48, 'Y', t68, 'Z'/)
 1200 format(10x,a8,t20,d15.8,t40,d15.8,t60,d15.8)

C  -- nfrg
      write(2,1250) nfrg
 1250 format(/3x, 'Number of fragments in system: ', i5)

C  -- eemb
      write(2,1300) eemb
 1300 format(/3x, 'Carrying out EE-MB expansion through order : ', i1)

C -- fragid
      write(2,1350)
      write(2,1400)
      do i = 1, nfrg
	 write(2,1410) i, (fragid(i,j), j = 1,3)
         if (afrag(i) .gt. 3) then
             write(2,1420) (fragid(i,j), j =4, afrag(i))
         end if
      end do
 1350 format(/3x,'You have assigned these atoms to these fragments:')
 1400 format(3x, 'Frag. #',t15,'Atom #s')
 1410 format(2x,i5, ' - ', 3i5)
 1420 format(11x,3i5)

C  -- cm
      write(2,1450)
      write(2,1500)
      do i = 1, nfrg
         write(2,1550) i, cm(i,1), cm(i,2)
      end do
 1450 format(/3x, 'You specified the following charges and ',
     *        'multiplicites for your fragments:')
 1500 format(3x,'Frag #',t15,'Charge',t25,'Mult')
 1550 format(2x,i5,t15,i3,t25,i2)

C -- background charges
      write(2,1600)
      write(2,1650)
      do i = 1, nbgq
         write(2,1700) bgq(i,1), bgq(i,2)
      end do
 1600 format(/3x,'You specified the following background charges:')
 1650 format(5x,'Atom Type', t30,'Charge')
 1700 format(7x,a20,a20)


C -- mem
      write(2,1800) mem
 1800 format(/3x,'Amount of Memory per Calculation : ', a20)

C -- esm
      write(2,1850) esm
 1850 format(/3x, 'Electronic Structure Package : ', a20)
      
C -- method
      write(2,1900) method
 1900 format(/3x,'Level of Electronic Structure Theory : ', a20)

C -- basis set info
      write(2,1950) bkey
 1950 format(/3x, 'Type of basis set specified : ', a20)
      write(2,1975)
 1975 format(/3x, 'Basis set specified : ')

      do i = 1, nbasis
         write(2,2000) basis(i)
 2000    format(3x, a80)
      end do
C  added 6/10/09 by eds
C -- keywords
      write(2,2010) keywrds
 2010 format(/3x, 'Keywords specified: ', a80)

C -- core potential information
      if (pseudo .eq. 1) then
        write(2,2015)
 2015   format(/3x, 'Effective core potential specified: ')
        do i = 1, ncore
           write(2,2020) core(i)
 2020      format(3x, a80)
         end do
      end if
c  end 6/10/09 by eds

C  Add by Hannah Leverentz, 3/9/2011
c 
C -- population/charge analysis information
c 
      if (cpop(1:4) .ne. 'none') then
        write(2,2022) cpop
      end if
c 
 2022 format(/3x,'Method of population/charge analysis: ',a20)      
c 
C  End by Hannah Leverentz, 3/9/2011

c -- MM pseudopotential                      !1102BW11
c               			     !1102BW11
      if (mmpseudo .eq. 1) then              !1102BW11
        do i = 1, nmmpp                      !1102BW11
          write(2,2023) allmmpp(i)           !1102BW11 
        end do                               !1102BW11
      end if                                 !1102BW11
 2023 format(/3x,'MM pseudopotential:',a80)  !1102BW11
c
c -- Screened charge
c
      if (lscreen .eq. 1) then               !1102BW11    
        write (2,2024)                       !1102BW11
      endif                                  !1102BW11
 2024 format(/3x,'Screened charges are used in the calculations') !1102BW11
c
c -- EE-MB-CE calculation                               !1115HL11
c					 	        !1115HL11
      if (leembce .eq. 1) then                          !1115HL11
        write (2,2025)                                  !1115HL11
      endif                                             !1115HL11
 2025 format(/3x,'The EE-MB-CE method is being used.')  !1115HL11
c
C -- energy/gradient/hessian information
      write(2,2030)                                     !1115HL11 changed from 2025 to 2030
 2030 format(/6x, 'Type of calculation carried out: ')  !1115HL11 changed from 2025 to 2030


      if (optkey .eq. 1) then
         write(2,2080)
      elseif (hesskey .eq. 1) then
         write(2,2070)
      elseif (gradkey .eq. 1) then
         write(2,2060)
      elseif (ekey .eq. 1) then
         write(2,2050)
      end if

 2050    format(/6x,'Single Point Energy Calculation')
 2060    format(/6x,'Gradient Calculation')
 2070    format(/6x,'Hessian Calculation')
 2080    format(/6x,'External Optimization')

      write(2,1050)
      
      return 

      end
c******************************************************************************
C
C    DEFALT
C
c******************************************************************************
C
C   created : October 17, 2006 by E. E. Dahlke
C   revised : June 10, 2008 by E Dahlke Speetzen
C             March 9, 2011 by H R Leverentz
C   purpose : set the default variables for the EE-MB program
C   revision:  to make use of the core and keywrds option
C              9 Mar 2011: to allow EE-MB population/charge analysis
C
      subroutine defalt(rstrt,eemb,ekey,gradkey,hesskey,optkey,
     *            title,esm,mem,method,basis,bkey,nbasis,nt,cm,
C  revised 6/10/09 by eds
C  Add by Hannah Leverentz, 3/9/2011
c    *            keywrds, core, pseudo, ncore)
     *            keywrds, core, pseudo, ncore, cpop,lscreen,
     *            allmmpp,nmmpp,mmpseudo,leembce)       !0603BW11 add lscreen; 0825BW11 add pseudo; 1115HL11 add leembce
C  End by Hannah Leverentz, 3/9/2011
c  end 6/10/09 by eds

      implicit none
      include 'common.inc'

      integer rstrt,eemb,ekey,gradkey,hesskey,optkey,nbasis, nt, i
C  added 6/10/08 by eds
      integer pseudo, ncore
c  end 6/10/09 eds
      integer cm(maxfrag,2)
      character*80 title(5), basis(1000)
c  added 6/10/09 by eds
      character*80  core(1000), keywrds
c  end 6/10/09 eds
      character*20 esm,mem,method,bkey
C  Add by Hannah Leverentz, 3/9/2011
      character*20 cpop
C  End by Hannah Leverentz, 3/9/2011
      integer lscreen                !0603BW11   
      integer nmmpp,mmpseudo        !0825BW11
      character*80 allmmpp(1000)    !0825BW11
      integer leembce               !1115HL11

C  set all integers
      rstrt = 0
      eemb = 2
      ekey = 1
      gradkey = 0
      hesskey = 0
      optkey = 0
      nbasis = 1
      nt = 1
      leembce = 0                   !1115HL11 
c added 6/10/09 by eds
      pseudo = 0
      ncore = 0
c end 6/10/09 by eds 
      do i = 1, maxfrag
         cm(i,1) = 0
         cm(i,2) = 1
      end do

C  set character strings
      bkey = 'lib'
      basis(1) = 'sto-3g'
      method = 'mpw1pw91'
      esm = 'g09'
      title(1) = 'title'
      mem = '300mb' 
c  added 6/10/09 by eds
      keywrds = 'SCF=(tight,maxcycle=500,xqc)'
c end 6/10/09 eds
C  Add by Hannah Leverentz, 3/9/2011
      cpop = 'none'
C  End by Hannah Leverentz, 3/9/2011

      lscreen = 0              !0603BW11
      mmpseudo=0               !0825BW11
      nmmpp=0                  !0825BW11

      return

      end

