C**********************************************************************,
C
C  READIN
C
C**********************************************************************
      subroutine readin(natms, rstrt,eemb, fragid, afrag, cm, 
     *            ekey, gradkey, hesskey, optkey, geom, asym, title,
     *            bgq, basis, nfrg, esm, method, nbasis,mem,nbgq,
C  Add by Hannah Leverentz, 3/9/2011
c    *            nt,bkey,keywrds,core,pseudo,ncore)
     *            nt,bkey,keywrds,core,pseudo,ncore,cpop,lscreen,
     *            allmmpp,nmmpp,mmpseudo,leembce)        !0603BW11 add lscreen; 0825BW11 add mmpseudo; 1115HL11 add leembce
C  End by Hannah Leverentz, 3/9/2011

C  Created:	September 20, 2006 by Erin E Dahlke
C  Revised:	June 10, 2009 by Erin Dahlke Speetzen
C               March 9, 2011 by Hannah Leverentz
C  Purpose:	to read in the eemb.in file
C
C  Description:	calls additional subroutines to read the input file and 
C		returns values for all user defined parameters.
C		modified from the read5.f subroutine of multilevel 4.2
C
C
C  Called by:	main
C  Calls:	defalt, rline, rgen, rfrag, rjob
C
C  Variables defined here:

      implicit none
      include 'common.inc'
      integer istrt, isect, iscend, iend
      integer natms, nfrg, nbasis, nbgq, nt, ncore
      integer rstrt, eemb,  ekey, gradkey, hesskey, optkey
      integer pseudo
      integer fragid(maxfrag,maxapf), afrag(maxfrag)
      integer cm(maxfrag,2)
      integer i, j 
C  Add by Hannah Leverentz, 3/9/2011
      character*20 cpop
C  End by Hannah Leverentz, 3/9/2011
      character*80 string
      character*80 title(5), basis(1000), core(1000)
      character*20 bgq(maxat,5), esm, method, mem, bkey    !0603BW11 change bgq(maxat,2) to bgq(maxat,5) 
      character*80 keywrds
      integer lscreen                                      !0603BW11 add lscreen
      integer nmmpp,mmpseudo        !0825BW11
      character*80 allmmpp(1000)    !0825BW11
      integer leembce               !1115HL11

C  read the first section of the input file

      call rline(1,string,istrt,isect,iscend,iend)

C  check that input file isn't empty

      if (iend .eq. 1) then
	write(2,1000)
 1000   format(3x, 'ERROR: the input file was empty')
	stop 'readin 1'
      endif

C  check that the first non-blank non-comment line is a section heading

      if(isect .ne. 1) then
	write(2,1100)
 1100   format(3x, 'ERROR: the first line of the input file must ',
     *             ' be a section header that begins with a *')
	stop 'readin 2'
      endif

C  call the right subroutine to read in the data .  
C  control is returned back to htis subroutine when another
C  section is found.  The loop continues until the end of the file is reached.
      j = 0
      do while (iend .eq. 0)
	j = istrt + 1
        if (string(j:j+6) .eq. 'general') then
	   call rgen(string,iend,istrt,natms,rstrt, asym, geom, title,nt)
 	elseif (string(j:j+3) .eq. 'frag') then
	   call rfrag(string,iend,istrt,natms,eemb,fragid,bgq,cm,nfrg,
     *                nbgq,afrag)
	elseif (string(j:j+2) .eq. 'job') then
	   call rjob(iend,esm,method,basis,ekey,hesskey,optkey, 
     *               gradkey,nbasis,mem,bkey,pseudo,core,keywrds,
C  Add by Hannah Leverentz, 3/9/2011
c    *               ncore)
     *               ncore, cpop, lscreen,allmmpp,nmmpp,mmpseudo,             !0603BW11 add lscreen 0825BW11 add mmpseudo
     *               leembce)                                                 !1115HL11
C  End by Hannah Leverentz, 3/9/2011
        else
	   write (2,1200) string(j:j+79)
	   stop 'readin 3'
        endif
      enddo

      call chkcompat(optkey,hesskey,gradkey,method,lscreen,leembce,	!1115HL11
     *               cpop,esm,pseudo)					!0627HQ12

 1200 format (3x, 'Error: the following is not a valid section name: ',
     *        a)
      return

      end

C**********************************************************************
C
C  RGEN
C
C**********************************************************************
      subroutine rgen(string, iend, istrt,natms, rstrt, asym,
     *                 geom, title,nt)


C  Created:	Sept 20, 2006 by Erin E Dahlke
C  Revised:
C  Purpose 	Read in general section of the input file
C
C  Description: Calls all of the subroutines needed to read in the
C		general section of the input file.  values are assigned
C		to the proper variable, or another funtion/subroutine 
C		is called to do this.  Based on the rmlgen subroutine 
C		from Multilevel 4.2
C
C
C  Called by:   readin
C  Calls:       rline, rvar, rtitl, rgeom, icint
C
C  Variables defined here:


      implicit none
      include 'common.inc'
      character*80 string,title(5)
      character*20 rvar, temp
      integer istrt, isect, iscend, iend
      integer rstrt, natms, atset, icint, nt
      integer i
      double precision cfloat

      atset = 0

C  read the first line of this section

      call rline(1,string,istrt,isect,iscend,iend)
C
C  loop through the keywords in this section until the end is reached
C
      do while (isect .eq. 0 .and. iend .eq. 0 .and. iscend .eq. 0)
C TITLE
C
         if(string(istrt:istrt+4) .eq. 'title') then
	   call rtitl(title,iend,nt)
C RESTART
         elseif (string(istrt:istrt+6) .eq. 'restart') then
           temp = rvar(string,istrt,'RESTART')
           read(unit=temp,fmt=*) rstrt
C NATOMS
	 elseif (string(istrt:istrt+5) .eq. 'natoms') then
           temp = rvar(string,istrt,'NATOMS')
           read(unit=temp,fmt=*) natms
	   atset = 1
           if (natms .gt. maxat) then
		write(2,1100) natms, maxat
 1100           format(3x,'ERROR: the number of atoms specified in ', 
     *              'the input file', i5, ' is larger than the ',
     *               'maximum number allowed ', i5)
		stop 'rgen 1'
           end if
C GEOM
         elseif (string(istrt:istrt+7) .eq. 'geometry') then
	   call rgeom(geom,asym,natms,string,istrt,iend)
C  Invalid option
	 else 
	   write(2,1000) string(istrt:80)
	   stop 'rgen 1'
 1000      format(3x,'ERROR: the following is not a valid option in ',
     *            'the GENERAL section: ', a)
         endif

         call rline(1,string,istrt,isect,iscend,iend)

      end do
      return
     
      end

C**********************************************************************
C
C  RFRAG
C
C**********************************************************************
      subroutine rfrag(string,iend,istrt,natms,eemb,fragid,bgq,cm,nfrg,
     *                 nbgq,afrag)


C  Created:     Sept 20, 2006 by Erin E Dahlke
C  Revised:
C  Purpose      Read in fragment section of the input file
C
C  Description: Calls all of the subroutines needed to read in the
C               fragment section of the input file
C
C
C  Called by:   readin
C  Calls:       rline
C
C  Variables defined here:

      implicit none
      include 'common.inc'
      character*80 string,title(5)
      character*20 rvar, bgq(maxat,5), temp       !0603BW11 bgq 2->5
      integer istrt, isect, iscend, iend
      integer nfrg, eemb, fragid(maxfrag,maxapf), afrag(maxfrag)
      integer iatom, natms, nbgq, nfset, fidset, bgqset
      integer cm(maxfrag,2)
      integer i, j
      double precision cfloat

C  read the first line of this section

      call rline(1,string,istrt,isect,iscend,iend)
C
C  loop through the keywords in this section until the end is reached
C
      do while (isect .eq. 0 .and. iend .eq. 0 .and. iscend .eq. 0)
C
C NFRAG
         if(string(istrt:istrt+4) .eq. 'nfrag') then
	   temp = rvar(string,istrt,'NFRAG')
           read(unit=temp,fmt=*) nfrg
           nfset = 1
           if (nfrg .gt. maxfrag) then
              write(2,1100) nfrg, maxfrag
 1100         format(3x,'The number of fragments specified ', i5,
     *              ' is larger than the maximum number allowed ',
     *              i5)
              stop 'rfrag 2'
           end if
C EEMB
         elseif (string(istrt:istrt+3) .eq. 'eemb') then
           temp = rvar(string,istrt,'EEMB')
	   read(unit=temp,fmt=*) eemb
C FRAGID
         elseif (string(istrt:istrt+5) .eq. 'fragid') then
           call rfid(fragid,natms,string,istrt,iend,nfrg,afrag)
	   fidset = 1
C  BGCHARGE
         elseif (string(istrt:istrt+7) .eq. 'bgcharge') then
           call rbgq(bgq,string,istrt,iend,nbgq)
           bgqset = 1
C  CHGMLT
         elseif (string(istrt:istrt+5) .eq. 'chgmlt') then
           call rcm(cm,string,istrt,iend,nfrg)
C  Invalid option
         else
           write(2,1000) string(istrt:80)
           stop 'rfrag 1'
 1000      format(3x,'ERROR: the following is not a valid option in ',
     *            'the FRAG section: ', a)
         endif

         call rline(1,string,istrt,isect,iscend,iend)

      end do
C  Check to make sure that all variables without default values
C  have been set
      if (nfset .ne. 1) then
          write(2,1200)
          stop 'rfrag 3'
       else if (fidset .ne. 1) then
          write(2,1300)
          stop 'rfrag 4'
       else if (bgqset .ne. 1) then
          write(2,1400)
          stop  'rfrag 5'
       end if

 1200 format(3x,'ERROR:  You did not specify the number of fragments ',
     *       'in your system -  NFRAG')
 1300 format(3x,'ERROR:  You did not list which atoms belong to which ',
     *       'fragment in your system -  FRAGID')
 1400 format(3x,'ERROR:  You did not give the background charges to ',
     *        'be used in the calculation - BGCHARGE')
     

      return

      end


**********************************************************************
C
C  RJOB
C
C**********************************************************************
      subroutine rjob(iend,esm,method,basis,ekey,hesskey,optkey,
     *                gradkey, nbasis,mem,bkey,pseudo,core,keywrds,
C  Add by Hannah Leverentz, 3/9/2011
c    *                ncore)
     *                ncore, cpop,lscreen,allmmpp,nallmmpp,mmpseudo,         !0603BW11 add lscreen 0825BW11 add mmpseudo,allmmpp,nallmmpp
     *                leembce)                                               !1115HL11

C  End by Hannah Leverentz, 3/9/2011

C  Created:     Sept 20, 2006 by Erin E Dahlke
C  Revised:	June 10, 2009 by Erin Dahlke Speetzen
C               March 9, 2011 by Hannah Leverentz
C  Purpose      Read in job section of the input file
C
C  Description: Calls all of the subroutines needed to read in the
C               job section of the input file
C
C
C  Called by:   readin
C  Calls:       rline
C
C  Variables defined here:


      implicit none
      include 'common.inc'
      character*80 keywrds
      character*80 string, basis(1000), core(1000)
      character*20 rvar, esm, method, temp, bkey, mem
      character*20 corekey
C  Add by Hannah Leverentz, 3/9/2011
      character*20 cpop
      integer ipop,pfin
C  End by Hannah Leverentz, 3/9/2011
      integer istrt, isect, iscend, iend
      integer rstrt, natms, atset,  nbasis, pseudo, ncore
      integer i, j
      integer ekey, gradkey, hesskey, optkey
      double precision cfloat
      integer lscreen        !0603BW11
      integer nallmmpp,mmpseudo       !0825BW11
      character*80 allmmpp(1000)      !0825BW11
      integer leembce                 !1115HL11

C  read the first line of this section

      call rline(1,string,istrt,isect,iscend,iend)
C
C  loop through the keywords in this section until the end is reached
C
      do while (isect .eq. 0 .and. iend .eq. 0 .and. iscend .eq. 0)
C  PROGRAM
         if (string(istrt:istrt+6) .eq. 'program') then
	    esm = rvar(string,istrt,'PROGRAM') 
C  METHOD
         elseif (string(istrt:istrt+5) .eq. 'method' ) then
            method = rvar(string,istrt, 'METHOD')
C  BASIS
         elseif (string(istrt:istrt+4) .eq. 'basis') then
            bkey = rvar(string,istrt,'BASIS')
            call rbasis(string,istrt,iend,basis,bkey,nbasis)

C  added 6/10/09 by Erin Dahlke Speetzen
C  CORE
         elseif (string(istrt:istrt+3) .eq. 'core') then
            pseudo = 1
            call rcore(string,istrt,iend,core,ncore,pseudo)
C   MMPSEUDO                                                       !0825BW11
C   Pseudo potentials used for the MM atoms                        !0825BW11
         elseif (string(istrt:istrt+7) .eq. 'mmpseudo') then       !0825BW11
            mmpseudo = 1                                           !0825BW11
            lscreen=1                                              !1031BW11 when pseudopotentials are used for MM atoms, always use the screening scheme
            call rcore(string,istrt,iend,allmmpp,nallmmpp,mmpseudo)!0825BW11
C  KEYWORDS
         elseif (string(istrt:istrt+7) .eq. 'keywords') then
              call rline(1,string,istrt,isect,iscend,iend)
              keywrds = string
C  end of addition 6/10/09
C  Add by Hannah Leverentz, 3/9/2011
C  POPULATION 
         elseif (string(istrt:istrt+9) .eq. 'population' ) then
            cpop = rvar(string,istrt,'POPULATION')
            call chkcpop(cpop,ipop,pfin)
C  End by Hannah Leverentz, 3/9/2011
C  SCREEND CHARGES                                           !0603BW11
C  whether to use screened charges                           !0603BW11
        elseif (string(istrt:istrt+5) .eq. 'scrchg') then    !0603BW11
            lscreen = 1                                      !0603BW11
C  NO screened charges
        elseif (string(istrt:istrt+5) .eq. 'noscrchg') then  !0603BW11
            lscreen = 0                                      !0603BW11
C  EEMBCE METHOD                                             !1115HL11
C  whether to use the EE-MB-CE method                        !1115HL11
        elseif (string(istrt:istrt+5) .eq. 'eembce') then    !1115HL11
            leembce = 1                                      !1115HL11
        elseif (string(istrt:istrt+7) .eq. 'noeembce') then  !1115HL11
            leembce = 0                                      !1115HL11
C  MEMORY
         elseif (string(istrt:istrt+2) .eq. 'mem' ) then
            mem = rvar(string,istrt,'MEM')
C  ENERGY
	 else if (string(istrt:istrt+5) .eq. 'energy') then
	    ekey = 1
C  NOENERGY
	 else if (string(istrt:istrt+7) .eq. 'noenergy') then
	    ekey = 0
C  GRADIENT
 	else if (string(istrt:istrt+7) .eq. 'gradient') then
	    gradkey = 1
C  NOGRADIENT
	else if (string(istrt:istrt+9) .eq. 'nogradient') then
	    gradkey = 0
C  HESSIAN
	elseif (string(istrt:istrt+6) .eq. 'hessian') then
	    hesskey = 1
C  NOHESSIAN
        elseif (string(istrt:istrt+8) .eq. 'nohessian') then
	    hesskey = 0
C  OPT
        elseif (string(istrt:istrt+2) .eq. 'opt') then 
            optkey = 1
C  NO OPT
        elseif (string(istrt:istrt+4) .eq. 'noopt') then
            optkey = 0
C  Invalid option
        else 
           write(2,1000) string
           stop 'rjob 1'
 1000      format(3x,'ERROR: the following is not a valid option in ',
     *            'the JOB section: ', a80)
        endif
         
        call rline(1,string,istrt,isect,iscend,iend)
      
      end do
      return

      end
C**********************************************************************
C
C  RLINE
C
C**********************************************************************
      subroutine rline(iunitr,string,istrt,isect,iscend,iend)

C  Created:     April 8, 1999 by P.L. Fast 
C  Revised:	Sept 20, 2006, taken from Multilevel 4.2
C  Purpose      find the first non-comment, non-blank line of a file
C		and the location of the first character on that line.
C		The contents of the line are placed in the variable string
C		it will be changed all to lower case using the case function
C		A flag is set at the start of a new section.

c  istrt:   location in 80 character string that is the first non-blank
c  isect:   a 1 is returned if the first non-blank character is a *
c           signalling the start of a section
c  iscend:  a 1 is returned if the line is 'end' signalling a section
c           end
c  iend:    a 1 is returned if the end of file 5 is encountered
c  string:  80 character string that is the contents of the current line
c

C
C  Description: Calls all of the subroutines needed to read in the
C               general section of the input file
C
C
C  Called by:   readin
C  Calls:       
C
C  Variables defined here:


      implicit none
      character*80 string,case
      integer istrt, isect, iscend, iend, iunitr, iblnk, icommt
      integer i, j

 1000 format(A80)

C  initialize the variables
      istrt = 1
      isect = 0
      iend = 0
      iblnk = 1
      iscend = 0
      icommt = 1

c
c     read in next line in iunitr - find first non-blank character  with
c     fchar and then check the line with chkln to see if it is a comment
c     line. Continue cycling through lines in infile until a non-blank,
c     non-comment line is found
c
      do while ((iblnk .eq. 1) .and. (icommt .eq. 1))
	 istrt = 1
         read(1,1000,end=9999) string
         call fchar(string, istrt, iblnk)
         if (iblnk .ne. 1) then
            call chkln(string, istrt, isect, icommt, iscend)
         endif
      end do
c
c     strip off any trailing comments on the line
c
      do 1 j=istrt,80
         if (string(j:j).eq.'#') then
            string(j:80) = ' '
            return
         endif
 1    continue
c
c     change to lower case
c
      string = case(string)
      return


 9999 iend=1

      return
      end
c
c*******************************************************************************
c  FCHAR
c*******************************************************************************
c
      subroutine fchar(string, istrt, ierr)
c
c     Created:  June 18, 1999 by J. M. Rodgers
c     Revised:  Sept 20, 2006 by E. E. Dahlke (taken from multilevel 4.2)
c     Purpose:  Find the next character on the line
c	
c
c     Description:  Scrolls through the positions in string starting
c         at position istrt.  Sets istrt to position of first character.
c         If there is no character left on the line, sets ierr to 1.
c
c     Called by:  rline
c     Calls:
c
      implicit none
c
      integer istrt, ierr, char, length
      character*80 string
      ierr = 0
      char = 0
      length = 80

      do while (istrt .le. length .and. char .eq. 0)
         if (string(istrt:istrt) .ne. ' ') then
            char = 1
         elseif (istrt .eq. length) then
            ierr = 1
         endif
         istrt = istrt + 1
      enddo
c
c     Sets istrt back 1 in order to counteract increment at end
c       of do-while loop.
c
      istrt = istrt -1

      return
      end
c*******************************************************************************
c  CHKLN
c*******************************************************************************
c
      subroutine chkln(string, istrt, isect, icommt, iscend)
c
c     Created:  June 18, 1999 by J. M. Rodgers
c     Revised:
c     Purpose:  Check line to see if it's a special type.
c
c     Taken from Multilevel 4.2
c
c     Description:  Examines character at istrt to see if the line
c       is a comment or a section header.  Examine 1st 4 characters
c       starting at istrt to see if it's a section end.
c
c     Called by:  rline, rlist
c     Calls:
c
      implicit none
c
      integer istrt, isect, icommt, iscend
      character*80 string, case, wrd

      isect = 0
      icommt = 0
      iscend = 0

      wrd = string
      wrd = case(wrd)

      if (wrd(istrt:istrt) .eq. '*') then
         isect=1
      elseif (wrd(istrt:istrt) .eq. '#') then
         icommt=1
      elseif (wrd(istrt:istrt+3) .eq. 'end ') then
         iscend = 1
      endif

      return
      end
c****************************************************************************
c  CASE
c****************************************************************************
c
      function case (string)
c
c     Function case
c
c     Created:  April 8, 1999 by P. L. Fast
c               This code was taken from Polyrate
c     Revised:
c     Purpose:  consistent case
c
c     Description:  this function takes a string of 80 characters and
c                   converts all the upper case letters to lower case.
c
c     Called by: rline, chkln, g98oute, mg98oute, mg98outg, mg98outh
c     Calls:
c
c     Variables defined here:
c
c
      implicit none
c
      integer  itry, i
      character*80 string, case
      character * 1 xlett
c
      do 10 i = 1, 80
        xlett = string(i:i)
        itry = ichar (xlett)
        if (xlett .ge. 'A' .and. xlett .le. 'Z') then
          itry = itry + 32
          string (i:i) = char (itry)
        endif
 10   continue
c
      case  = string
c
      return
      end
c****************************************************************************
C
C  RTITL

c****************************************************************************

      subroutine rtitl(title,iend,nt)

c     Created:  April 15, 1999 by P. L. Fast
c     Revised:  June 17, 1999 by jmr
c		Sept 20, 2006 by E. E. Dahlke
c     Purpose:  read in the title
c
c     Description:	this subroutine reads in the title of the input file
C			by looping throught the ines until it finds the end.
C			will only allow a five line title.  based on the rlist
C			subroutine from multilevel 4.2
c                   
c
c     Called by: rgen
c     Calls: fchar, chkln
c
c     Variables defined here:
c
c

      implicit none

      character*80 string, title(5)
      integer istrt, isect, iend, iblnk, iscend, icommt, nt
      integer i, j

      i = 0
      isect = 0
      iend = 0
      iscend = 0
c
c     Reads and checks the next non-blank, non-comment line.
c
  20  i = i + 1
      iblnk = 1
      icommt = 1
      do while ((iblnk .eq. 1) .or. (icommt .eq. 1))
         istrt=1
         read(1,800,end=9999) string
 800     format(a80)
         call fchar(string, istrt, iblnk)
         call chkln(string, istrt, isect, icommt, iscend)
      end do
c
c     Exits if a new section is reached.  Ends if an 'END' is found.
c     Exits if the list is longer than 5 lines and no 'END' is found.
c     Otherwise saves the line to list
c
      if (isect .eq. 1) then
	 write(2,1000)
1000     format(/3x, 'ERROR:  No termination of TITLE section.')
         stop 'rtitl 1'
      elseif (iscend .eq. 1) then
         goto 40
      elseif (i .gt. 5) then
         write (2, 1100) 
1100     format (/3x, 'ERROR: TITLE section longer than 5 lines.')
         stop 'rtitl 2'
      else
         title(i) = string
         nt = i
         goto 20
      endif

   40 return

9999  write (2, 1200) 
1200  format(3x, 'ERROR:  No termination of TITLE  before EOF.')
      stop 'rtitl 3'
      end

c*******************************************************************************
c  RVAR
c*******************************************************************************
c
      function rvar(string, istrt, varname)
c
c     Created:  June 21, 1999 by J.M. Rodgers
c     Revised:  Sept 20, 2006 by E. E. Dahlke (taken from Multilevel)
c     Purpose:  Read a variable following a keyword and assign to var
c
c     Called by:  rgen
c     Calls:  fspace, rword
c
      implicit none
c

      integer istrt, ifin, length, ierr
      character*80 string
      character*20 rvar
      character*(*) varname

      call rword(string,istrt,ierr)
      if (ierr .eq. 1) then
         write (2, 800) varname
 800     format(/3x, 'ERROR: ', a15, ' must have an argument.')
         stop 'rvar 1'
      endif
      ifin = istrt+1
      call fspace(string, ifin, ierr)
      if (ierr .eq. 1) then
         length = ifin - istrt + 1
      else
         length = ifin - istrt
      endif
      if ( length .gt. 20 ) then
         write (2, 900) varname, string(istrt:ifin)
 900     format(/3x, 'ERROR: ', a15, ' argument too long: ', /a80)
         stop 'rvar 2'
      endif
      rvar = string(istrt:ifin)

      end
c
c*******************************************************************************
c  RWORD
c*******************************************************************************
c
      subroutine rword(string,istrt,ierr)
c
c     Created:  April 15, 1999 by P. L. Fast
c               Taken from Polyrate
c     Revised:  June 18, 1999 by JMR
c		Sept 20, 2006 by E. E. Dahlke
C		Taken from Multilevel
c     Purpose:  read the next word on the line after the word that istrt
c                   is in.
c
c     Called by:  rvar
c     Calls:  fspace, fchar
c
c
      implicit none
c
      integer istrt, ierr
      character*80  string
c
c clear flags
c
      ierr = 0
c
c search for the first blank - signaling the end of the current word
c
      call fspace(string, istrt, ierr)
      if (ierr .eq. 1) then
         return
      endif
c
c find first letter of next word
c
      call fchar(string, istrt, ierr)
      return
      end
c
c*******************************************************************************
c  RGEOM
c*******************************************************************************
c
      subroutine rgeom(geom,asym,natms,string,istrt,iend)
c
c     Created:  April 20, 1999 by P. L. Fast
c     Revised:  Sept 21, 2006 by E. E. Dahlke - Taken from Mulitlevel 4.2
c     Purpose:  Read the atoms and geometry from the input file.
c
c     Called by:  rgen
c     Calls:  rline, rword, cfloat
c
      implicit none
      include 'common.inc'
      integer i, istrt, isect, iend, iatom, ierr, iscend
      integer ifin, ibeg, ifind, iasym, numatom, geomunit, chindex
      integer natms
      double precision cfloat
      character*80  string

c
c read each line in the GEOM section until an 'end' is found.  for each
c line read the atomic symbol and cartesian coordinates.
c
      iatom = 0
      call rline(1,string,istrt,isect,iscend,iend)
c
      do while (iscend .ne. 1)
c
c  get the atomic symbol and store the symbol (with the first letter 
C  back in uppercase)

         iatom = iatom + 1
         ibeg = istrt
         call rword(string,istrt,ierr)
         ifin = istrt - 1
         if (ierr.ne.1) then
            ifind = 0
            iasym = 1
            do while(ifind.eq.0)
                  chindex = ichar(string(ibeg:ibeg)) - 32
                  string(ibeg:ibeg) = char(chindex)
	          asym(iatom) = string(ibeg:ibeg+2)
                  ifind = 1
            end do
c
c
c get coordinates off rest of line
c
            do 100 i = -2, 0
               ibeg = istrt
               call rword(string,istrt,ierr)
               ifin = istrt - 1
c
c check that the coordinate was found on the line
c
               if (ierr.eq.1 .and. i.ne.0) then
                  write (2,1100) string
 1100             format(/3x,'ERROR:  invalid Cartesian coordinate ',
     *                   ' in GEOM.  Invalid line is: ',a80)
                  stop 'rgeom 2'
               endif
c
c save coordinate:
c    i = 1  x-coordinate
c    i = 2  y-coordinate
c    i = 3  z-coordinate
c
               geom(iatom,i+3) =  cfloat(string(ibeg:ifin))
c
 100        continue
c
         else
            write (2,1200) string
 1200       format(3x,'ERROR: invalid line in GEOM: ',a80)
            stop 'rgeom 3'
         endif
c
         call rline(1,string,istrt,isect,iscend,iend)
c
      end do
c
c  return the final iatom value.
c
      numatom = iatom
      if (numatom .ne. natms) then
	  write (2,1300) numatom, natms
 1300	  format(3x,'ERROR: NATOMS = ', i4, 'does not equal ',
     *          'the number of atoms in your geometry ', i4)
          stop 'rgeom 4'
      end if

      return
      end
c
c
c
c*******************************************************************************
c  FSPACE
c*******************************************************************************
c
      subroutine fspace(string, istrt, ierr)
c
c     Created:  June 18, 1999 by J. M. Rodgers
c     Revised:  Sept 21, 2006 by E. E. Dahlke 
c		Taken from Multilevel 4.2
c     Purpose:  Find the next space on the line
c
c     Description:  Scrolls through the positions in string starting
c        at position istrt.  Sets istrt to the position of the first
c        space.  Or if there is no spacebar left on the line, sets
c        ierr to 1.
c
c     Called by:  rword, rcss, cfloat, icint, rvar, rcoef
c     Calls:
c
      implicit none
c
      integer istrt, ierr, blnk, length
      character* 80  string

      ierr=0
      blnk = 0
      length = 80
      do while (istrt .le. length .and. blnk .eq. 0)
         if(string(istrt:istrt) .eq. ' ') then
            blnk = 1
         elseif (istrt .eq. length) then
            ierr = 1
         endif
         istrt = istrt + 1
      enddo
c
c     Sets istrt back 1 in order to counter act increment at end
c       of do while loop.
c
      istrt = istrt - 1

      return
      end
c
c*******************************************************************************
c  CFLOAT
c*******************************************************************************
c
      function cfloat(string)
c
c     Created:  April 20, 1999 by P. L. Fast
c     Revised:  Sept 21, 2006 by E. E. Dahlke
C		Taken from Multilevel 4.2
c     Purpose:  Convert a string to a double precision number
c
c     Called by: rgeom
c     Calls:  fchar, fspace
c
      implicit none
c
      integer length, ibeg, iblnk, ifin, ierr, i, j
      integer idec, iue, ile, iud, ild, isum, iexp, n
      double precision  cfloat, f1, f2, f3, sign, esign
      character*(*) string
      character*80 number
      character ch
      logical lexp, ldec
c
      lexp = .false.
      ldec = .false.

      idec = 0
      iue = 0
      ile = 0
      iud = 0
      ild = 0
      isum = 0
      iexp = 0

      length = len(string)
      if (length .eq. 0) then
         cfloat = 0.0d0
         return
      endif
c
c     find the first nonblank character
c
      ibeg = 1
      call fchar(string, ibeg, iblnk)
c
c     if it is a blank string set function to zero
c
      if (iblnk .eq. 1) then
         cfloat = 0.0d0
         return
      endif
c
c     find the first blank character after the number
c
      ifin = ibeg+1
      call fspace(string, ifin, ierr)
      if (ierr .eq. 1) then
         ifin = length
      else
         ifin = ifin - 1
      endif
c
c     strip the blanks before and after the number
c
      number = string(ibeg:ifin)
      length = ifin - ibeg + 1
c
c     find the decimal point
c
      idec = index(number,'.')
      if (idec .ne. 0) ldec = .true.
c
c     finds the exponential symbol and ensures that there
c     is only one such marker (d or e) in the whole number
c
      iue = index(number,'e')
      ile = index(number(iue+1:),'e')
      iud = index(number,'d')
      ild = index(number(iud+1:),'d')
      isum = iue + ile + iud + ild
      iexp = max0(iue,ile,iud,ild)
      if (isum .gt. iexp) then
         write(2,1000) string
         stop 'cfloat 2'
      endif
      if (iexp .ne. 0) then
         lexp = .true.
      else
         iexp = length + 1
      endif
c
      if (.not. ldec) idec = iexp
c
c     get the number before decimal
c
      ibeg = 2
      if (number(1:1) .eq. '+') then
         sign = 1.0d0
      elseif(number(1:1) .eq. '-') then
         sign = -1.0d0
      else
         sign = 1.0d0
         ibeg = 1
      endif
      if (ibeg .eq. iexp) then
         f1 = 1.0d0
      else
         f1 = 0.0d0
      endif
      do 50 i = ibeg,idec-1
         ch = number(i:i)
         if (ch .ge. '0' .and. ch .le. '9') then
            n = ichar(ch) - ichar('0')
            f1 = f1 * 10.0d0 + dble(n)
         else
            write(2,1001) string
            stop 'cfloat 3'
         endif
50    continue
c
c     get the number after decimal
c
      f2 = 0.0d0
      if (ldec) then
         j = 0
         do 60 i = idec+1,iexp-1
            ch = number(i:i)
            if (ch .ge. '0' .and. ch .le. '9') then
               n = ichar(ch) - ichar('0')
               f2 = f2 * 10.0d0 + dble(n)
               j = j + 1
            else
               write(2,1002) string
               stop 'cfloat 4'
            endif
60       continue
         f2 = f2 / 10.0d0 ** dble(j)
      endif
c
c    get the exponent
c
      esign = 1.0d0
      f3 = 0.0d0
      if (lexp) then
         ibeg = iexp + 2
         if (number(iexp+1:iexp+1) .eq. '+') then
            esign = 1.0d0
         elseif(number(iexp+1:iexp+1) .eq. '-') then
            esign = -1.0d0
         else
            esign = 1.0d0
            ibeg = iexp + 1
         endif
         do 70 i = ibeg,length
            ch = number(i:i)
            if (ch .ge. '0' .and. ch .le. '9') then
               n = ichar(ch) - ichar('0')
               f3 = f3 * 10.0d0 + dble(n)
            else
               write(2,1003) string
               stop 'cfloat 5'
            endif
70       continue
      endif
c
      cfloat = (sign * (f1 + f2)) * 10.0d0 ** (esign*f3)
c
      return
c
1000  format(/1x,'illegal number-1: ',a80)
1001  format(/1x,'illegal number-2: ',a80)
1002  format(/1x,'illegal number-3: ',a80)
1003  format(/1x,'illegal number-4: ',a80)
c
      end
c*******************************************************************************
C
C  RFID
C
c*******************************************************************************
      subroutine rfid(fragid,natms,string,istrt,iend,nfrg,afrag)
C     Purpose: to read in the atom numbers that compose each fragment
C
C     Called by: readin
C     Calls:

      implicit none
      include 'common.inc'
      integer i,j
      integer istrt, iend, ifrag, isect,iscend
      integer natms
      integer ibeg, ifin, nfrg, ierr, icint
      integer fragid(maxfrag,maxapf), afrag(maxfrag)
      character*80 string
      character*20 temp   
      double precision cfloat

c  
c  read each line in FRAGID section until 'end' is found.  each line 
c  corresponds to a fragment (line 1 <=> frag 1) and the number on
c  line n correspond to the atoms in fragment n
c  also keep track of how many atoms are in each fragment, this is in
c  array afrag
       ifrag = 0
       j = 0
       call rline(1,string,istrt,isect,iscend,ierr)
       do while (iscend .ne. 1)
       i = 0
C  read in the atom numbers for each fragment
       ifrag = ifrag + 1
       ierr = 0
          do while (ierr .ne. 1)
	     j = j +1
             i = i + 1
             ibeg = istrt
             call rword(string,istrt,ierr)
             ifin = istrt - 1

c  save fragment id numbers
	      temp = string(ibeg:ifin)
	      read(unit=temp,fmt=*) fragid(ifrag,i)
	      afrag(ifrag) = i

          end do


         call rline(1,string,istrt,isect,iscend,iend)
      end do
C make sure that all atoms are accounted for the in fragment id
      if (j .ne. natms) then
         write(2,1300) j, natms
 1300    format(3x, 'ERROR: the total number of atoms listed in the ',
     *           'fragment ids ', i4, ' does not match the total ',
     *           'number of atoms listed in the input file ', i4)
         stop 'rfid 1'
      end if
      return

      end
c*******************************************************************************
C
C  RBGQ
C
c*******************************************************************************
      subroutine rbgq(bgq,string,istrt,iend,iatom)

C     Created: Sept 21, 2006 by E. E. Dahlke (based on rgeom)
C     Revised: June,15,2011 by Bo Wang
C     Purpose: to read in the charges for each different atom type
C
C     Called by: readin
C     Calls:

      implicit none
      include 'common.inc'
      integer i,j
      integer istrt, iend, iatom, isect,iscend
      integer ibeg, ifin, ierr, ifind, chindex
      character*80 string
cx     character*20 bgq(maxat,2)                                        !0602BW11
      character*20 bgq(maxat,5)                                         !0602BW11
c
c  read each line in BGCHARGE section until 'end' is found.  each line
c  each line corresponds to an atom type and the charge that atom should have
c  the total number of different charges is stored in iatom

       iatom = 0
       call rline(1,string,istrt,isect,iscend,ierr)

       do while (iscend .ne. 1)
       iatom = iatom + 1
       ierr = 0
          do i = -1, 3                                                  !0602BW11 read the information of the charge, the screening, and nuclear charges
             ibeg = istrt
             call rword(string,istrt,ierr)
             ifin = istrt - 1
C  if it's an atomic symbol make capital again
             if (i .eq. -1) then
                  chindex = ichar(string(ibeg:ibeg)) - 32
                  string(ibeg:ibeg) = char(chindex)
                  bgq(iatom,i+2) = string(ibeg:ifin)
C  otherwise just store
             else
                  bgq(iatom,i+2) = string(ibeg:ifin)
 	     end if
          end do


         call rline(1,string,istrt,isect,iscend,iend)
      end do


      return

      end
C***************************************************************************
C
C  RCM
C
c*******************************************************************************
      subroutine rcm(cm,string,istrt,iend,nfrg)

C     Created: Sept 21, 2006 by E. E. Dahlke (based on rgeom)
C     Revised:
C     Purpose: to read in the charge and multiplicity for each fragment
C
C     Called by: readin
C     Calls:

      implicit none
      include 'common.inc'
      integer i,j
      integer istrt, iend, natom, isect,iscend
      integer ibeg, ifin, ierr, ifrag, nfrg, icint
      integer cm(maxfrag,2)
      character*80 string
      character*20 rvar2, temp
      double precision cfloat

c
c  read each line in BGCHARGE section until 'end' is found.  each line
c  each line corresponds to an atom type and the charge that atom should have
c  the total number of different charges is stored in iatom

       ifrag = 0
       call rline(1,string,istrt,isect,iscend,ierr)
       do while (iscend .ne. 1)
       ifrag = ifrag + 1
       ierr = 0
          do i = -1, 0
c  save atom types and charges
             ibeg = istrt
	     call rword(string,istrt,ierr)
             ifin = istrt - 1
             temp = string(ibeg:ifin)
	     read(unit=temp,fmt=*) cm(ifrag,i+2)

          end do


         call rline(1,string,istrt,isect,iscend,iend)
      end do
      if (ifrag .ne. nfrg) then
         write(2,1000) ifrag, nfrg
 1000    format(3x,'The number of fragments that have a charge and ',
     *          'multiplicity listed : ', i4, ' does not match the ',
     *          'number of fragments: ', i4)
         stop 'rcm 1'
      end if

      return

      end

c*******************************************************************************
C
C  RBASIS
C
c*******************************************************************************
      subroutine rbasis(string,istrt,iend,basis,bkey,nbasis)
	
C     Created: Sept 22, 2006 by E. E. Dahlke
C     Revised:
C     Purpose:  to read in the basis set for an EEMB calculation
C
C     Called by: rjob
C     Calls:

      implicit none
      include 'common.inc'
      integer i,j
      integer natms, istrt, iend, ifrag, isect,iscend,ierr
      integer nbasis
      character*80 string, basis(1000)
      character*20 bkey

      i = 1
      if (bkey(1:3) .eq. 'lib' .or. bkey(1:3) .eq. 'gen') then
          do while (iscend .ne. 1)
             call rline2(1,string,istrt,isect,iscend,ierr)
             basis(i) = string(istrt:80)
             nbasis = i 
             i = i + 1
          end do 
      else 
          write(2,1100) bkey
          stop 'rbasis 2'
      end if     
      nbasis = nbasis - 1

C  make sure some basis was listed
      if (i .eq. 0) then
	write(2,1000) 
 1000   format(3x, 'ERROR:  you did not specify a basis set!!!')
        stop 'rbasis 1'
      end if

 1100 format(3x, 'ERROR: ', a8, ' is not a valid option at this time',
     *      '  please use the BASIS library or BASIS general option')


      return
      end

C**********************************************************************
C
C  RCORE
C
C**********************************************************************
      subroutine rcore(string,istrt,iend,core,ncore,pseudo)

C  Created:  	June 10, 2009  by EriN Dahlke Speetzen
C  Revised:
C
C  Purpose:  	To read in information about the core potentials for a 
C            	calculation
C
C  Called by:  	rjob
C  Calls:      

      implicit none
      include 'common.inc'
      integer i,j
      integer natms, istrt, iend, ifrag, isect, iscend, ierr
      integer ncore, pseudo
      character*80 string, core(1000)


      i = 1
      if (pseudo .eq. 1) then
          do while (iscend .ne. 1)
             call rline(1,string,istrt,isect,iscend,ierr)
             core(i) = string(istrt:80)
             ncore = i
             i = i + 1
           end do
      end if
      ncore = ncore - 1

C  make sure some core potential was listed
      if (i .eq. 0) then
          write(2,1000)
 1000     format(3x, 'ERROR:  you did not specify a core potential!!!')
          stop 'rcore 1'
       end if

 1100 format(3x, 'ERROR: ', a8, ' is not a valid option at this time'
     *       ' please use the CORE library or CORE general option')

      return
      end


C**********************************************************************
C     
C  RLINE2
C     
C**********************************************************************
      subroutine rline2(iunitr,string,istrt,isect,iscend,iend)
      
C  Created:     April 8, 1999 by P.L. Fast (RLINE)
C  Revised:     Sept 20, 2006, taken from Multilevel 4.2
C	        October 18, 2006 by E. E. Dahlke
C  Purpose      find the first non-comment, non-blank line of a file
C               and the location of the first character on that line.
C               The contents of the line are placed in the variable string
          
c  istrt:   location in 80 character string that is the first non-blank
c  isect:   a 1 is returned if the first non-blank character is a *
c           signalling the start of a section
c  iscend:  a 1 is returned if the line is 'end' signalling a section
c           end
c  iend:    a 1 is returned if the end of file 5 is encountered
c  string:  80 character string that is the contents of the current line
c       
      
C         
C  Description: Calls all of the subroutines needed to read in the
C               general section of the input file
C        
C       
C  Called by:   readin
C  Calls:    
C
C  Variables defined here:


      implicit none
      character*80 string
      integer istrt, isect, iscend, iend, iunitr, iblnk, icommt
      integer i, j

 1000 format(A80)

C  initialize the variables
      istrt = 1
      isect = 0
      iend = 0
      iblnk = 1
      iscend = 0
      icommt = 1

c
c     read in next line in iunitr - find first non-blank character  with
c     fchar and then check the line with chkln to see if it is a comment
c     line. Continue cycling through lines in infile until a non-blank,
c     non-comment line is found
c
      do while ((iblnk .eq. 1) .and. (icommt .eq. 1))
         istrt = 1
         read(1,1000,end=9999) string
         call fchar(string, istrt, iblnk)
         if (iblnk .ne. 1) then
            call chkln(string, istrt, isect, icommt, iscend)
         endif
      end do
c
c     strip off any trailing comments on the line
c
      do 1 j=istrt,80
         if (string(j:j).eq.'#') then
            string(j:80) = ' '
            return
         endif
 1    continue
c
      return


 9999 iend=1
      return
      end

C******************************************************************************
C
C     CHKCPOP
C
C******************************************************************************
C
C
C       Created: March 8, 2011 by H. R. Leverentz
C       Revised:
C       Purpose: subroutine to check to make sure that the requested method
C                of population/charge analysis is supported by this program
C
C       Description: this subroutine will stop the program with an error 
C                    message if an unsupported method of population/charge
C                    analysis has been selected; also, this subroutine will
C                    return an integer variable "ipop" that corresponds to
C                    the type of population/charge analysis to be performed
C
C         On input: cpop   = a character variable read from the eemb.inp file
C                            that indicates the type of population analysis
C                            to be performed (e.g. 'mk' or 'chelpg')
C
C         On output: ipop  = 0   =>  no population/charge analysis 
C                          = 1   =>  Merz-Singh-Kollman (MK) analysis
C                          = 2   =>  ChElPG analysis
C                    pfin  = the location of the last character in the cpop
C                            string
C
      subroutine chkcpop(cpop,ipop,pfin)  

      implicit none
      include 'common.inc'

      character*20 cpop
      integer ipop,pfin

      if (cpop(1:4) .eq. 'none') then
        ipop = 0
        pfin = 4
      elseif (cpop(1:2) .eq. 'mk') then
        ipop = 1
        pfin = 2
      elseif (cpop(1:6) .eq. 'chelpg') then
        ipop = 2
        pfin = 6
      elseif (cpop(1:8) .eq. 'mulliken') then
        ipop = 3
        pfin = 8
      else
        write(2,1000) cpop
        stop 'chkcpop 1'
      endif

      return

 1000 format(/3x,'Error: ',a,' is not a method of population or charge',
     *           ' analysis',/10x,'that is supported by this version',
     *           ' of MBPAC. Please select one of the following:',
     *       //12x,'mk, chelpg',///)

      end subroutine chkcpop


C******************************************************************************
C
C     CHKCOMPAT
C
C******************************************************************************
C
C
C       Created: Nov. 15, 2011 by H. R. Leverentz
C       Revised:
C       Purpose: subroutine to check to make sure that the type of calculation
C                and the keywords specified by the user are compatible with
C                one another and to generate an error message and stop 
C                the program if they are not.  This subroutine should be 
C                checked and revised whenever MBPAC is revised.
C
C       Description: this subroutine will stop the program with an error 
C                    message if incompatible methods and keywords have been
C                    selected.
C
      subroutine chkcompat(optkey,hesskey,gradkey,method,lscreen,
     *     leembce,cpop,esm,pseudo) !0627HQ12 added esm and pseudo    

      implicit none
      include 'common.inc'

      integer ncemeth
      parameter (ncemeth = 8)           

      integer i,j,k,l,m,n
      integer optkey,hesskey,gradkey
      integer lscreen,leembce
      character*20 method,cemethod(ncemeth),cpop

      integer pseudo                                 !0627HQ12
      character*20 esm                               !0627HQ12

C Screened charge calculations cannot be run with anything other than
C  single-point energy calculations.

      if (lscreen.eq.1) then
        if ((optkey.eq.1).or.(hesskey.eq.1).or.(gradkey.eq.1)) then
          write(2,1010) 
          stop 'chkcompat 1'
        endif
      endif

C EE-MB-CE calculations cannot be run with anything other than single-point
C  energy calculations.  Also, EE-MB-CE calculations must be used with
C  a post-hartree-fock method of wave function theory (WFT) such as MP2 or
C  CCSD(T).  EE-MB-CE calculations cannot be run with population analyses.
     
      cemethod(1) = 'mp2     '
      cemethod(2) = 'mp3     '
      cemethod(3) = 'mp4     '
      cemethod(4) = 'mp4d    '
      cemethod(5) = 'mp4dq   '
      cemethod(6) = 'mp4sdq  '
      cemethod(7) = 'qcisd(t)'
      cemethod(8) = 'ccsd(t) '

      if (leembce.eq.1) then
        if ((optkey.eq.1).or.(hesskey.eq.1).or.(gradkey.eq.1)) then
          write(2,2010)
          stop 'chkcompat 2'
        endif 
        n = 0
        do i=1,ncemeth
          if (method(1:8).ne.cemethod(i)) then
            n = n+1
          endif
        end do
        if (n.eq.ncemeth) then
          write(2,2020) method
          stop 'chkcompat 3'
        endif
        if (cpop(1:4).ne.'none') then
          write(2,2030) cpop  
          stop 'chkcompat 4'
        endif
      endif

 1010 format(/3x,'ERROR: screened charge calculations can only be run',/,
     *       '          with single-point energy calculations, but',/,
     *       '          a gradient, hessian, or geometry optimization',/,
     *       '          has been selected.  TERMINATING PROGRAM.')

 2010 format(/3x,'ERROR: EE-MB-CE calculations can only be run',/,
     *       '          with single-point energy calculations, but',/,
     *       '          a gradient, hessian, or geometry optimization',/,
     *       '          has been selected.  TERMINATING PROGRAM.')

 2020 format(/3x,'ERROR: You have chosen to use the EE-MB-CE ',/,
     *       '          approximation with the ',a20,'method, but',/,
     *       '          only the following methods may be used with',/,
     *       '          the EE-MB-CE approximation in this version',/,
     *       '          of MBPAC: mp2, mp3, mp4, mp4d, mp4dq, mp4sdq,',/,
     *       '          qcisd(t), and ccsd(t).  TERMINATING PROGRAM.')

 2030 format(/3x,'ERROR: EE-MB-CE calculations cannot be run with',/,
     *       '          any method of population analysis, such as',/,
     *       '          ',a20,'.  TERMINATING PROGRAM.')

C Only single-point energies can be run using Molpro (i.e., no gradients,
C  Hessians, or geometry optimizations can currently be run with Molpro).
C  Also, screened charges, pseudopotentials, and the 'population' 
C  keyword cannot be used with Molpro in the current version.

      if (esm.eq.'molpro') then                                     !0627HQ12
        if ((optkey.eq.1).or.(hesskey.eq.1).or.(gradkey.eq.1)) then !0627HQ12
          write(2,1110)                                             !0627HQ12
          stop 'chkcompat 5'                                        !0627HQ12
        elseif (lscreen.eq.1) then                                  !0627HQ12
          write(2,1120)                                             !0627HQ12
          stop 'chkcompat 6'                                        !0627HQ12
        elseif (cpop(1:4).ne.'none') then                           !0627HQ12
          write(2,1130)                                             !0627HQ12
          stop 'chkcompat 7'                                        !0627HQ12
        elseif (pseudo.eq.1) then                                   !0627HQ12
          write(2,1140)                                             !0627HQ12
          stop 'chkcompat 8'                                        !0627HQ12
        endif                                                       !0627HQ12
      endif                                                         !0627HQ12

 1110 format('ERROR: only single point energies may be used ',      !0627HQ12
     *       'with Molpro')                                         !0627HQ12
 1120 format('ERROR: screened charge calculations cannot be ',      !0627HQ12
     *       'run with Molpro')                                     !0627HQ12
 1130 format('ERROR: population analyse cannot be run with Molpro') !0627HQ12
 1140 format('ERROR: the core keyword cannot be used with Molpro')  !0627HQ12


      return

      end subroutine chkcompat



