#!/usr/bin/perl 

#  this script is used to drive the mbpac program
#  Usage:
#	eemb.pl input_file.inp number_of_processors

use Cwd;

##################################
# The scratch directory name in the SHUTTLE scripts 
# 0627HQ12

$scratchname = "/scratch/uvdev1/hannahl/mbpac/testruns";

#
#################################



if ($#ARGV !=  1) {
   die "You must specify a file and the number of processors to run this calculation!\n";
}

$name = $ARGV[0];
$nproc = $ARGV[1];

#  set name of output file

@temp = split(/\./, $name);
pop @temp;
$name1 = $temp[0];  #0627HQ12
push @temp, "out";
$out = join(".", @temp);

#  set the paths for all important directories
$wrkdir = cwd;
$mbdir = `cat ~/.mbpac_path`;
chomp ($mbdir);
$scriptdir = "$mbdir" . "/script";
$exedir = "$mbdir" . "/exe";

if ($nproc == 1) {
  $exe = "$exedir" . "/mbpac.exe";
} else {
  $exe = "mpirun -np $nproc $exedir\/pmbpac.exe";

#  BLADE if submitted to the queque
#   $exe = "mpirun -np $nproc -hostfile $ENV{PBS_NODEFILE} $exedir\/pmbpac.exe";

#  BLADE if run interactively - example for 4 processors - you must fill in node names
#   $exe = "mpirun -np $nproc blade285 blade286 blade287 blade288 $exedir\/pmbpac.exe";

}

#  Create a subdirectory in the scratch space
#  0627HQ12
system("mkdir $scratchname/$name1");

#  copy the shuttle scripts to the working directory and fix their scratch lines
#  0627HQ12
&fix_scratch("g09shuttle.pl");
&fix_scratch("ex_shuttle");
&fix_scratch("g09-ex-shuttle.pl");
&fix_scratch("Gau_External_2");
&fix_scratch("molpshuttle.pl");

#  copy input file to correct name, run the calculation, move the 
#  output file to the correct name at the end.

system("cp $name eemb.inp");
system("$exe");
system("cp eemb.out $out");

#  delete eemb.inp, eemb.out and time

system("rm -f eemb.inp");
system("rm -f eemb.out");
system("rm -f time");
system("rm -f g09shuttle.pl");
system("rm -f ex_shuttle");
system("rm -f g09-ex-shuttle.pl");
system("rm -f Gau_External_2");
system("rm -f Test.FChk");
system("rm -f molpshuttle.pl");      #0627HQ12


#Subroutine to fix the shuttle script to have a nice subdirectory in the scratch space
#0627HQ12
sub fix_scratch {
    open INP, "<$scriptdir\/@_[0]" or die "CANNOT OPEN SHUTTLE SCRIPT";
    open OUT, ">$wrkdir\/@_[0]" or die "CANNOT OPEN OUTPUT SHUTTLE SCRIPT";
    while (<INP>) {
	if ($_ =~ /$scratchname/) {
	    chomp;
	    chop; #remove ; in shuttle
	    chop; #remove " in shuttle
	    print OUT "$_\/$name1\";";
	} else {
	    print OUT "$_";
	}
    }
}
